@@ -5,7 +5,7 @@
 // Implementation of Drawable (Graphic objects)
 //
 
-#define MAGICK_IMPLEMENTATION
+#define MAGICKCORE_IMPLEMENTATION  1
 #define MAGICK_DRAWABLE_IMPLEMENTATION
 
 #include "Magick++/Include.h"
@@ -85,8 +85,9 @@ Magick::Drawable& Magick::Drawable::operator= (const Magick::Drawable& original_
 {
   if (this != &original_)
     {
+      DrawableBase* temp_dp = (original_.dp ? original_.dp->copy() : 0);
       delete dp;
-      dp = (original_.dp ? original_.dp->copy() : 0);
+      dp = temp_dp;
     }
   return *this;
 }
@@ -164,8 +165,9 @@ Magick::VPath& Magick::VPath::operator= (const Magick::VPath& original_ )
 {
   if (this != &original_)
     {
+      VPathBase* temp_dp = (original_.dp ? original_.dp->copy() : 0);
       delete dp;
-      dp = (original_.dp ? original_.dp->copy() : 0);
+      dp = temp_dp;
     }
   return *this;
 }
@@ -263,7 +265,8 @@ Magick::DrawableBezier::DrawableBezier ( const CoordinateList &coordinates_ )
 }
 // Copy constructor
 Magick::DrawableBezier::DrawableBezier( const Magick::DrawableBezier& original_ )
-  : _coordinates(original_._coordinates)
+  : DrawableBase (original_),
+    _coordinates(original_._coordinates)
 {
 }
 // Destructor
@@ -319,7 +322,8 @@ Magick::DrawablePushClipPath::DrawablePushClipPath( const std::string &id_)
 }
 Magick::DrawablePushClipPath::DrawablePushClipPath
 ( const Magick::DrawablePushClipPath& original_ ) //multithread safe const char*
-  : _id(original_._id.c_str())
+  : DrawableBase (original_),
+    _id(original_._id.c_str())
 {
 }
 Magick::DrawablePushClipPath::~DrawablePushClipPath( void )
@@ -344,7 +348,8 @@ Magick::DrawableClipPath::DrawableClipPath( const std::string &id_ )
 }
 
 Magick::DrawableClipPath::DrawableClipPath ( const Magick::DrawableClipPath& original_ )
-  : _id(original_._id.c_str())
+  : DrawableBase (original_),
+    _id(original_._id.c_str())
 {
 }
 Magick::DrawableClipPath::~DrawableClipPath( void )
@@ -352,7 +357,7 @@ Magick::DrawableClipPath::~DrawableClipPath( void )
 }
 void Magick::DrawableClipPath::operator()( MagickLib::DrawContext context_ ) const
 {
-	DrawSetClipPath( context_, _id.c_str());
+	(void) DrawSetClipPath( context_, _id.c_str());
 }
 Magick::DrawableBase* Magick::DrawableClipPath::copy() const
 {
@@ -490,15 +495,17 @@ Magick::DrawableCompositeImage& Magick::DrawableCompositeImage::operator=
       _y = original_._y;
       _width = original_._width;
       _height = original_._height;
+      Image* temp_image = new Image(*original_._image);
       delete _image;
-      _image = new Image(*original_._image);
+      _image = temp_image;
     }
   return *this;
 }
 void Magick::DrawableCompositeImage::filename( const std::string &filename_ )
 {
+  Image* temp_image = new Image(filename_);
   delete _image;
-  _image = new Image(filename_);
+  _image = temp_image;
 }
 std::string Magick::DrawableCompositeImage::filename( void ) const
 {
@@ -507,8 +514,9 @@ std::string Magick::DrawableCompositeImage::filename( void ) const
 
 void Magick::DrawableCompositeImage::image( const Magick::Image &image_ )
 {
+  Image* temp_image = new Image(image_);
   delete _image;
-  _image = new Image(image_);
+  _image = temp_image;
 }
 Magick::Image Magick::DrawableCompositeImage::image( void ) const
 {
@@ -528,8 +536,13 @@ std::string Magick::DrawableCompositeImage::magick( void )
 void Magick::DrawableCompositeImage::operator()
   ( MagickLib::DrawContext context_ ) const
 {
-  DrawComposite( context_, _composition, _x, _y, _width, _height,
-                 _image->constImage() );
+  MagickWand
+    *magick_wand;
+
+  magick_wand=NewMagickWandFromImage(_image->constImage());
+  (void) DrawComposite( context_, _composition, _x, _y, _width, _height,
+     magick_wand );
+  magick_wand=DestroyMagickWand(magick_wand);
 }
 
 Magick::DrawableBase* Magick::DrawableCompositeImage::copy() const
@@ -559,7 +572,8 @@ Magick::DrawableFillColor::DrawableFillColor( const Magick::Color &color_ )
 }
 Magick::DrawableFillColor::DrawableFillColor
 ( const Magick::DrawableFillColor& original_ )
-  : _color(original_._color)
+  : DrawableBase (original_),
+    _color(original_._color)
 {
 }
 Magick::DrawableFillColor::~DrawableFillColor( void )
@@ -572,7 +586,7 @@ void Magick::DrawableFillColor::operator()
   PixelWand *pixel_wand=NewPixelWand();
   PixelSetQuantumColor(pixel_wand,&color);
   DrawSetFillColor(context_,pixel_wand);
-  DestroyPixelWand(pixel_wand);
+  pixel_wand=DestroyPixelWand(pixel_wand);
 }
 Magick::DrawableBase* Magick::DrawableFillColor::copy() const
 {
@@ -600,7 +614,7 @@ Magick::DrawableFillOpacity::~DrawableFillOpacity ( void )
 void Magick::DrawableFillOpacity::operator()
   ( MagickLib::DrawContext context_ ) const
 {
-  DrawSetFillOpacity( context_, _opacity );
+  DrawSetFillAlpha( context_, _opacity );
 }
 Magick::DrawableBase* Magick::DrawableFillOpacity::copy() const
 {
@@ -628,7 +642,8 @@ Magick::DrawableFont::DrawableFont ( const std::string &family_,
 {
 }
 Magick::DrawableFont::DrawableFont ( const Magick::DrawableFont& original_ )
-  : _font(original_._font),
+  : DrawableBase (original_),
+    _font(original_._font),
     _family(original_._family),
     _style(original_._style),
     _weight(original_._weight),
@@ -643,13 +658,13 @@ void Magick::DrawableFont::operator()( MagickLib::DrawContext context_ ) const
   // font
   if(_font.length())
     {
-      DrawSetFont( context_, _font.c_str() );
+      (void) DrawSetFont( context_, _font.c_str() );
     }
 
   if(_family.length())
     {
       // font-family
-      DrawSetFontFamily( context_, _family.c_str() );
+      (void) DrawSetFontFamily( context_, _family.c_str() );
 
       // font-style
       DrawSetFontStyle( context_, _style );
@@ -712,7 +727,8 @@ Magick::DrawablePath::DrawablePath ( const VPathList &path_ )
 {
 }
 Magick::DrawablePath::DrawablePath ( const Magick::DrawablePath& original_ )
-  : _path(original_._path)
+  : DrawableBase (original_),
+    _path(original_._path)
 {
 }
 Magick::DrawablePath::~DrawablePath ( void )
@@ -767,7 +783,8 @@ Magick::DrawablePolygon::DrawablePolygon ( const CoordinateList &coordinates_ )
 }
 Magick::DrawablePolygon::DrawablePolygon
 ( const Magick::DrawablePolygon& original_ )
-  : _coordinates(original_._coordinates)
+  : DrawableBase (original_),
+    _coordinates(original_._coordinates)
 {
 }
 Magick::DrawablePolygon::~DrawablePolygon ( void )
@@ -806,7 +823,8 @@ Magick::DrawablePolyline::DrawablePolyline
 }
 Magick::DrawablePolyline::DrawablePolyline
 ( const Magick::DrawablePolyline& original_ )
-  : _coordinates(original_._coordinates)
+  : DrawableBase (original_),
+    _coordinates(original_._coordinates)
 {
 }
 Magick::DrawablePolyline::~DrawablePolyline ( void )
@@ -872,7 +890,7 @@ Magick::DrawablePopPattern::~DrawablePopPattern ( void )
 void Magick::DrawablePopPattern::operator()
   ( MagickLib::DrawContext context_ ) const
 {
-  DrawPopPattern( context_ );
+  (void) DrawPopPattern( context_ );
 }
 Magick::DrawableBase* Magick::DrawablePopPattern::copy() const
 {
@@ -892,7 +910,8 @@ Magick::DrawablePushPattern::DrawablePushPattern
 }
 Magick::DrawablePushPattern::DrawablePushPattern
 ( const Magick::DrawablePushPattern& original_ )
-  : _id(original_._id),
+  : DrawableBase (original_),
+    _id(original_._id),
     _x(original_._x),
     _y(original_._y),
     _width(original_._width),
@@ -905,7 +924,7 @@ Magick::DrawablePushPattern::~DrawablePushPattern ( void )
 void Magick::DrawablePushPattern::operator()
   ( MagickLib::DrawContext context_ ) const
 {
-  DrawPushPattern( context_, _id.c_str(), _x, _y, _width, _height );
+  (void) DrawPushPattern( context_, _id.c_str(), _x, _y, _width, _height );
 }
 Magick::DrawableBase* Magick::DrawablePushPattern::copy() const
 {
@@ -1014,7 +1033,8 @@ Magick::DrawableDashArray::DrawableDashArray( const unsigned int* dasharray_ )
 }
 Magick::DrawableDashArray::DrawableDashArray
 (const Magick::DrawableDashArray& original_)
-  : _size(0),
+  : DrawableBase (original_),
+    _size(0),
     _dasharray(0)
 {
   dasharray( original_._dasharray );
@@ -1037,7 +1057,7 @@ Magick::DrawableDashArray& Magick::DrawableDashArray::operator=
 void Magick::DrawableDashArray::operator()
   ( MagickLib::DrawContext context_ ) const
 {
-  DrawSetStrokeDashArray( context_, _size, _dasharray );
+  (void) DrawSetStrokeDashArray( context_, _size, _dasharray );
 }
 Magick::DrawableBase* Magick::DrawableDashArray::copy() const
 {
@@ -1179,7 +1199,8 @@ Magick::DrawableStrokeColor::DrawableStrokeColor
 }
 Magick::DrawableStrokeColor::DrawableStrokeColor
 ( const Magick::DrawableStrokeColor& original_ )
-  : _color(original_._color)
+  : DrawableBase (original_),
+    _color(original_._color)
 {
 }
 Magick::DrawableStrokeColor::~DrawableStrokeColor ( void )
@@ -1192,7 +1213,7 @@ void Magick::DrawableStrokeColor::operator()
   PixelWand *pixel_wand=NewPixelWand();
   PixelSetQuantumColor(pixel_wand,&color);
   DrawSetStrokeColor(context_,pixel_wand);
-  DestroyPixelWand(pixel_wand);
+  pixel_wand=DestroyPixelWand(pixel_wand);
 }
 Magick::DrawableBase* Magick::DrawableStrokeColor::copy() const
 {
@@ -1206,7 +1227,7 @@ Magick::DrawableStrokeOpacity::~DrawableStrokeOpacity ( void )
 void Magick::DrawableStrokeOpacity::operator()
   ( MagickLib::DrawContext context_ ) const
 {
-  DrawSetStrokeOpacity( context_, _opacity );
+  DrawSetStrokeAlpha( context_, _opacity );
 }
 Magick::DrawableBase* Magick::DrawableStrokeOpacity::copy() const
 {
@@ -1245,7 +1266,8 @@ Magick::DrawableText::DrawableText ( const double x_, const double y_,
 {
 }
 Magick::DrawableText::DrawableText( const Magick::DrawableText& original_ )
-  : _x(original_._x),
+  : DrawableBase (original_),
+    _x(original_._x),
     _y(original_._y),
     _text(original_._text),
     _encoding(original_._encoding)
@@ -1272,7 +1294,8 @@ Magick::DrawableTextAntialias::DrawableTextAntialias ( bool flag_ )
 {
 }
 Magick::DrawableTextAntialias::DrawableTextAntialias( const Magick::DrawableTextAntialias &original_ )
-  : _flag(original_._flag)
+  : DrawableBase (original_),
+    _flag(original_._flag)
 {
 }
 Magick::DrawableTextAntialias::~DrawableTextAntialias ( void )
@@ -1297,7 +1320,8 @@ Magick::DrawableTextDecoration::DrawableTextDecoration
 }
 Magick::DrawableTextDecoration::DrawableTextDecoration
   ( const Magick::DrawableTextDecoration &original_ )
-    : _decoration(original_._decoration)
+    : DrawableBase (original_),
+      _decoration(original_._decoration)
 {
 }
 Magick::DrawableTextDecoration::~DrawableTextDecoration( void )
@@ -1321,7 +1345,8 @@ Magick::DrawableTextUnderColor::DrawableTextUnderColor
 }
 Magick::DrawableTextUnderColor::DrawableTextUnderColor
 ( const Magick::DrawableTextUnderColor& original_ )
-  : _color(original_._color)
+  : DrawableBase (original_),
+    _color(original_._color)
 {
 }
 Magick::DrawableTextUnderColor::~DrawableTextUnderColor ( void )
@@ -1334,7 +1359,7 @@ void Magick::DrawableTextUnderColor::operator()
   PixelWand *pixel_wand=NewPixelWand();
   PixelSetQuantumColor(pixel_wand,&color);
   DrawSetTextUnderColor(context_,pixel_wand);
-  DestroyPixelWand(pixel_wand);
+  pixel_wand=DestroyPixelWand(pixel_wand);
 }
 Magick::DrawableBase* Magick::DrawableTextUnderColor::copy() const
 {
@@ -1456,7 +1481,8 @@ Magick::PathArcAbs::PathArcAbs ( const PathArcArgsList &coordinates_ )
 {
 }
 Magick::PathArcAbs::PathArcAbs ( const Magick::PathArcAbs& original_ )
-  : _coordinates(original_._coordinates)
+  : VPathBase (original_),
+    _coordinates(original_._coordinates)
 {
 }
 Magick::PathArcAbs::~PathArcAbs ( void )
@@ -1486,7 +1512,8 @@ Magick::PathArcRel::PathArcRel ( const PathArcArgsList &coordinates_ )
 {
 }
 Magick::PathArcRel::PathArcRel ( const Magick::PathArcRel& original_ )
-  : _coordinates(original_._coordinates)
+  : VPathBase (original_),
+    _coordinates(original_._coordinates)
 {
 }
 Magick::PathArcRel::~PathArcRel ( void )
@@ -1601,8 +1628,9 @@ Magick::PathCurvetoAbs::PathCurvetoAbs ( const PathCurveToArgsList &args_ )
 {
 }
 Magick::PathCurvetoAbs::PathCurvetoAbs
-( const Magick::PathCurvetoAbs& original_ )
- : _args(original_._args)
+ ( const Magick::PathCurvetoAbs& original_ )
+   : VPathBase (original_),
+     _args(original_._args)
 {
 }
 Magick::PathCurvetoAbs::~PathCurvetoAbs ( void )
@@ -1632,7 +1660,8 @@ Magick::PathCurvetoRel::PathCurvetoRel ( const PathCurveToArgsList &args_ )
 }
 Magick::PathCurvetoRel::PathCurvetoRel
 ( const Magick::PathCurvetoRel& original_ )
-  : _args(original_._args)
+  : VPathBase (original_),
+    _args(original_._args)
 {
 }
 Magick::PathCurvetoRel::~PathCurvetoRel ( void )
@@ -1664,7 +1693,8 @@ Magick::PathSmoothCurvetoAbs::PathSmoothCurvetoAbs
 }
 Magick::PathSmoothCurvetoAbs::PathSmoothCurvetoAbs
 ( const Magick::PathSmoothCurvetoAbs& original_ )
-  : _coordinates(original_._coordinates)
+  : VPathBase (original_),
+    _coordinates(original_._coordinates)
 {
 }
 Magick::PathSmoothCurvetoAbs::~PathSmoothCurvetoAbs ( void )
@@ -1699,7 +1729,8 @@ Magick::PathSmoothCurvetoRel::PathSmoothCurvetoRel
 }
 Magick::PathSmoothCurvetoRel::PathSmoothCurvetoRel
 ( const Magick::PathSmoothCurvetoRel& original_ )
-  : _coordinates(original_._coordinates)
+  : VPathBase (original_),
+    _coordinates(original_._coordinates)
 {
 }
 Magick::PathSmoothCurvetoRel::~PathSmoothCurvetoRel ( void )
@@ -1806,7 +1837,8 @@ Magick::PathQuadraticCurvetoAbs::PathQuadraticCurvetoAbs
 }
 Magick::PathQuadraticCurvetoAbs::PathQuadraticCurvetoAbs
 ( const Magick::PathQuadraticCurvetoAbs& original_ )
-  : _args(original_._args)
+  : VPathBase (original_),
+    _args(original_._args)
 {
 }
 Magick::PathQuadraticCurvetoAbs::~PathQuadraticCurvetoAbs ( void )
@@ -1838,7 +1870,8 @@ Magick::PathQuadraticCurvetoRel::PathQuadraticCurvetoRel
 }
 Magick::PathQuadraticCurvetoRel::PathQuadraticCurvetoRel
 ( const Magick::PathQuadraticCurvetoRel& original_ )
-  : _args(original_._args)
+  : VPathBase (original_),
+    _args(original_._args)
 {
 }
 Magick::PathQuadraticCurvetoRel::~PathQuadraticCurvetoRel ( void )
@@ -1870,7 +1903,8 @@ Magick::PathSmoothQuadraticCurvetoAbs::PathSmoothQuadraticCurvetoAbs
 }
 Magick::PathSmoothQuadraticCurvetoAbs::PathSmoothQuadraticCurvetoAbs
 ( const Magick::PathSmoothQuadraticCurvetoAbs& original_ )
-  : _coordinates(original_._coordinates)
+  : VPathBase (original_),
+    _coordinates(original_._coordinates)
 {
 }
 Magick::PathSmoothQuadraticCurvetoAbs::~PathSmoothQuadraticCurvetoAbs ( void )
@@ -1901,7 +1935,8 @@ Magick::PathSmoothQuadraticCurvetoRel::PathSmoothQuadraticCurvetoRel
 }
 Magick::PathSmoothQuadraticCurvetoRel::PathSmoothQuadraticCurvetoRel
 ( const PathSmoothQuadraticCurvetoRel& original_ )
-  : _coordinates(original_._coordinates)
+  : VPathBase (original_),
+    _coordinates(original_._coordinates)
 {
 }
 Magick::PathSmoothQuadraticCurvetoRel::~PathSmoothQuadraticCurvetoRel ( void )
@@ -1933,7 +1968,8 @@ Magick::PathLinetoAbs::PathLinetoAbs ( const CoordinateList &coordinates_ )
 {
 }
 Magick::PathLinetoAbs::PathLinetoAbs ( const Magick::PathLinetoAbs& original_ )
-  : _coordinates(original_._coordinates)
+  : VPathBase (original_),
+    _coordinates(original_._coordinates)
 {
 }
 Magick::PathLinetoAbs::~PathLinetoAbs ( void )
@@ -1960,7 +1996,8 @@ Magick::PathLinetoRel::PathLinetoRel ( const CoordinateList &coordinates_ )
 {
 }
 Magick::PathLinetoRel::PathLinetoRel ( const Magick::PathLinetoRel& original_ )
-  : _coordinates(original_._coordinates)
+  : VPathBase (original_),
+    _coordinates(original_._coordinates)
 {
 }
 Magick::PathLinetoRel::~PathLinetoRel ( void )
@@ -2049,7 +2086,8 @@ Magick::PathMovetoAbs::PathMovetoAbs ( const CoordinateList &coordinates_ )
 {
 }
 Magick::PathMovetoAbs::PathMovetoAbs ( const Magick::PathMovetoAbs& original_ )
-  : _coordinates(original_._coordinates)
+  : VPathBase (original_),
+    _coordinates(original_._coordinates)
 {
 }
 Magick::PathMovetoAbs::~PathMovetoAbs ( void )
@@ -2076,7 +2114,8 @@ Magick::PathMovetoRel::PathMovetoRel ( const CoordinateList &coordinates_ )
 {
 }
 Magick::PathMovetoRel::PathMovetoRel ( const Magick::PathMovetoRel& original_ )
-  : _coordinates(original_._coordinates)
+  : VPathBase (original_),
+    _coordinates(original_._coordinates)
 {
 }
 Magick::PathMovetoRel::~PathMovetoRel ( void )
