@@ -555,6 +555,9 @@ namespace Magick
     void            shear ( const double xShearAngle_,
           const double yShearAngle_ );
     
+    // adjust the image contrast with a non-linear sigmoidal contrast algorithm
+    void            sigmoidalContrast ( const unsigned int sharpen_, const double contrast, const double midpoint = QuantumRange / 2.0 );
+
     // Solarize image (similar to effect seen when exposing a
     // photographic film to light during the development process)
     void            solarize ( const double factor_ = 50.0 );
@@ -809,6 +812,10 @@ namespace Magick
     void            endian ( const EndianType endian_ );
     EndianType      endian ( void ) const;
 
+    // Exif profile (BLOB)
+    void exifProfile( const Blob& exifProfile_ );
+    Blob exifProfile( void ) const; 
+
     // Image file name
     void            fileName ( const std::string &fileName_ );
     std::string     fileName ( void ) const;
@@ -944,6 +951,10 @@ typedef struct _ImageStatistics
     // color reduced.
     double          normalizedMeanError ( void ) const;
 
+    // Image orientation
+    void            orientation ( const OrientationType orientation_ );
+    OrientationType orientation ( void ) const;
+
     // Preferred size and location of an image canvas.
     void            page ( const Geometry &pageSize_ );
     Geometry        page ( void ) const;
