@@ -44,7 +44,7 @@
 namespace MagickLib
 {
 #include <magick/MagickCore.h>
-#include <wand/magick-wand.h>
+#include <wand/MagickWand.h>
 #undef inline // Remove possible definition from config.h
 
 #undef class
@@ -128,6 +128,7 @@ namespace Magick
   using MagickLib::OpacityChannel;
   using MagickLib::BlackChannel;
   using MagickLib::MatteChannel;
+  using MagickLib::DefaultChannels;
   
   // Color-space types
   using MagickLib::CMYKColorspace;
@@ -153,19 +154,26 @@ namespace Magick
   // Composition operations
   using MagickLib::AddCompositeOp;
   using MagickLib::AtopCompositeOp;
+  using MagickLib::BlendCompositeOp;
   using MagickLib::BumpmapCompositeOp;
   using MagickLib::ClearCompositeOp;
   using MagickLib::ColorizeCompositeOp;
   using MagickLib::CompositeOperator;
   using MagickLib::CopyBlueCompositeOp;
   using MagickLib::CopyCompositeOp;
+  using MagickLib::CopyCyanCompositeOp;
   using MagickLib::CopyGreenCompositeOp;
+  using MagickLib::CopyMagentaCompositeOp;
   using MagickLib::CopyOpacityCompositeOp;
   using MagickLib::CopyRedCompositeOp;
+  using MagickLib::CopyYellowCompositeOp;
   using MagickLib::DarkenCompositeOp;
   using MagickLib::DifferenceCompositeOp;
   using MagickLib::DisplaceCompositeOp;
   using MagickLib::DissolveCompositeOp;
+  using MagickLib::DstOverCompositeOp;
+  using MagickLib::ExclusionCompositeOp;
+  using MagickLib::HardLightCompositeOp;
   using MagickLib::HueCompositeOp;
   using MagickLib::InCompositeOp;
   using MagickLib::LightenCompositeOp;
@@ -180,13 +188,11 @@ namespace Magick
   using MagickLib::PlusCompositeOp;
   using MagickLib::SaturateCompositeOp;
   using MagickLib::ScreenCompositeOp;
+  using MagickLib::SoftLightCompositeOp;
   using MagickLib::SubtractCompositeOp;
   using MagickLib::ThresholdCompositeOp;
   using MagickLib::UndefinedCompositeOp;
   using MagickLib::XorCompositeOp;
-  using MagickLib::CopyCyanCompositeOp;
-  using MagickLib::CopyMagentaCompositeOp;
-  using MagickLib::CopyYellowCompositeOp;
   using MagickLib::CopyBlackCompositeOp;
   
   // Compression algorithms
@@ -487,6 +493,7 @@ namespace Magick
   using MagickLib::ConstituteImage;
   using MagickLib::ContrastImage;
   using MagickLib::ConvolveImage;
+  using MagickLib::CopyMagickString;
   using MagickLib::CorruptImageError;
   using MagickLib::CorruptImageFatalError;
   using MagickLib::CorruptImageWarning;
@@ -645,6 +652,7 @@ namespace Magick
   using MagickLib::GetPageGeometry;
   using MagickLib::GetPixels;
   using MagickLib::GetQuantizeInfo;
+  using MagickLib::GetQuantumInfo;
   using MagickLib::GetTypeMetrics;
   using MagickLib::GlobExpression;
   using MagickLib::GreaterValue;
@@ -712,6 +720,7 @@ namespace Magick
   using MagickLib::ProfileInfo;
   using MagickLib::QuantizeImage;
   using MagickLib::QuantizeInfo;
+  using MagickLib::QuantumInfo;
   using MagickLib::QueryColorDatabase;
   using MagickLib::RGBTransformImage;
   using MagickLib::RaiseImage;
@@ -757,6 +766,7 @@ namespace Magick
   using MagickLib::SharpenImage;
   using MagickLib::ShaveImage;
   using MagickLib::ShearImage;
+  using MagickLib::SigmoidalContrastImageChannel;
   using MagickLib::SignatureImage;
   using MagickLib::SizeBlob;
   using MagickLib::SolarizeImage;
