@@ -1,6 +1,6 @@
 // This may look like C code, but it is really -*- C++ -*-
 //
-// Copyright Bob Friesenhahn, 1999, 2000, 2001, 2002
+// Copyright Bob Friesenhahn, 1999, 2000, 2001, 2002, 2003
 //
 // Definition of Options
 //
@@ -14,8 +14,8 @@
 #if !defined(Magick_Options_header)
 #define Magick_Options_header
 
-#include <string>
 #include "Magick++/Include.h"
+#include <string>
 #include "Magick++/Color.h"
 #include "Magick++/Geometry.h"
 #include "Magick++/Drawable.h"
@@ -153,6 +153,10 @@ namespace Magick
     void            resolutionUnits ( ResolutionType resolutionUnits_ );
     ResolutionType  resolutionUnits ( void ) const;
 
+    // Image sampling factor
+    void            samplingFactor ( const std::string &samplingFactor_ );
+    std::string     samplingFactor ( void ) const;
+
     // Image size (required for raw formats)
     void            size ( const Geometry &geometry_ );
     Geometry        size ( void ) const;
