@@ -149,6 +149,18 @@ namespace Magick
     Geometry _geometry;
   };
 
+  // Extract channel from image
+  class MagickDLLDecl channelImage : public std::unary_function<Image&,void>
+  {
+  public:
+    channelImage( const ChannelType channel_ );
+
+    void operator()( Image &image_ ) const;
+
+  private:
+    ChannelType _channel;
+  };
+
   // Charcoal effect image (looks like charcoal sketch)
   class MagickDLLDecl charcoalImage : public std::unary_function<Image&,void>
   {
@@ -547,16 +559,39 @@ namespace Magick
     std::string _label;
   };
 
-  // Extract channel from image
-  class MagickDLLDecl channelImage : public std::unary_function<Image&,void>
+
+  // Level image
+  class MagickDLLDecl levelImage : public std::unary_function<Image&,void>
   {
   public:
-    channelImage( const ChannelType channel_ );
+    levelImage( const double black_point,
+                const double white_point,
+                const double mid_point=1.0 );
 
     void operator()( Image &image_ ) const;
 
   private:
-    ChannelType _channel;
+    double _black_point;
+    double _white_point;
+    double _mid_point;
+  };
+
+  // Level image channel
+  class MagickDLLDecl levelChannelImage : public std::unary_function<Image&,void>
+  {
+  public:
+    levelChannelImage( const Magick::ChannelType channel,
+                       const double black_point,
+                       const double white_point,
+                       const double mid_point=1.0 );
+
+    void operator()( Image &image_ ) const;
+
+  private:
+    Magick::ChannelType _channel;
+    double _black_point;
+    double _white_point;
+    double _mid_point;
   };
 
   // Magnify image by integral size
@@ -684,7 +719,7 @@ namespace Magick
   // are opaque then they are set to the specified opacity value,
   // otherwise they are blended with the supplied opacity value.  The
   // value of opacity_ ranges from 0 (completely opaque) to
-  // MaxRGB. The defines OpaqueOpacity and TransparentOpacity are
+  // QuantumRange. The defines OpaqueOpacity and TransparentOpacity are
   // available to specify completely opaque or completely transparent,
   // respectively.
   class MagickDLLDecl opacityImage : public std::unary_function<Image&,void>
@@ -1560,6 +1595,22 @@ namespace Magick
     unsigned int _scene;
   };
 
+  // adjust the image contrast with a non-linear sigmoidal contrast algorithm
+  class MagickDLLDecl sigmoidalContrastImage : public std::unary_function<Image&,void>
+  {
+  public:
+    sigmoidalContrastImage( const unsigned int sharpen_,
+		  const double contrast,
+			const double midpoint = QuantumRange / 2.0 );
+
+    void operator()( Image &image_ ) const;
+
+  private:
+    unsigned int _sharpen;
+    double contrast;
+    double midpoint;
+  };
+
   // Width and height of a raw image
   class MagickDLLDecl sizeImage : public std::unary_function<Image&,void>
   {
@@ -1756,6 +1807,7 @@ namespace Magick
     MagickLib::GetImageException( first_->image(), &exceptionInfo );
     unlinkImages( first_, last_ );
     throwException( exceptionInfo );
+    MagickLib::DestroyExceptionInfo( &exceptionInfo );
   }
 
   // Append images from list into single image in either horizontal or
@@ -1774,6 +1826,7 @@ namespace Magick
     unlinkImages( first_, last_ );
     appendedImage_->replaceImage( image );
     throwException( exceptionInfo );
+    MagickLib::DestroyExceptionInfo( &exceptionInfo );
   }
 
   // Average a set of images.
@@ -1790,6 +1843,7 @@ namespace Magick
     unlinkImages( first_, last_ );
     averagedImage_->replaceImage( image );
     throwException( exceptionInfo );
+    MagickLib::DestroyExceptionInfo( &exceptionInfo );
   }
 
   // Merge a sequence of images.
@@ -1819,6 +1873,7 @@ namespace Magick
 
     // Report any error
     throwException( exceptionInfo );
+    MagickLib::DestroyExceptionInfo( &exceptionInfo );
   }
 
   // Return format coders matching specified conditions.
@@ -1965,6 +2020,7 @@ namespace Magick
     // Deallocate histogram array
     histogram_array=(MagickLib::ColorPacket *)
       MagickLib::RelinquishMagickMemory(histogram_array);
+    MagickLib::DestroyExceptionInfo( &exceptionInfo );
   }
                       
   // Break down an image sequence into constituent parts.  This is
@@ -1991,6 +2047,7 @@ namespace Magick
 
     // Report any error
     throwException( exceptionInfo );
+    MagickLib::DestroyExceptionInfo( &exceptionInfo );
   }
 
   //
@@ -2006,6 +2063,7 @@ namespace Magick
     MagickLib::GetImageException( first_->image(), &exceptionInfo );
     unlinkImages( first_, last_ );
     throwException( exceptionInfo );
+    MagickLib::DestroyExceptionInfo( &exceptionInfo );
   }
 
   // Merge a sequence of image frames which represent image layers.
@@ -2022,6 +2080,7 @@ namespace Magick
     unlinkImages( first_, last_ );
     flattendImage_->replaceImage( image );
     throwException( exceptionInfo );
+    MagickLib::DestroyExceptionInfo( &exceptionInfo );
   }
 
   // Replace the colors of a sequence of images with the closest color
@@ -2040,7 +2099,7 @@ namespace Magick
     linkImages( first_, last_ );
     MagickLib::MapImages( first_->image(),
 			  mapImage_.constImage(),
-			  dither_ );
+			  dither_ ? MagickLib::MagickTrue : MagickLib::MagickFalse);
     MagickLib::GetImageException( first_->image(), &exceptionInfo );
     if ( exceptionInfo.severity != MagickLib::UndefinedException )
       {
@@ -2075,6 +2134,7 @@ namespace Magick
       }
 
     unlinkImages( first_, last_ );
+    MagickLib::DestroyExceptionInfo( &exceptionInfo );
   }
 
   // Create a composite image by combining several separate images.
@@ -2129,6 +2189,7 @@ namespace Magick
     // Report any transparentImage() error
     MagickLib::GetImageException( first_->image(), &exceptionInfo );
     throwException( exceptionInfo );
+    MagickLib::DestroyExceptionInfo( &exceptionInfo );
   }
 
   // Morph a set of images
@@ -2155,6 +2216,7 @@ namespace Magick
 
     // Report any error
     throwException( exceptionInfo );
+    MagickLib::DestroyExceptionInfo( &exceptionInfo );
   }
 
   // Inlay a number of images to form a single coherent picture.
@@ -2170,6 +2232,7 @@ namespace Magick
     unlinkImages( first_, last_ );
     mosaicImage_->replaceImage( image );
     throwException( exceptionInfo );
+    MagickLib::DestroyExceptionInfo( &exceptionInfo );
   }
 
   // Quantize colors in images using current quantization settings
@@ -2207,6 +2270,7 @@ namespace Magick
       }
 
     unlinkImages( first_, last_ );
+    MagickLib::DestroyExceptionInfo( &exceptionInfo );
   }
 
   // Read images into existing container (appending to container)
@@ -2223,6 +2287,7 @@ namespace Magick
     MagickLib::DestroyImageInfo(imageInfo);
     insertImages( sequence_, images);
     throwException( exceptionInfo );
+    MagickLib::DestroyExceptionInfo( &exceptionInfo );
   }
   template <class Container>
   void readImages( Container *sequence_,
@@ -2236,6 +2301,7 @@ namespace Magick
     MagickLib::DestroyImageInfo(imageInfo);
     insertImages( sequence_, images );
     throwException( exceptionInfo );
+    MagickLib::DestroyExceptionInfo( &exceptionInfo );
   }
 
   // Write Images
@@ -2261,6 +2327,7 @@ namespace Magick
       return;
 
     throwException( exceptionInfo );
+    MagickLib::DestroyExceptionInfo( &exceptionInfo );
   }
   // Write images to BLOB
   template <class InputIterator>
@@ -2285,6 +2352,7 @@ namespace Magick
     unlinkImages( first_, last_ );
 
     throwException( exceptionInfo );
+    MagickLib::DestroyExceptionInfo( &exceptionInfo );
   }
 
 } // namespace Magick
