@@ -7,7 +7,7 @@
 // A wrapper around DrawInfo, ImageInfo, and QuantizeInfo
 //
 
-#define MAGICK_IMPLEMENTATION  1
+#define MAGICKCORE_IMPLEMENTATION  1
 
 #include "Magick++/Include.h"
 #include <string>
@@ -258,6 +258,7 @@ void Magick::Options::fillPattern ( const MagickLib::Image *fillPattern_ )
 		    static_cast<MagickBooleanType>(MagickTrue),
 		    &exceptionInfo );
       throwException( exceptionInfo );
+  DestroyExceptionInfo( &exceptionInfo );
     }
 }
 const MagickLib::Image* Magick::Options::fillPattern ( void  ) const
@@ -314,6 +315,7 @@ std::string Magick::Options::format ( void ) const
   GetExceptionInfo(&exception);
   if ( *_imageInfo->magick != '\0' )
     magick_info = GetMagickInfo( _imageInfo->magick , &exception);
+  DestroyExceptionInfo(&exception);
   
   if (( magick_info != 0 ) && 
       ( *magick_info->description != '\0' ))
@@ -342,6 +344,7 @@ void Magick::Options::magick ( const std::string &magick_ )
     throwExceptionExplicit( OptionWarning,
 			    "Unrecognized image format",
 			    magick_.c_str() );
+  DestroyExceptionInfo(&exception);
 }
 std::string Magick::Options::magick ( void ) const
 {
@@ -573,6 +576,7 @@ void Magick::Options::strokePattern ( const MagickLib::Image *strokePattern_ )
 		    MagickTrue,
 		    &exceptionInfo );
       throwException( exceptionInfo );
+  DestroyExceptionInfo( &exceptionInfo );
     }
 }
 const MagickLib::Image* Magick::Options::strokePattern ( void  ) const
