@@ -258,9 +258,9 @@ int main( int /*argc*/, char ** argv)
     // baseFilename
     //
     // Base filename is color for xc images
-    if ( image.baseFilename() != "#FF0000" &&
-	 image.baseFilename() != "#FFFF00000000" &&
-	 image.baseFilename() != "#FFFFFFFF0000000000000000")
+    if ( image.baseFilename() != "xc:#FF0000" &&
+	 image.baseFilename() != "xc:#FFFF00000000" &&
+	 image.baseFilename() != "xc:#FFFFFFFF0000000000000000")
       {
 	++failures;
 	cout << "Line: " << __LINE__
@@ -447,10 +447,10 @@ int main( int /*argc*/, char ** argv)
     //
     // classType
     //
-    if ( image.classType() != DirectClass )
+    if ( image.classType() != PseudoClass )
       {
 	++failures;
-	cout << "Line: " << __LINE__ << ", classType is not DirectClass" << endl;
+	cout << "Line: " << __LINE__ << ", classType is not PseudoClass" << endl;
       }
 
     //
@@ -525,8 +525,8 @@ int main( int /*argc*/, char ** argv)
       }
 
     // Test set/get
-    image.compressType(RunlengthEncodedCompression);
-    if ( image.compressType() != RunlengthEncodedCompression )
+    image.compressType(RLECompression);
+    if ( image.compressType() != RLECompression )
       {
 	++failures;
 	cout << "Line: " << __LINE__ << ", compressType set/get failed" << endl;
@@ -913,11 +913,11 @@ int main( int /*argc*/, char ** argv)
     // matte
     //
     // Test default
-    if ( image.matte() == false )
+    if ( image.matte() != false )
       {
 	++failures;
 	cout << "Line: " << __LINE__
-             << ", matte default is not true as expected" << endl;
+             << ", matte default is not false as expected" << endl;
       }
 
     // Test set/get
@@ -1090,12 +1090,12 @@ int main( int /*argc*/, char ** argv)
     // page
     //
     // Test default
-    if ( image.page() != Geometry(0,0,0,0) )
+    if ( image.page() != Geometry(640,480,0,0) )
       {
 	++failures;
 	cout << "Line: " << __LINE__ << ", page default "
 	     << "(" << string(image.page()) << ")"
-	     << " is not empty as expected" << endl;
+	     << " is not 640x480 as expected" << endl;
       }
 
     // Test set/get
