@@ -53,6 +53,7 @@
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/quantum.h"
+#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/version.h"
@@ -533,6 +534,9 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickBooleanType
     status;
 
+  QuantumInfo
+    quantum_info;
+
   QuantumType
     quantum_type;
 
@@ -843,11 +847,12 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     default:
     {
       image->colorspace=RGBColorspace;
-      if (dpx.image.image_element[0].transfer ==  PrintingDensityColorimetric)
+      if (dpx.image.image_element[0].transfer == PrintingDensityColorimetric)
         image->colorspace=LogColorspace;
       break;
     }
   }
+  GetQuantumInfo(image_info,&quantum_info);
   length=GetBytesPerRow(image->columns,samples_per_pixel,image->depth,
     dpx.image.image_element[0].packing == 0 ? MagickFalse : MagickTrue);
   pixels=(unsigned char *) AcquireMagickMemory(length);
@@ -861,7 +866,7 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     p=ReadBlobStream(image,length,pixels,&count);
     if ((size_t) count != length)
       break;
-    status=ExportQuantumPixels(image,quantum_type,0,p);
+    status=ExportQuantumPixels(image,&quantum_info,quantum_type,p);
     if (status == MagickFalse)
       break;
     if (SyncImagePixels(image) == MagickFalse)
@@ -992,6 +997,9 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
   MagickBooleanType
     status;
 
+  QuantumInfo
+    quantum_info;
+
   QuantumType
     quantum_type;
 
@@ -1026,7 +1034,8 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
   if (status == MagickFalse)
     return(status);
   (void) ResetMagickMemory(&dpx.file,0,sizeof(dpx.file));
-  (void) SetImageColorspace(image,LogColorspace);
+  if (image_info->colorspace == UndefinedColorspace)
+    (void) SetImageColorspace(image,LogColorspace);
   /*
     Write file header.
   */
@@ -1149,6 +1158,7 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
   /*
     Convert pixel packets to DPX raster image.
   */
+  GetQuantumInfo(image_info,&quantum_info);
   quantum_type=RGBPadQuantum;
   length=GetBytesPerRow(image->columns,3,image->depth,MagickTrue);
   if (0)
@@ -1164,7 +1174,7 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
     p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    status=ImportQuantumPixels(image,quantum_type,0,pixels);
+    status=ImportQuantumPixels(image,&quantum_info,quantum_type,pixels);
     if (status == MagickFalse)
       break;
     count=WriteBlob(image,length,pixels);
