@@ -54,7 +54,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
-#include "magick/pixel.h"
+#include "magick/quantum.h"
 #include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/statistic.h"
@@ -701,7 +701,7 @@ static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,
   Image *image)
 {
   char
-    buffer[81],
+    buffer[MaxTextExtent],
     *fits_info;
 
   long
@@ -713,6 +713,9 @@ static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,
   MagickSizeType
     length;
 
+  QuantumInfo
+    quantum_info;
+
   register const PixelPacket
     *p;
 
@@ -737,7 +740,8 @@ static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  (void) SetImageColorspace(image,RGBColorspace);
+  if (image_info->colorspace == UndefinedColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
   /*
     Allocate image memory.
   */
@@ -753,41 +757,45 @@ static MagickBooleanType WriteFITSImage(const ImageInfo *image_info,
   */
   for (i=0; i < 2880; i++)
     fits_info[i]=' ';
-  (void) strcpy(buffer,"SIMPLE  =                    T");
-  (void) strncpy(fits_info+0,buffer,strlen(buffer));
+  (void) CopyMagickString(buffer,"SIMPLE  =                    T",
+    MaxTextExtent);
+  (void) CopyMagickString(fits_info+0,buffer,2880);
   (void) FormatMagickString(buffer,MaxTextExtent,"BITPIX  =           %10lu",
     image->depth);
-  (void) strncpy(fits_info+80,buffer,strlen(buffer));
-  (void) strcpy(buffer,"NAXIS   =                    2");
-  (void) strncpy(fits_info+160,buffer,strlen(buffer));
+  (void) CopyMagickString(fits_info+80,buffer,2880);
+  (void) CopyMagickString(buffer,"NAXIS   =                    2",
+    MaxTextExtent);
+  (void) CopyMagickString(fits_info+160,buffer,2880);
   (void) FormatMagickString(buffer,MaxTextExtent,"NAXIS1  =           %10lu",
     image->columns);
-  (void) strncpy(fits_info+240,buffer,strlen(buffer));
+  (void) CopyMagickString(fits_info+240,buffer,2880);
   (void) FormatMagickString(buffer,MaxTextExtent,"NAXIS2  =           %10lu",
     image->rows);
-  (void) strncpy(fits_info+320,buffer,strlen(buffer));
+  (void) CopyMagickString(fits_info+320,buffer,2880);
   (void) FormatMagickString(buffer,MaxTextExtent,"DATAMIN =           %10u",0);
-  (void) strncpy(fits_info+400,buffer,strlen(buffer));
+  (void) CopyMagickString(fits_info+400,buffer,2880);
   (void) FormatMagickString(buffer,MaxTextExtent,"DATAMAX =           %10u",
     (1 << image->depth)-1);
-  (void) strncpy(fits_info+480,buffer,strlen(buffer));
+  (void) CopyMagickString(fits_info+480,buffer,2880);
   (void) FormatMagickString(buffer,MaxTextExtent,"BZERO   =           %10u",
     image->depth > 8 ? 32768 : 0);
-  (void) strncpy(fits_info+560,buffer,strlen(buffer));
-  (void) strcpy(buffer,"HISTORY Created by ImageMagick.");
-  (void) strncpy(fits_info+640,buffer,strlen(buffer));
-  (void) strcpy(buffer,"END");
-  (void) strncpy(fits_info+720,buffer,strlen(buffer));
+  (void) CopyMagickString(fits_info+560,buffer,2880);
+  (void) CopyMagickString(buffer,"HISTORY Created by ImageMagick.",
+    MaxTextExtent);
+  (void) CopyMagickString(fits_info+640,buffer,2880);
+  (void) CopyMagickString(buffer,"END",MaxTextExtent);
+  (void) CopyMagickString(fits_info+720,buffer,2880);
   (void) WriteBlob(image,2880,(unsigned char *) fits_info);
   /*
     Convert image to fits scale PseudoColor class.
   */
+  GetQuantumInfo(image_info,&quantum_info);
   for (y=(long) image->rows-1; y >= 0; y--)
   {
     p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    (void) ImportQuantumPixels(image,GrayQuantum,0,pixels);
+    (void) ImportQuantumPixels(image,&quantum_info,GrayQuantum,pixels);
     (void) WriteBlob(image,packet_size*image->columns,pixels);
     if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
         (QuantumTick(image->rows-y-1,image->rows) != MagickFalse))
