@@ -119,9 +119,9 @@ static Image *ReadGRADIENTImage(const ImageInfo *image_info,
       image=DestroyImage(image);
       return((Image *) NULL);
     }
-  (void) strcpy(colorname,"white");
+  (void) CopyMagickString(colorname,"white",MaxTextExtent);
   if (PixelIntensityToQuantum(&start_color) > (QuantumRange/2))
-    (void) strcpy(colorname,"black");
+    (void) CopyMagickString(colorname,"black",MaxTextExtent);
   (void) sscanf(image_info->filename,"%*[^-]-%s",colorname);
   if (QueryColorDatabase(colorname,&stop_color,exception) == MagickFalse)
     {
