@@ -205,8 +205,11 @@ static int BlobRead(jas_stream_obj_t *object,char *buffer,const int length)
   ssize_t
     count;
 
-  count=ReadBlob(((StreamManager *) object)->image,(size_t) length,
-    (void *) buffer);
+  StreamManager
+    *source;
+
+  source=(StreamManager *) object;
+  count=ReadBlob(source->image,(size_t) length,(void *) buffer);
   return((int) count);
 }
 
@@ -215,19 +218,30 @@ static int BlobWrite(jas_stream_obj_t *object,char *buffer,const int length)
   size_t
     count;
 
-  count=WriteBlob(((StreamManager *) object)->image,(size_t) length,
-    (void *) buffer);
+  StreamManager
+    *source;
+
+  source=(StreamManager *) object;
+  count=WriteBlob(source->image,(size_t) length,(void *) buffer);
   return((int) count);
 }
 
 static long BlobSeek(jas_stream_obj_t *object,long offset,int origin)
 {
-  return(SeekBlob(((StreamManager *) object)->image,offset,origin));
+  StreamManager
+    *source;
+
+  source=(StreamManager *) object;
+  return(SeekBlob(source->image,offset,origin));
 }
 
 static int BlobClose(jas_stream_obj_t *object)
 {
-  CloseBlob(((StreamManager *) object)->image);
+  StreamManager
+    *source;
+
+  source=(StreamManager *) object;
+  CloseBlob(source->image);
   return(0);
 }
 
@@ -569,13 +583,13 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
           (jas_iccprof_save(icc_profile,icc_stream) == 0) &&
           (jas_stream_flush(icc_stream) == 0))
         {
-          jas_stream_memobj_t
-            *blob;
-
           StringInfo
             *icc_profile,
             *profile;
 
+          jas_stream_memobj_t
+            *blob;
+
           /*
             Extract the icc profile, handle errors without much noise.
           */
@@ -585,7 +599,7 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
               "Profile: ICC, %lu bytes",(unsigned long) blob->len_);
           profile=AcquireStringInfo(blob->len_);
           SetStringInfoDatum(profile,blob->buf_);
-          icc_profile=GetImageProfile(image,"icc");
+          icc_profile=(StringInfo *) GetImageProfile(image,"icc");
           if (icc_profile == (StringInfo *) NULL)
             SetImageProfile(image,"icc",profile);
           else
@@ -794,7 +808,8 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
   /*
     Intialize JPEG 2000 API.
   */
-  (void) SetImageColorspace(image,RGBColorspace);
+  if (image_info->colorspace == UndefinedColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
   jp2_stream=JP2StreamManager(image);
   if (jp2_stream == (jas_stream_t *) NULL)
     ThrowWriterException(DelegateError,"UnableToManageJP2Stream");
