@@ -191,7 +191,7 @@ static int stringnicmp(const char *p,const char *q,size_t n)
     p++;
     q++;
   }
-  return(toupper(*p)-toupper(*q));
+  return(toupper((int) *p)-toupper((int) *q));
 }
 
 static int convertHTMLcodes(char *s, int len)
@@ -213,7 +213,7 @@ static int convertHTMLcodes(char *s, int len)
             break;
         }
         if (o < 6)
-          (void) strcpy(s+1, s+1+o);
+          (void) CopyMagickString(s+1,s+1+o,strlen(s+1+o)+1);
         *s = val;
         return o;
       }
@@ -229,7 +229,7 @@ static int convertHTMLcodes(char *s, int len)
         if (html_codes[i].len <= len)
           if (stringnicmp(s, html_codes[i].code, html_codes[i].len) == 0)
             {
-              (void) strcpy(s+1, s+html_codes[i].len);
+              (void) CopyMagickString(s+1,s+html_codes[i].len,strlen(s+html_codes[i].len));
               *s = html_codes[i].val;
               return html_codes[i].len-1;
             }
@@ -368,7 +368,7 @@ static long parse8BIM(Image *ifile, Image *ofile)
               case 2:
                 name=(char *) AcquireMagickMemory(strlen(newstr)+MaxTextExtent);
                 if (name)
-                  (void) strcpy(name,newstr);
+                  (void) CopyMagickString(name,newstr,strlen(newstr)+MaxTextExtent);
                 break;
             }
             state++;
@@ -627,7 +627,7 @@ static long parse8BIMW(Image *ifile, Image *ofile)
               case 2:
                 name=(char *) AcquireMagickMemory(strlen(newstr)+MaxTextExtent);
                 if (name)
-                  (void) strcpy(name,newstr);
+                  (void) CopyMagickString(name,newstr,strlen(newstr)+MaxTextExtent);
                 break;
             }
             state++;
@@ -1041,12 +1041,12 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
   MagickBooleanType
     status;
 
-  size_t
-    length;
-
   StringInfo
     *profile;
 
+  size_t
+    length;
+
   void
     *blob;
 
@@ -1269,12 +1269,13 @@ static Image *ReadMETAImage(const ImageInfo *image_info,
         }
       AttachBlob(buff->blob,blob,length);
       /* write out the header - length field patched below */
-      WriteBlob(buff,12,(unsigned char *) "8BIM\04\04\0\0\0\0\0\0");
+      WriteBlob(buff,11,(unsigned char *) "8BIM\04\04\0\0\0\0\01");
+      WriteBlobByte(buff,0xe0);
       if (LocaleCompare(image_info->magick,"IPTCTEXT") == 0)
         {
           length=parse8BIM(image, buff);
           if (length & 1)
-            WriteBlobByte(buff,0x0);
+            WriteBlobByte(buff,0x00);
         }
       else if (LocaleCompare(image_info->magick,"IPTCWTEXT") == 0)
         {
@@ -2118,12 +2119,12 @@ static int format8BIM(Image *ifile, Image *ofile)
 static MagickBooleanType WriteMETAImage(const ImageInfo *image_info,
   Image *image)
 {
+  const StringInfo
+    *profile;
+
   MagickBooleanType
     status;
 
-  StringInfo
-    *profile;
-
   size_t
     length;
 
