@@ -124,8 +124,8 @@ static MagickBooleanType
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  IsMIFF() returns MagickTrue if the image format type, identified by the magick
-%  string, is MIFF.
+%  IsMIFF() returns MagickTrue if the image format type, identified by the
+%  magick string, is MIFF.
 %
 %  The format of the IsMIFF method is:
 %
@@ -413,6 +413,9 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
   PixelPacket
     pixel;
 
+  QuantumInfo
+    quantum_info;
+
   QuantumType
     quantum_type;
 
@@ -442,9 +445,6 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
   ssize_t
     count;
 
-  StringInfo
-    *profile;
-
   unsigned char
     *compress_pixels,
     *pixels;
@@ -774,6 +774,9 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
                   }
                 if (LocaleNCompare(keyword,"profile-",8) == 0)
                   {
+                    StringInfo
+                      *profile;
+
                     if (profiles == (LinkedListInfo *) NULL)
                       profiles=NewLinkedList(0);
                     (void) AppendValueToLinkedList(profiles,
@@ -950,6 +953,9 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
         const char
           *name;
 
+        const StringInfo
+          *profile;
+
         /*
           Read image profiles.
         */
@@ -1073,6 +1079,7 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
     /*
       Read image pixels.
     */
+    GetQuantumInfo(image_info,&quantum_info);
     quantum_type=RGBQuantum;
     if (image->storage_class == PseudoClass)
       quantum_type=image->matte != MagickFalse ? IndexAlphaQuantum :
@@ -1131,7 +1138,7 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
               if (code >= 0)
                 status=MagickTrue;
             }
-          status=ExportQuantumPixels(image,quantum_type,0,pixels);
+          status=ExportQuantumPixels(image,&quantum_info,quantum_type,pixels);
           break;
         }
 #endif
@@ -1174,7 +1181,7 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
               if (code >= 0)
                 status=MagickTrue;
             }
-          status=ExportQuantumPixels(image,quantum_type,0,pixels);
+          status=ExportQuantumPixels(image,&quantum_info,quantum_type,pixels);
           break;
         }
 #endif
@@ -1204,7 +1211,7 @@ static Image *ReadMIFFImage(const ImageInfo *image_info,
         default:
         {
           p=ReadBlobStream(image,packet_size*image->columns,pixels,&count);
-          status=ExportQuantumPixels(image,quantum_type,0,p);
+          status=ExportQuantumPixels(image,&quantum_info,quantum_type,p);
           break;
         }
       }
@@ -1350,7 +1357,8 @@ ModuleExport void UnregisterMIFFImage(void)
 %
 %  The format of the WriteMIFFImage method is:
 %
-%      MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
+%        Image *image)
 %
 %  Compression code contributed by Kyle Shorter.
 %
@@ -1379,11 +1387,11 @@ static inline size_t PopRunlengthPacket(Image *image,unsigned char *pixels,
       {
         case 32:
         {
-          *q++=(unsigned char) (index >> 24);
-          *q++=(unsigned char) (index >> 16);
+          *q++=(unsigned char) ((unsigned long) index >> 24);
+          *q++=(unsigned char) ((unsigned long) index >> 16);
         }
         case 16:
-          *q++=(unsigned char) (index >> 8);
+          *q++=(unsigned char) ((unsigned long) index >> 8);
         case 8:
         {
           *q++=(unsigned char) index;
@@ -1534,6 +1542,9 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
   PixelPacket
     pixel;
 
+  QuantumInfo
+    quantum_info;
+
   QuantumType
     quantum_type;
 
@@ -1585,7 +1596,7 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
     image->depth=GetImageQuantumDepth(image,MagickTrue);
     if ((image->storage_class == PseudoClass) &&
         (image->colors > (1UL << image->depth)))
-      image->storage_class=DirectClass;
+      (void) SetImageStorageClass(image,DirectClass);
     compression=NoCompression;
     switch (image->compression)
     {
@@ -1902,6 +1913,7 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
     /*
       Write image pixels to file.
     */
+    GetQuantumInfo(image_info,&quantum_info);
     quantum_type=RGBQuantum;
     if (image->storage_class == PseudoClass)
       quantum_type=image->matte != MagickFalse ? IndexAlphaQuantum :
@@ -1937,7 +1949,7 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
             }
           zip_info.next_in=pixels;
           zip_info.avail_in=(uInt) (packet_size*image->columns);
-          (void) ImportQuantumPixels(image,quantum_type,0,pixels);
+          (void) ImportQuantumPixels(image,&quantum_info,quantum_type,pixels);
           do
           {
             zip_info.next_out=compress_pixels;
@@ -1990,7 +2002,7 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
             }
           bzip_info.next_in=(char *) pixels;
           bzip_info.avail_in=(unsigned int) (packet_size*image->columns);
-          (void) ImportQuantumPixels(image,quantum_type,0,pixels);
+          (void) ImportQuantumPixels(image,&quantum_info,quantum_type,pixels);
           do
           {
             bzip_info.next_out=(char *) compress_pixels;
@@ -2039,7 +2051,7 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
           for (x=0; x < (long) image->columns; x++)
           {
             if ((length < 255) && (x < (long) (image->columns-1)) &&
-                (ColorMatch(p,&pixel) != MagickFalse) &&
+                (IsColorEqual(p,&pixel) != MagickFalse) &&
                 ((image->matte == MagickFalse) ||
                  (p->opacity == pixel.opacity)) &&
                 ((indexes == (IndexPacket *) NULL) || (index == indexes[x])))
@@ -2061,7 +2073,7 @@ static MagickBooleanType WriteMIFFImage(const ImageInfo *image_info,
         }
         default:
         {
-          (void) ImportQuantumPixels(image,quantum_type,0,pixels);
+          (void) ImportQuantumPixels(image,&quantum_info,quantum_type,pixels);
           (void) WriteBlob(image,packet_size*image->columns,pixels);
           break;
         }
