@@ -150,7 +150,8 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
     density[MaxTextExtent],
     filename[MaxTextExtent],
     geometry[MaxTextExtent],
-    options[MaxTextExtent];
+    options[MaxTextExtent],
+    input_filename[MaxTextExtent];
 
   const DelegateInfo
     *delegate_info;
@@ -206,6 +207,14 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image=DestroyImageList(image);
       return((Image *) NULL);
     }
+  status=AcquireUniqueSymbolicLink(image_info->filename,input_filename);
+  if (status == MagickFalse)
+    {
+      ThrowFileException(exception,FileOpenError,"UnableToCreateTemporaryFile",
+        image_info->filename);
+      image=DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Set the page density.
   */
@@ -230,7 +239,7 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Determine page geometry from the PCL media box.
   */
-  cmyk=MagickFalse;
+  cmyk=image_info->colorspace == CMYKColorspace ? MagickTrue : MagickFalse;
   count=0;
   (void) ResetMagickMemory(&bounding_box,0,sizeof(bounding_box));
   (void) ResetMagickMemory(&bounds,0,sizeof(bounds));
@@ -245,7 +254,7 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
       Note PCL elements.
     */
     *p++=(char) c;
-    if ((c != (int) '/') && (c != EOF) &&
+    if ((c != (int) '/') && (c != '\n') &&
         ((size_t) (p-command) < (MaxTextExtent/2)))
       continue;
     *p='\0';
@@ -338,11 +347,12 @@ static Image *ReadPCLImage(const ImageInfo *image_info,ExceptionInfo *exception)
     GetDelegateCommands(delegate_info),
     read_info->antialias != MagickFalse ? 4 : 1,
     read_info->antialias != MagickFalse ? 4 : 1,geometry,density,options,
-    read_info->filename,image_info->filename);
+    read_info->filename,input_filename);
   status=SystemCommand(read_info->verbose,command) != 0 ? MagickTrue :
     MagickFalse;
   image=ReadImage(read_info,exception);
   (void) RelinquishUniqueFileResource(read_info->filename);
+  (void) RelinquishUniqueFileResource(input_filename);
   read_info=DestroyImageInfo(read_info);
   if (image == (Image *) NULL)
     ThrowReaderException(DelegateError,"PCLDelegateFailed");
@@ -402,6 +412,9 @@ ModuleExport void RegisterPCLImage(void)
   entry->encoder=(EncoderHandler *) WritePCLImage;
   entry->magick=(MagickHandler *) IsPCL;
   entry->adjoin=MagickFalse;
+  entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
+  entry->thread_support=MagickFalse;
   entry->description=ConstantString("Printer Control Language");
   entry->module=ConstantString("PCL");
   (void) RegisterMagickInfo(entry);
@@ -496,7 +509,8 @@ static MagickBooleanType WritePCLImage(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  (void) SetImageColorspace(image,RGBColorspace);
+  if (image_info->colorspace == UndefinedColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
   /*
     Initialize the printer.
   */
