@@ -54,7 +54,7 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
-#include "magick/pixel.h"
+#include "magick/quantum.h"
 #include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
@@ -698,6 +698,9 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
   PDBInfo
     pdb_info;
 
+  QuantumInfo
+    quantum_info;
+
   register const PixelPacket
     *p;
 
@@ -736,7 +739,8 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  (void) SetImageColorspace(image,RGBColorspace);
+  if (image_info->colorspace == UndefinedColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
   bits_per_pixel=image->depth;
   if (GetImageType(image,&image->exception) == BilevelType)
     bits_per_pixel=1;
@@ -802,10 +806,12 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
   scanline=(unsigned char *) AcquireMagickMemory(packet_size*image->columns);
   if (scanline == (unsigned char *) NULL)
     ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-  (void) SetImageColorspace(image,RGBColorspace);
+    if (image_info->colorspace == UndefinedColorspace)
+    (void) SetImageColorspace(image,RGBColorspace);
   /*
     Convert to GRAY raster scanline.
   */
+  GetQuantumInfo(image_info,&quantum_info);
   bits=8/(long) bits_per_pixel-1;  /* start at most significant bits */
   literal=0;
   repeat=0;
@@ -816,7 +822,7 @@ static MagickBooleanType WritePDBImage(const ImageInfo *image_info,Image *image)
     p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
     if (p == (const PixelPacket *) NULL)
       break;
-    (void) ImportQuantumPixels(image,GrayQuantum,0,scanline);
+    (void) ImportQuantumPixels(image,&quantum_info,GrayQuantum,scanline);
     for (x=0; x < pdb_image.width; x++)
     {
       if (x < (long) image->columns)
