@@ -59,9 +59,10 @@
 #include "magick/magick.h"
 #include "magick/memory_.h"
 #include "magick/monitor.h"
+#include "magick/option.h"
+#include "magick/quantum.h"
 #include "magick/resource_.h"
 #include "magick/resize.h"
-#include "magick/quantum.h"
 #include "magick/static.h"
 #include "magick/string_.h"
 #include "magick/transform.h"
@@ -84,6 +85,103 @@ static MagickBooleanType
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   I n v o k e P o s t s r i p t D e l e g a t e                             %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  InvokePostscriptDelegate() executes the postscript interpreter with the
+%  specified command.
+%
+%  The format of the InvokePostscriptDelegate method is:
+%
+%      MagickBooleanType InvokePostscriptDelegate(
+%        const MagickBooleanType verbose,const char *command)
+%
+%  A description of each parameter follows:
+%
+%    o verbose: A value other than zero displays the command prior to
+%      executing it.
+%
+%    o command: The address of a character string containing the command to
+%      execute.
+%
+*/
+static MagickBooleanType InvokePostscriptDelegate(
+  const MagickBooleanType verbose,const char *command)
+{
+#if defined(HasGS) || defined(__WINDOWS__)
+  char
+    **argv;
+
+  const GhostscriptVectors
+    *gs_func;
+
+  gs_main_instance
+    *interpreter;
+
+  int
+    argc,
+    code,
+    status;
+
+  register long
+    i;
+
+#if defined(__WINDOWS__)
+  gs_func=NTGhostscriptDLLVectors();
+#else
+  GhostscriptVectors
+    gs_func_struct;
+
+  gs_func=(&gs_func_struct);
+  gs_func_struct.exit=gsapi_exit;
+  gs_func_struct.init_with_args=gsapi_init_with_args;
+  gs_func_struct.new_instance=gsapi_new_instance;
+  gs_func_struct.run_string=gsapi_run_string;
+  gs_func_struct.delete_instance=gsapi_delete_instance;
+#endif
+  if (gs_func == (GhostscriptVectors *) NULL)
+    return(SystemCommand(verbose,command));
+  if (verbose != MagickFalse)
+    {
+      (void) fputs("[ghostscript library]",stdout);
+      (void) fputs(strchr(command,' '),stdout);
+    }
+  status=(gs_func->new_instance)(&interpreter,(void *) NULL);
+  if (status < 0)
+    return(MagickFalse);
+  argv=StringToArgv(command,&argc);
+  status=(gs_func->init_with_args)(interpreter,argc-1,argv+1);
+  if (status == 0)
+    status=(gs_func->run_string)
+      (interpreter,"systemdict /start get exec\n",0,&code);
+  (gs_func->exit)(interpreter);
+  (gs_func->delete_instance)(interpreter);
+#if defined(__WINDOWS__)
+  NTGhostscriptUnLoadDLL();
+#endif
+  for (i=0; i < (long) argc; i++)
+    argv[i]=(char *) RelinquishMagickMemory(argv[i]);
+  argv=(char **) RelinquishMagickMemory(argv);
+  if ((status == 0) || (status <= -100))
+    {
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+        "Ghostscript returns status %d, exit code %d",status,code);
+      return(MagickFalse);
+    }
+  return(MagickTrue);
+#else
+  return(SystemCommand(verbose,command) != 0 ? MagickTrue : MagickFalse);
+#endif
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   I s P D F                                                                 %
 %                                                                             %
 %                                                                             %
@@ -148,6 +246,7 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 #define DeviceCMYK  "DeviceCMYK"
 #define MediaBox  "MediaBox"
 #define RenderPostscriptText  "  Rendering Postscript...  "
+#define TrimBox  "TrimBox"
 
   char
     command[MaxTextExtent],
@@ -155,8 +254,12 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     filename[MaxTextExtent],
     geometry[MaxTextExtent],
     options[MaxTextExtent],
+    input_filename[MaxTextExtent],
     postscript_filename[MaxTextExtent];
 
+  const char
+    *option;
+
   const DelegateInfo
     *delegate_info;
 
@@ -172,6 +275,8 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   MagickBooleanType
     cmyk,
+    cropbox,
+    trimbox,
     status;
 
   PointInfo
@@ -215,6 +320,14 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image=DestroyImageList(image);
       return((Image *) NULL);
     }
+  status=AcquireUniqueSymbolicLink(image_info->filename,input_filename);
+  if (status == MagickFalse)
+    {
+      ThrowFileException(exception,FileOpenError,"UnableToCreateTemporaryFile",
+        image_info->filename);
+      image=DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Set the page density.
   */
@@ -239,7 +352,15 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
   /*
     Determine page geometry from the PDF media box.
   */
-  cmyk=MagickFalse;
+  cmyk=image_info->colorspace == CMYKColorspace ? MagickTrue : MagickFalse;
+  cropbox=MagickFalse;
+  option=GetImageOption(image_info,"pdf:use-cropbox");
+  if (option != (const char *) NULL)
+    cropbox=IsMagickTrue(option);
+  trimbox=MagickFalse;
+  option=GetImageOption(image_info,"pdf:use-trimbox");
+  if (option != (const char *) NULL)
+    trimbox=IsMagickTrue(option);
   count=0;
   (void) ResetMagickMemory(&bounding_box,0,sizeof(bounding_box));
   (void) ResetMagickMemory(&bounds,0,sizeof(bounds));
@@ -254,7 +375,7 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       Note PDF elements.
     */
     *p++=(char) c;
-    if ((c != (int) '/') && (c != EOF) &&
+    if ((c != (int) '/') && (c != '\n') &&
         ((size_t) (p-command) < (MaxTextExtent/2)))
       continue;
     *p='\0';
@@ -264,7 +385,8 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     */
     if (LocaleNCompare(DeviceCMYK,command,strlen(DeviceCMYK)) == 0)
       cmyk=MagickTrue;
-    if (LocaleNCompare(CropBox,command,strlen(CropBox)) == 0)
+    if ((cropbox != MagickFalse) &&
+        (LocaleNCompare(CropBox,command,strlen(CropBox)) == 0))
       {
         /*
           Note region defined by crop box.
@@ -275,7 +397,20 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
           count=(ssize_t) sscanf(command,"CropBox[%lf %lf %lf %lf",
             &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
       }
-    if (LocaleNCompare(MediaBox,command,strlen(MediaBox)) == 0)
+    if ((trimbox != MagickFalse) &&
+        (LocaleNCompare(TrimBox,command,strlen(TrimBox)) == 0))
+      {
+        /*
+          Note region defined by trim box.
+        */
+        count=(ssize_t) sscanf(command,"TrimBox [%lf %lf %lf %lf",
+          &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
+        if (count != 4)
+          count=(ssize_t) sscanf(command,"TrimBox[%lf %lf %lf %lf",
+            &bounds.x1,&bounds.y1,&bounds.x2,&bounds.y2);
+      }
+    if ((cropbox == MagickFalse) && (trimbox == MagickFalse) &&
+        (LocaleNCompare(MediaBox,command,strlen(MediaBox)) == 0))
       {
         /*
           Note region defined by media box.
@@ -342,6 +477,10 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
       return((Image *) NULL);
     }
   *options='\0';
+  if (cropbox != MagickFalse)
+    (void) ConcatenateMagickString(options,"-dUseCropBox ",MaxTextExtent);
+  if (trimbox != MagickFalse)
+    (void) ConcatenateMagickString(options,"-dUseTrimBox ",MaxTextExtent);
   read_info=CloneImageInfo(image_info);
   *read_info->magick='\0';
   if (read_info->number_scenes != 0)
@@ -366,11 +505,12 @@ static Image *ReadPDFImage(const ImageInfo *image_info,ExceptionInfo *exception)
     GetDelegateCommands(delegate_info),
     read_info->antialias != MagickFalse ? 4 : 1,
     read_info->antialias != MagickFalse ? 4 : 1,geometry,density,options,
-    read_info->filename,postscript_filename,image_info->filename);
+    read_info->filename,postscript_filename,input_filename);
   status=InvokePostscriptDelegate(read_info->verbose,command);
   image=ReadImage(read_info,exception);
   (void) RelinquishUniqueFileResource(postscript_filename);
   (void) RelinquishUniqueFileResource(read_info->filename);
+  (void) RelinquishUniqueFileResource(input_filename);
   read_info=DestroyImageInfo(read_info);
   if (image == (Image *) NULL)
     ThrowReaderException(DelegateError,"PostscriptDelegateFailed");
@@ -433,6 +573,7 @@ ModuleExport void RegisterPDFImage(void)
   entry->adjoin=MagickFalse;
   entry->blob_support=MagickFalse;
   entry->seekable_stream=MagickTrue;
+  entry->thread_support=MagickFalse;
   entry->description=ConstantString("Encapsulated Portable Document Format");
   entry->module=ConstantString("PDF");
   (void) RegisterMagickInfo(entry);
@@ -442,6 +583,7 @@ ModuleExport void RegisterPDFImage(void)
   entry->magick=(MagickHandler *) IsPDF;
   entry->blob_support=MagickFalse;
   entry->seekable_stream=MagickTrue;
+  entry->thread_support=MagickFalse;
   entry->description=ConstantString("Portable Document Format");
   entry->module=ConstantString("PDF");
   (void) RegisterMagickInfo(entry);
@@ -627,6 +769,14 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
   compression=image->compression;
   switch (compression)
   {
+    case FaxCompression:
+    case Group4Compression:
+    {
+      if ((IsMonochromeImage(image,&image->exception) == MagickFalse) ||
+          (image->matte != MagickFalse))
+        compression=RLECompression;
+      break;
+    }
 #if !defined(HasJPEG)
     case JPEGCompression:
     {
@@ -791,7 +941,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
       else
         if ((image->gravity != UndefinedGravity) &&
             (LocaleCompare(image_info->magick,"PDF") == 0))
-          (void) strcpy(page_geometry,PSPageGeometry);
+          (void) CopyMagickString(page_geometry,PSPageGeometry,MaxTextExtent);
     (void) ParseMetaGeometry(page_geometry,&geometry.x,&geometry.y,
       &geometry.width,&geometry.height);
     scale.x=(double) (geometry.width*delta.x)/resolution.x;
@@ -830,7 +980,8 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
       matte=MagickFalse;
     if (compression == JPEG2000Compression)
       {
-	      (void) SetImageColorspace(image,RGBColorspace);
+        if (image_info->colorspace == UndefinedColorspace)
+  	      (void) SetImageColorspace(image,RGBColorspace);
         matte=MagickFalse;
       }
     (void) FormatMagickString(buffer,MaxTextExtent,
@@ -919,12 +1070,12 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     (void) FormatMagickString(buffer,MaxTextExtent,"%lu 0 obj\n",object);
     (void) WriteBlobString(image,buffer);
     if ((image->storage_class == DirectClass) || (image->colors > 256))
-      (void) strcpy(buffer,"[ /PDF /Text /ImageC");
+      (void) CopyMagickString(buffer,"[ /PDF /Text /ImageC",MaxTextExtent);
     else
-      if (compression == FaxCompression)
-        (void) strcpy(buffer,"[ /PDF /Text /ImageB");
+      if ((compression == FaxCompression) || (compression == Group4Compression))
+        (void) CopyMagickString(buffer,"[ /PDF /Text /ImageB",MaxTextExtent);
       else
-        (void) strcpy(buffer,"[ /PDF /Text /ImageI");
+        (void) CopyMagickString(buffer,"[ /PDF /Text /ImageI",MaxTextExtent);
     (void) WriteBlobString(image,buffer);
     (void) WriteBlobString(image," ]\n");
     (void) WriteBlobString(image,"endobj\n");
@@ -993,7 +1144,8 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
         (void) WriteBlobString(image,buffer);
         (void) WriteBlobString(image,"/ColorSpace /DeviceGray\n");
         (void) FormatMagickString(buffer,MaxTextExtent,"/BitsPerComponent %d\n",
-          compression == FaxCompression ? 1 : 8);
+          (compression == FaxCompression) || (compression == Group4Compression)
+          ? 1 : 8);
         (void) WriteBlobString(image,buffer);
         (void) FormatMagickString(buffer,MaxTextExtent,"/Length %lu 0 R\n",
           object+1);
@@ -1112,7 +1264,8 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
         if (image->colorspace != CMYKColorspace)
           break;
         (void) WriteBlobString(image,buffer);
-        (void) strcpy(buffer,"/Decode [1 0 1 0 1 0 1 0]\n");
+        (void) CopyMagickString(buffer,"/Decode [1 0 1 0 1 0 1 0]\n",
+          MaxTextExtent);
         break;
       }
       case JPEG2000Compression:
@@ -1121,7 +1274,8 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
         if (image->colorspace != CMYKColorspace)
           break;
         (void) WriteBlobString(image,buffer);
-        (void) strcpy(buffer,"/Decode [1 0 1 0 1 0 1 0]\n");
+        (void) CopyMagickString(buffer,"/Decode [1 0 1 0 1 0 1 0]\n",
+          MaxTextExtent);
         break;
       }
       case LZWCompression:
@@ -1135,11 +1289,13 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
          break;
       }
       case FaxCompression:
+      case Group4Compression:
       {
-        (void) strcpy(buffer,"/Filter [ /CCITTFaxDecode ]\n");
+        (void) CopyMagickString(buffer,"/Filter [ /CCITTFaxDecode ]\n",
+          MaxTextExtent);
         (void) WriteBlobString(image,buffer);
         (void) FormatMagickString(buffer,MaxTextExtent,
-          "/DecodeParms [ << >> << /K %s /Columns %ld /Rows %ld >> ]\n",
+          "/DecodeParms [ << /K %s /Columns %ld /Rows %ld >> ]\n",
           CCITTParam,image->columns,image->rows);
         break;
       }
@@ -1160,7 +1316,8 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
       object+2);
     (void) WriteBlobString(image,buffer);
     (void) FormatMagickString(buffer,MaxTextExtent,"/BitsPerComponent %d\n",
-      compression == FaxCompression ? 1 : 8);
+      (compression == FaxCompression) || (compression == Group4Compression) ?
+      1 : 8);
     (void) WriteBlobString(image,buffer);
     if (matte != MagickFalse)
       {
@@ -1177,7 +1334,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     number_pixels=(MagickSizeType) image->columns*image->rows;
     if ((4*number_pixels) != (MagickSizeType) ((size_t) (4*number_pixels)))
       ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
-    if ((compression == FaxCompression) ||
+    if ((compression == FaxCompression) || (compression == Group4Compression) ||
         ((image_info->type != TrueColorType) &&
          (IsGrayImage(image,&image->exception) != MagickFalse)))
 
@@ -1185,6 +1342,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
         switch (compression)
         {
           case FaxCompression:
+          case Group4Compression:
           {
             if (LocaleCompare(CCITTParam,"0") == 0)
               {
@@ -1515,17 +1673,18 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     (void) FormatMagickString(buffer,MaxTextExtent,"%lu 0 obj\n",object);
     (void) WriteBlobString(image,buffer);
     if (image->colorspace == CMYKColorspace)
-      (void) strcpy(buffer,"/DeviceCMYK\n");
+      (void) CopyMagickString(buffer,"/DeviceCMYK\n",MaxTextExtent);
     else
       if ((compression == FaxCompression) ||
+          (compression == Group4Compression) ||
           ((image_info->type != TrueColorType) &&
            (IsGrayImage(image,&image->exception) != MagickFalse)))
-          (void) strcpy(buffer,"/DeviceGray\n");
+          (void) CopyMagickString(buffer,"/DeviceGray\n",MaxTextExtent);
       else
         if ((image->storage_class == DirectClass) || (image->colors > 256) ||
             (compression == JPEGCompression) ||
             (compression == JPEG2000Compression))
-          (void) strcpy(buffer,"/DeviceRGB\n");
+          (void) CopyMagickString(buffer,"/DeviceRGB\n",MaxTextExtent);
         else
           (void) FormatMagickString(buffer,MaxTextExtent,
             "[ /Indexed /DeviceRGB %lu %lu 0 R ]\n",
@@ -1559,7 +1718,8 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
         if (image->colorspace != CMYKColorspace)
           break;
         (void) WriteBlobString(image,buffer);
-        (void) strcpy(buffer,"/Decode [1 0 1 0 1 0 1 0]\n");
+        (void) CopyMagickString(buffer,"/Decode [1 0 1 0 1 0 1 0]\n",
+          MaxTextExtent);
         break;
       }
       case JPEG2000Compression:
@@ -1568,7 +1728,8 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
         if (image->colorspace != CMYKColorspace)
           break;
         (void) WriteBlobString(image,buffer);
-        (void) strcpy(buffer,"/Decode [1 0 1 0 1 0 1 0]\n");
+        (void) CopyMagickString(buffer,"/Decode [1 0 1 0 1 0 1 0]\n",
+          MaxTextExtent);
         break;
       }
       case LZWCompression:
@@ -1582,11 +1743,13 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
         break;
       }
       case FaxCompression:
+      case Group4Compression:
       {
-        (void) strcpy(buffer,"/Filter [ /CCITTFaxDecode ]\n");
+        (void) CopyMagickString(buffer,"/Filter [ /CCITTFaxDecode ]\n",
+          MaxTextExtent);
         (void) WriteBlobString(image,buffer);
         (void) FormatMagickString(buffer,MaxTextExtent,
-          "/DecodeParms [ << >> << /K %s /Columns %lu /Rows %lu >> ]\n",
+          "/DecodeParms [ << /K %s /Columns %lu /Rows %lu >> ]\n",
           CCITTParam,image->columns,image->rows);
         break;
       }
@@ -1608,7 +1771,8 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
       object-1);
     (void) WriteBlobString(image,buffer);
     (void) FormatMagickString(buffer,MaxTextExtent,"/BitsPerComponent %d\n",
-      compression == FaxCompression ? 1 : 8);
+      (compression == FaxCompression) || (compression == Group4Compression) ?
+      1 : 8);
     (void) WriteBlobString(image,buffer);
     (void) FormatMagickString(buffer,MaxTextExtent,"/Length %lu 0 R\n",
       object+1);
@@ -1618,17 +1782,21 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     offset=TellBlob(image);
     number_pixels=(MagickSizeType) tile_image->columns*tile_image->rows;
     if ((compression == FaxCompression) ||
+        (compression == Group4Compression) ||
         ((image_info->type != TrueColorType) &&
          (IsGrayImage(tile_image,&image->exception) != MagickFalse)))
       {
         switch (compression)
         {
           case FaxCompression:
+          case Group4Compression:
           {
             if (LocaleCompare(CCITTParam,"0") == 0)
-              (void) HuffmanEncodeImage(image_info,tile_image);
-            else
-              (void) Huffman2DEncodeImage(image_info,tile_image);
+              {
+                (void) HuffmanEncodeImage(image_info,tile_image);
+                break;
+              }
+            (void) Huffman2DEncodeImage(image_info,tile_image);
             break;
           }
           case JPEGCompression:
@@ -1911,7 +2079,7 @@ static MagickBooleanType WritePDFImage(const ImageInfo *image_info,Image *image)
     (void) WriteBlobString(image,buffer);
     (void) WriteBlobString(image,"<<\n");
     if ((image->storage_class != DirectClass) && (image->colors <= 256) &&
-        (compression != FaxCompression))
+        (compression != FaxCompression) && (compression != Group4Compression))
       {
         if (compression == NoCompression)
           (void) WriteBlobString(image,"/Filter [ /ASCII85Decode ]\n");
