@@ -78,6 +78,103 @@ static MagickBooleanType
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   I n v o k e P o s t s r i p t D e l e g a t e                             %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  InvokePostscriptDelegate() executes the postscript interpreter with the
+%  specified command.
+%
+%  The format of the InvokePostscriptDelegate method is:
+%
+%      MagickBooleanType InvokePostscriptDelegate(
+%        const MagickBooleanType verbose,const char *command)
+%
+%  A description of each parameter follows:
+%
+%    o verbose: A value other than zero displays the command prior to
+%      executing it.
+%
+%    o command: The address of a character string containing the command to
+%      execute.
+%
+*/
+static MagickBooleanType InvokePostscriptDelegate(
+  const MagickBooleanType verbose,const char *command)
+{
+#if defined(HasGS) || defined(__WINDOWS__)
+  char
+    **argv;
+
+  const GhostscriptVectors
+    *gs_func;
+
+  gs_main_instance
+    *interpreter;
+
+  int
+    argc,
+    code,
+    status;
+
+  register long
+    i;
+
+#if defined(__WINDOWS__)
+  gs_func=NTGhostscriptDLLVectors();
+#else
+  GhostscriptVectors
+    gs_func_struct;
+
+  gs_func=(&gs_func_struct);
+  gs_func_struct.exit=gsapi_exit;
+  gs_func_struct.init_with_args=gsapi_init_with_args;
+  gs_func_struct.new_instance=gsapi_new_instance;
+  gs_func_struct.run_string=gsapi_run_string;
+  gs_func_struct.delete_instance=gsapi_delete_instance;
+#endif
+  if (gs_func == (GhostscriptVectors *) NULL)
+    return(SystemCommand(verbose,command));
+  if (verbose != MagickFalse)
+    {
+      (void) fputs("[ghostscript library]",stdout);
+      (void) fputs(strchr(command,' '),stdout);
+    }
+  status=(gs_func->new_instance)(&interpreter,(void *) NULL);
+  if (status < 0)
+    return(MagickFalse);
+  argv=StringToArgv(command,&argc);
+  status=(gs_func->init_with_args)(interpreter,argc-1,argv+1);
+  if (status == 0)
+    status=(gs_func->run_string)
+      (interpreter,"systemdict /start get exec\n",0,&code);
+  (gs_func->exit)(interpreter);
+  (gs_func->delete_instance)(interpreter);
+#if defined(__WINDOWS__)
+  NTGhostscriptUnLoadDLL();
+#endif
+  for (i=0; i < (long) argc; i++)
+    argv[i]=(char *) RelinquishMagickMemory(argv[i]);
+  argv=(char **) RelinquishMagickMemory(argv);
+  if ((status == 0) || (status <= -100))
+    {
+      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
+        "Ghostscript returns status %d, exit code %d",status,code);
+      return(MagickFalse);
+    }
+  return(MagickTrue);
+#else
+  return(SystemCommand(verbose,command) != 0 ? MagickTrue : MagickFalse);
+#endif
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   I s P S                                                                   %
 %                                                                             %
 %                                                                             %
@@ -175,6 +272,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
 #define HiResBoundingBox  "HiResBoundingBox:"
 #define ImageData  "ImageData:"
 #define PageBoundingBox  "PageBoundingBox:"
+#define LanguageLevel  "LanguageLevel:"
 #define PageMedia  "PageMedia:"
 #define Pages  "Pages:"
 #define PhotoshopProfile  "BeginPhotoshop:"
@@ -187,6 +285,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     density[MaxTextExtent],
     filename[MaxTextExtent],
     geometry[MaxTextExtent],
+    input_filename[MaxTextExtent],
     options[MaxTextExtent],
     postscript_filename[MaxTextExtent],
     *q,
@@ -246,6 +345,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   unsigned long
     columns,
     height,
+    language_level,
     length,
     pages,
     rows,
@@ -269,6 +369,14 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image=DestroyImageList(image);
       return((Image *) NULL);
     }
+  status=AcquireUniqueSymbolicLink(image_info->filename,input_filename);
+  if (status == MagickFalse)
+    {
+      ThrowFileException(exception,FileOpenError,"UnableToCreateTemporaryFile",
+        image_info->filename);
+      image=DestroyImageList(image);
+      return((Image *) NULL);
+    }
   /*
     Initialize hex values.
   */
@@ -319,8 +427,9 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   rows=0;
   iptc_profile=(StringInfo *) NULL;
   xml_profile=(StringInfo *) NULL;
+  language_level=1;
   skip=MagickFalse;
-  cmyk=MagickFalse;
+  cmyk=image_info->colorspace == CMYKColorspace ? MagickTrue : MagickFalse;
   pages=(~0UL);
   p=command;
   for (c=ReadBlobByte(image); c != EOF; c=ReadBlobByte(image))
@@ -331,7 +440,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       Note document structuring comments.
     */
     *p++=(char) c;
-    if ((c != (int) '%') && (c != EOF) &&
+    if ((c != (int) '%') && (c != '\n') &&
         ((size_t) (p-command) < (MaxTextExtent/2)))
       continue;
     *p='\0';
@@ -348,6 +457,8 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (LocaleNCompare(PostscriptLevel,command,strlen(PostscriptLevel)) == 0)
       if (GlobExpression(command,"*EPSF-*") != MagickFalse)
         pages=1;
+    if (LocaleNCompare(LanguageLevel,command,strlen(LanguageLevel)) == 0)
+      (void) sscanf(command,LanguageLevel " %lu",&language_level);
     if (LocaleNCompare(Pages,command,strlen(Pages)) == 0)
       (void) sscanf(command,Pages " %lu",&pages);
     if (LocaleNCompare(ImageData,command,strlen(Pages)) == 0)
@@ -464,7 +575,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if (cmyk != MagickFalse)
       delegate_info=GetDelegateInfo("ps:cmyk",(char *) NULL,exception);
     else
-      if ((pages == 1) && ((image_info->channel & OpacityChannel) != 0))
+      if ((pages == 1) && (language_level > 2))
         delegate_info=GetDelegateInfo("ps:alpha",(char *) NULL,exception);
       else
         delegate_info=GetDelegateInfo("ps:color",(char *) NULL,exception);
@@ -495,7 +606,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     GetDelegateCommands(delegate_info),
     read_info->antialias != MagickFalse ? 4 : 1,
     read_info->antialias != MagickFalse ? 4 : 1,geometry,density,options,
-    read_info->filename,postscript_filename,image_info->filename);
+    read_info->filename,postscript_filename,input_filename);
   status=InvokePostscriptDelegate(read_info->verbose,command);
   image=ReadImage(read_info,exception);
   if (image == (Image *) NULL)
@@ -507,6 +618,7 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     }
   (void) RelinquishUniqueFileResource(postscript_filename);
   (void) RelinquishUniqueFileResource(read_info->filename);
+  (void) RelinquishUniqueFileResource(input_filename);
   if (image == (Image *) NULL)
     {
       /*
@@ -594,6 +706,7 @@ ModuleExport void RegisterPSImage(void)
   entry->adjoin=MagickFalse;
   entry->blob_support=MagickFalse;
   entry->seekable_stream=MagickTrue;
+  entry->thread_support=MagickFalse;
   entry->description=ConstantString(
    "Encapsulated PostScript Interchange format");
   entry->module=ConstantString("PS");
@@ -605,6 +718,7 @@ ModuleExport void RegisterPSImage(void)
   entry->adjoin=MagickFalse;
   entry->blob_support=MagickFalse;
   entry->seekable_stream=MagickTrue;
+  entry->thread_support=MagickFalse;
   entry->description=ConstantString("Encapsulated PostScript");
   entry->module=ConstantString("PS");
   (void) RegisterMagickInfo(entry);
@@ -625,6 +739,7 @@ ModuleExport void RegisterPSImage(void)
   entry->adjoin=MagickFalse;
   entry->blob_support=MagickFalse;
   entry->seekable_stream=MagickTrue;
+  entry->thread_support=MagickFalse;
   entry->description=ConstantString(
     "Encapsulated PostScript Interchange format");
   entry->module=ConstantString("PS");
@@ -636,6 +751,7 @@ ModuleExport void RegisterPSImage(void)
   entry->module=ConstantString("PS");
   entry->blob_support=MagickFalse;
   entry->seekable_stream=MagickTrue;
+  entry->thread_support=MagickFalse;
   entry->description=ConstantString("PostScript");
   (void) RegisterMagickInfo(entry);
 }
@@ -1061,7 +1177,8 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
     /*
       Scale relative to dots-per-inch.
     */
-    (void) SetImageColorspace(image,RGBColorspace);
+    if (image_info->colorspace == UndefinedColorspace)
+      (void) SetImageColorspace(image,RGBColorspace);
     delta.x=DefaultResolution;
     delta.y=DefaultResolution;
     flags=ParseGeometry(PSDensityGeometry,&geometry_info);
@@ -1098,7 +1215,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
       else
         if ((image->gravity != UndefinedGravity) &&
             (LocaleCompare(image_info->magick,"PS") == 0))
-          (void) strcpy(page_geometry,PSPageGeometry);
+          (void) CopyMagickString(page_geometry,PSPageGeometry,MaxTextExtent);
     (void) ParseMetaGeometry(page_geometry,&geometry.x,&geometry.y,
       &geometry.width,&geometry.height);
     scale.x=(double) (geometry.width*delta.x)/resolution.x;
@@ -1124,9 +1241,10 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
           Output Postscript header.
         */
         if (LocaleCompare(image_info->magick,"PS") == 0)
-          (void) strcpy(buffer,"%!PS-Adobe-3.0\n");
+          (void) CopyMagickString(buffer,"%!PS-Adobe-3.0\n",MaxTextExtent);
         else
-          (void) strcpy(buffer,"%!PS-Adobe-3.0 EPSF-3.0\n");
+          (void) CopyMagickString(buffer,"%!PS-Adobe-3.0 EPSF-3.0\n",
+            MaxTextExtent);
         (void) WriteBlobString(image,buffer);
         (void) WriteBlobString(image,"%%Creator: (ImageMagick)\n");
         (void) FormatMagickString(buffer,MaxTextExtent,"%%%%Title: (%s)\n",
@@ -1145,7 +1263,8 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
         bounds.y2=(double) geometry.y+(geometry.height+text_size);
         if ((image_info->adjoin != MagickFalse) &&
             (GetNextImageInList(image) != (Image *) NULL))
-          (void) strcpy(buffer,"%%%%BoundingBox: (atend)\n");
+          (void) CopyMagickString(buffer,"%%%%BoundingBox: (atend)\n",
+            MaxTextExtent);
         else
           {
             (void) FormatMagickString(buffer,MaxTextExtent,
@@ -1534,7 +1653,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
                 {
                   if ((image->matte != MagickFalse) &&
                       (p->opacity == TransparentOpacity))
-                    (void) strcpy(buffer,"ffffff");
+                    (void) CopyMagickString(buffer,"ffffff",MaxTextExtent);
                   else
                     (void) FormatMagickString(buffer,MaxTextExtent,
                       "%02X%02X%02X",ScaleQuantumToChar(p->red),
