@@ -395,14 +395,11 @@ static MagickBooleanType SerializeImageIndexes(const ImageInfo *image_info,
 }
 
 static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
-  Image *image)
+  Image *image,const CompressionType compression)
 {
   char
     buffer[MaxTextExtent];
 
-  CompressionType
-    compression;
-
   Image
     *mask_image;
 
@@ -430,7 +427,6 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   assert(image->matte != MagickFalse);
   status=MagickTrue;
-  compression=image->compression;
   /*
     Note BeginData DSC comment for update later.
   */
@@ -450,6 +446,7 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
       (void) FormatMagickString(buffer,MaxTextExtent,
         "currentfile %lu %lu "PS3_NoCompression" ByteStreamDecodeFilter\n",
         image->columns,image->rows);
+      break;
     }
     case FaxCompression:
     case Group4Compression:
@@ -522,7 +519,8 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
     case Group4Compression:
     default:
     {
-      if (LocaleCompare(CCITTParam,"0") == 0)
+      if ((compression == FaxCompression) ||
+          (LocaleCompare(CCITTParam,"0") == 0))
         status=HuffmanEncodeImage(image_info,mask_image);
       else
         status=Huffman2DEncodeImage(image_info,mask_image);
@@ -714,13 +712,11 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       "/DisplayImage",
       "{",
       "  /buffer 512 string def",
-      "",
       "  % Translation.",
       "  currentfile buffer readline pop",
       "  token pop /x exch def",
       "  token pop /y exch def pop",
       "  x y translate",
-      "",
       "  % Image size and font size.",
       "  currentfile buffer readline pop",
       "  token pop /x exch def",
@@ -735,7 +731,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       "  % Clipping path.",
       "  currentfile buffer readline pop",
       "  token pop /clipped exch def pop",
-      "  % EPS.",
+      "  % Showpage.",
       "  currentfile buffer readline pop",
       "  token pop /sp exch def pop",
       "  % Image pixel size.",
@@ -850,6 +846,14 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
   compression=image->compression;
   switch (compression)
   {
+    case FaxCompression:
+    case Group4Compression:
+    { 
+      if ((IsMonochromeImage(image,&image->exception) == MagickFalse) ||
+          (image->matte != MagickFalse))
+        compression=RLECompression;
+      break;
+    }
 #if !defined(HasJPEG)
     case JPEGCompression:
     {
@@ -916,7 +920,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       else
         if ((image->gravity != UndefinedGravity) &&
             (LocaleCompare(image_info->magick,"PS") == 0))
-          (void) strcpy(page_geometry,PSPageGeometry);
+          (void) CopyMagickString(page_geometry,PSPageGeometry,MaxTextExtent);
     (void) ParseMetaGeometry(page_geometry,&geometry.x,&geometry.y,
       &geometry.width,&geometry.height);
     scale.x=(double) (geometry.width*delta.x)/resolution.x;
@@ -942,9 +946,10 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
           Postscript header on the first page.
         */
         if (LocaleCompare(image_info->magick,"PS3") == 0)
-          (void) strcpy(buffer,"%!PS-Adobe-3.0\n");
+          (void) CopyMagickString(buffer,"%!PS-Adobe-3.0\n",MaxTextExtent);
         else
-          (void) strcpy(buffer,"%!PS-Adobe-3.0 EPSF-3.0\n");
+          (void) CopyMagickString(buffer,"%!PS-Adobe-3.0 EPSF-3.0\n",
+            MaxTextExtent);
         (void) WriteBlobString(image,buffer);
         (void) FormatMagickString(buffer,MaxTextExtent,
           "%%%%Creator: ImageMagick %s\n",MagickLibVersionText);
@@ -1005,7 +1010,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
             (void) WriteBlobString(image,"%%Orientation: Portrait\n");
             (void) WriteBlobString(image,"%%PageOrder: Ascend\n");
             if (image_info->adjoin == MagickFalse)
-              (void) strcpy(buffer,"%%Pages: 1\n");
+              (void) CopyMagickString(buffer,"%%Pages: 1\n",MaxTextExtent);
             else
               (void) FormatMagickString(buffer,MaxTextExtent,"%%%%Pages: %lu\n",
                 (unsigned long) GetImageListLength(image));
@@ -1088,7 +1093,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     */
     if ((image->clip_mask == (Image *) NULL) ||
         (LocaleNCompare("8BIM:",image->clip_mask->magick_filename,5) != 0))
-      (void)WriteBlobString(image,"/ClipImage {} def\n");
+      (void) WriteBlobString(image,"/ClipImage {} def\n");
     else
       {
         const ImageAttribute
@@ -1109,7 +1114,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       Image mask.
     */
     if ((image->matte != MagickFalse) &&
-        (WritePS3MaskImage(image_info,image) == MagickFalse))
+        (WritePS3MaskImage(image_info,image,compression) == MagickFalse))
       {
         CloseBlob(image);
         return(MagickFalse);
@@ -1167,13 +1172,11 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
         (void) WriteBlobString(image,"true\n");
       else
         (void) WriteBlobString(image,"false\n");
-    if ((compression == FaxCompression) || (compression == Group4Compression))
-      (void) SetImageType(image,BilevelType);
     /*
       Showpage for non-EPS.
     */
-    (void) WriteBlobString(image,
-      LocaleCompare(image_info->magick,"PS3") == 0 ?  "true\n" : "false\n");
+    (void) WriteBlobString(image, LocaleCompare(image_info->magick,"PS3") == 0 ?
+      "true\n" : "false\n");
     /*
       Image columns, rows, and color space.
     */
@@ -1190,8 +1193,8 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       Render with imagemask operator?
     */
     option=GetImageOption(image_info,"ps3:imagemask");
-    (void) WriteBlobString(image,(option != (const char *) NULL) &&
-      (IsMonochromeImage(image,&image->exception) != MagickFalse) ?
+    (void) WriteBlobString(image,((option != (const char *) NULL) &&
+      (IsMonochromeImage(image,&image->exception) != MagickFalse)) ?
       "true\n" : "false\n");
     /*
       Output pixel data.
@@ -1282,8 +1285,11 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
               case NoCompression:
               default:
               {
+                Ascii85Initialize(image);
                 for (i=0; i < (long) length; i++)
-                  (void) WriteBlobByte(image,pixels[i]);
+                  Ascii85Encode(image,pixels[i]);
+                Ascii85Flush(image);
+                status=MagickTrue;
                 break;
               }
               case LZWCompression:
