@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -154,8 +154,8 @@ ModuleExport void RegisterTILEImage(void)
   entry->decoder=(DecoderHandler *) ReadTILEImage;
   entry->raw=MagickTrue;
   entry->endian_support=MagickTrue;
-  entry->description=AcquireString("Tile image with a texture");
-  entry->module=AcquireString("TILE");
+  entry->description=ConstantString("Tile image with a texture");
+  entry->module=ConstantString("TILE");
   (void) RegisterMagickInfo(entry);
 }
 
