@@ -12,7 +12,7 @@
 %                              December 1998                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -33,11 +33,13 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/attribute.h"
-#include "magick/cache.h"
-#include "magick/gem.h"
-#include "magick/string_.h"
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <time.h>
+#include <assert.h>
+#include <math.h>
+#include "magick/MagickCore.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -50,7 +52,7 @@
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  AnalyzeImage(0 computes the brightness and saturation mean and standard
+%  AnalyzeImage() computes the brightness and saturation mean and standard
 %  deviation and stores these values as attributes of the image.
 %
 %  The format of the AnalyzeImage method is:
@@ -62,10 +64,14 @@
 %    o image: The address of a structure of type Image.
 %
 */
-#define PRECISION "%.0f"
 ModuleExport MagickBooleanType AnalyzeImage(Image **image,const int argc,
   char **argv)
 {
+#define PRECISION "%.0f"
+
+  char
+    text[MaxTextExtent];
+
   double
     bsumX = 0.0,
     bsumX2 = 0.0,
@@ -85,87 +91,47 @@ ModuleExport MagickBooleanType AnalyzeImage(Image **image,const int argc,
   long
     y;
 
-  register long
-    x;
-
-  register PixelPacket
+  register const PixelPacket
     *p;
 
-  char
-    text[MaxTextExtent];
+  register long
+    x;
 
   assert(image != (Image **) NULL);
   assert(*image != (Image *) NULL);
   for (y=0; y < (int) (*image)->rows; y++)
   {
-    p=GetImagePixels((*image),0,y,(*image)->columns,1);
-    if (p == (PixelPacket *) NULL)
+    p=AcquireImagePixels((*image),0,y,(*image)->columns,1,&(*image)->exception);
+    if (p == (const PixelPacket *) NULL)
       break;
-    if (y == 0)
-      {
-        (void) FormatMagickString(text,MaxTextExtent,"#%02x%02x%02x",
-          p->red,p->green,p->blue);
-        (void) SetImageAttribute((*image),"TopLeftColor",text);
-      }
-    if (y == (long) ((*image)->rows-1))
-      {
-        (void) FormatMagickString(text,MaxTextExtent,"#%02x%02x%02x",
-          p->red,p->green,p->blue);
-        (void) SetImageAttribute((*image),"BottomLeftColor",text);
-      }
     for (x=0; x < (long) (*image)->columns; x++)
     {
-      TransformHSL(p->red,p->green,p->blue,&hue,&saturation,&brightness);
-      brightness *= QuantumRange;
-            bsumX += brightness;
-            bsumX2 += brightness * brightness;
-      saturation *= QuantumRange;
-            ssumX += saturation;
-            ssumX2 += saturation * saturation;
+      TransformHSB(p->red,p->green,p->blue,&hue,&saturation,&brightness);
+      brightness*=QuantumRange;
+      bsumX+=brightness;
+      bsumX2+=brightness*brightness;
+      saturation*=QuantumRange;
+      ssumX+=saturation;
+      ssumX2+=saturation*saturation;
       total_pixels++;
       p++;
     }
-    p--; /* backup one pixel to allow us to sample */
-    if (y == 0)
-      {
-        (void) FormatMagickString(text,MaxTextExtent,"#%02x%02x%02x",
-          p->red,p->green,p->blue);
-        (void) SetImageAttribute((*image),"TopRightColor",text);
-      }
-    if (y == (long) ((*image)->rows-1))
-      {
-        (void) FormatMagickString(text,MaxTextExtent,"#%02x%02x%02x",
-          p->red,p->green,p->blue);
-        (void) SetImageAttribute((*image),"BottomRightColor",text);
-      }
   }
-  if (total_pixels > 0.0)
-    {
-      brightness_mean = bsumX/total_pixels;
-      (void) FormatMagickString(text,MaxTextExtent,PRECISION,brightness_mean);
-      (void) SetImageAttribute((*image),"BrightnessMean",text);
-      /*  This formula gives a slightly biased result */
-      brightness_stdev =
-          sqrt(bsumX2/total_pixels - (bsumX/total_pixels*bsumX/total_pixels));
-      (void) FormatMagickString(text,MaxTextExtent,PRECISION,brightness_stdev);
-      (void) SetImageAttribute((*image),"BrightnessStddev",text);
-      /* Now the correction for bias. */
-      /*  stdev = stdev*sqrt((double)total_pixels/(double)(total_pixels-1)); */
-      /* Now calculate the standard deviation of the mean */
-      /*  brightness_stdevmean = bstdev/sqrt((double)total_pixels); */
-
-      saturation_mean = ssumX/total_pixels;
-      (void) FormatMagickString(text,MaxTextExtent,PRECISION,saturation_mean);
-      (void) SetImageAttribute((*image),"SaturationMean",text);
-      /* This formula gives a slightly biased result */
-      saturation_stdev =sqrt(ssumX2/total_pixels -
-        (ssumX/total_pixels*ssumX/total_pixels));
-      (void) FormatMagickString(text,MaxTextExtent,PRECISION,saturation_stdev);
-      (void) SetImageAttribute((*image),"SaturationStddev",text);
-      /* Now the correction for bias. */
-      /*  stdev = stdev*sqrt((double)total_pixels/(double)(total_pixels-1)); */
-      /* Now calculate the standard deviation of the mean */
-      /*  saturation_stdevmean = sstdev/sqrt((double)total_pixels); */
-    }
+  if (total_pixels <= 0.0)
+    return(MagickFalse);
+  brightness_mean=bsumX/total_pixels;
+  (void) FormatMagickString(text,MaxTextExtent,PRECISION,brightness_mean);
+  (void) SetImageAttribute((*image),"BrightnessMean",text);
+  brightness_stdev=sqrt(bsumX2/total_pixels-(bsumX/total_pixels*bsumX/
+    total_pixels));
+  (void) FormatMagickString(text,MaxTextExtent,PRECISION,brightness_stdev);
+  (void) SetImageAttribute((*image),"BrightnessStddev",text);
+  saturation_mean=ssumX/total_pixels;
+  (void) FormatMagickString(text,MaxTextExtent,PRECISION,saturation_mean);
+  (void) SetImageAttribute((*image),"SaturationMean",text);
+  saturation_stdev=sqrt(ssumX2/total_pixels-(ssumX/total_pixels*ssumX/
+    total_pixels));
+  (void) FormatMagickString(text,MaxTextExtent,PRECISION,saturation_stdev);
+  (void) SetImageAttribute((*image),"SaturationStddev",text);
   return(MagickTrue);
 }
