@@ -189,30 +189,28 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
     }
     case GRAYColorspace:
     {
-      IndexPacket
-        index;
-
       /*
         Convert RGB to GRAY colorspace.
       */
       if (IsGrayImage(image,&image->exception) != MagickFalse)
         return(MagickTrue);
-      if (AllocateImageColormap(image,MaxColormapSize) == MagickFalse)
-        ThrowBinaryException(ResourceLimitError,"MemoryAllocationFailed",
-          image->filename);
+      if (image->storage_class == PseudoClass)
+        for (i=0; i < (long) image->colors; i++)
+        {
+          image->colormap[i].red=PixelIntensityToQuantum(image->colormap+i);
+          image->colormap[i].green=image->colormap[i].red;
+          image->colormap[i].blue=image->colormap[i].red;
+        }
       for (y=0; y < (long) image->rows; y++)
       {
         q=GetImagePixels(image,0,y,image->columns,1);
         if (q == (PixelPacket *) NULL)
           break;
-        indexes=GetIndexes(image);
         for (x=0; x < (long) image->columns; x++)
         {
-          index=PixelIntensityToQuantum(q);
-          q->red=index;
-          q->green=index;
-          q->blue=index;
-          indexes[x]=index;
+          q->red=PixelIntensityToQuantum(q);
+          q->green=q->red;
+          q->blue=q->red;
           q++;
         }
         if (SyncImagePixels(image) == MagickFalse)
@@ -256,7 +254,6 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
         if (SyncImagePixels(image) == MagickFalse)
           break;
       }
-      image->colorspace=RGBColorspace;
       return(MagickTrue);
     }
     case HSLColorspace:
@@ -295,7 +292,6 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
         if (SyncImagePixels(image) == MagickFalse)
           break;
       }
-      image->colorspace=RGBColorspace;
       return(MagickTrue);
     }
     case HWBColorspace:
@@ -334,7 +330,6 @@ MagickExport MagickBooleanType RGBTransformImage(Image *image,
         if (SyncImagePixels(image) == MagickFalse)
           break;
       }
-      image->colorspace=RGBColorspace;
       return(MagickTrue);
     }
     case LogColorspace:
@@ -898,6 +893,9 @@ MagickExport MagickBooleanType SetImageColorspace(Image *image,
 MagickExport MagickBooleanType TransformRGBImage(Image *image,
   const ColorspaceType colorspace)
 {
+#define D50X  (0.9642)
+#define D50Y  (1.0)
+#define D50Z  (0.8249)
 #define MaxYCC  ((MagickRealType) (1.3584*MaxMap+0.5))
 #define TransformRGBImageTag  "Transform/Image"
 
@@ -1043,6 +1041,8 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
           break;
       }
       image->colorspace=RGBColorspace;
+      if (SyncImageCache(image) == MagickFalse)
+        return(MagickFalse);
       return(y == (long) image->rows ? MagickTrue : MagickFalse);
     }
     case HSBColorspace:
@@ -1153,6 +1153,34 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
       image->colorspace=RGBColorspace;
       return(y == (long) image->rows ? MagickTrue : MagickFalse);
     }
+    case LABColorspace:
+    {
+      /*
+        Transform image from LAB to XYZ.
+          * * * not implemented * * *
+      */
+      if (image->storage_class == PseudoClass)
+        {
+          if (SyncImage(image) == MagickFalse)
+            return(MagickFalse);
+          if (SetImageStorageClass(image,DirectClass) == MagickFalse)
+            return(MagickFalse);
+        }
+      for (y=0; y < (long) image->rows; y++)
+      {
+        q=GetImagePixels(image,0,y,image->columns,1);
+        if (q == (PixelPacket *) NULL)
+          break;
+        for (x=0; x < (long) image->columns; x++)
+        {
+          q++;
+        }
+        if (SyncImagePixels(image) == MagickFalse)
+          break;
+      }
+      image->colorspace=RGBColorspace;
+      break;
+    }
     case LogColorspace:
     {
       const ImageAttribute
@@ -1343,6 +1371,7 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
       }
       break;
     }
+    case LABColorspace:
     case XYZColorspace:
     {
       /*
@@ -1623,8 +1652,6 @@ MagickExport MagickBooleanType TransformRGBImage(Image *image,
     }
   }
   image->colorspace=RGBColorspace;
-  if (SyncImageCache(image) == MagickFalse)
-    return(MagickFalse);
   /*
     Free resources.
   */
