@@ -64,6 +64,7 @@
 #include "magick/stream.h"
 #include "magick/string_.h"
 #include "magick/timer.h"
+#include "magick/transform.h"
 #include "magick/utility.h"
 
 static SemaphoreInfo
@@ -212,7 +213,6 @@ MagickExport void DestroyConstitute(void)
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 
 #if defined(__cplusplus) || defined(c_plusplus)
@@ -295,7 +295,7 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
     magick_filename[MaxTextExtent];
 
   const char
-    *option;
+    *value;
 
   const DelegateInfo
     *delegate_info;
@@ -352,8 +352,6 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
         status;
 
       image=AllocateImage(read_info);
-      if (image == (Image *) NULL)
-        return((Image *) NULL);
       (void) CopyMagickString(image->filename,read_info->filename,
         MaxTextExtent);
       status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
@@ -427,14 +425,21 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       if ((magick_info == (const MagickInfo *) NULL) ||
           (GetMagickDecoder(magick_info) == (DecoderHandler *) NULL))
         {
-          if (IsAccessible(read_info->filename) == MagickFalse)
-            (void) ThrowMagickException(exception,GetMagickModule(),
-              FileOpenError,"UnableToOpenFile","`%s': %s",read_info->filename,
-              strerror(errno));
-          else
+          if (IsAccessible(read_info->filename) != MagickFalse)
             (void) ThrowMagickException(exception,GetMagickModule(),
               MissingDelegateError,"NoDecodeDelegateForThisImageFormat","`%s'",
               read_info->filename);
+          else
+            {
+              char
+                *message;
+
+              message=GetExceptionMessage(errno);
+              (void) ThrowMagickException(exception,GetMagickModule(),
+                FileOpenError,"UnableToOpenFile","`%s': %s",read_info->filename,
+                message);
+              message=(char *) RelinquishMagickMemory(message);
+            }
           read_info=DestroyImageInfo(read_info);
           return((Image *) NULL);
         }
@@ -480,9 +485,14 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
     }
   if (GetBlobError(image) != MagickFalse)
     {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
       (void) ThrowMagickException(exception,GetMagickModule(),CorruptImageError,
         "AnErrorHasOccurredReadingFromFile","`%s': %s",read_info->filename,
-        strerror(errno));
+        message);
+      message=(char *) RelinquishMagickMemory(message);
       image=DestroyImageList(image);
       read_info=DestroyImageInfo(read_info);
       return((Image *) NULL);
@@ -495,6 +505,13 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       MaxTextExtent);
     if (IsBlobTemporary(image) != MagickFalse)
       (void) CopyMagickString(next->filename,filename,MaxTextExtent);
+    attribute=GetImageAttribute(next,"EXIF:Orientation");
+    if ((attribute != (const ImageAttribute *) NULL) &&
+        (attribute->value != (char *) NULL))
+      {
+        next->orientation=(OrientationType) atol(attribute->value);
+        (void) DeleteImageAttribute(next,"EXIF:Orientation");
+      }
     attribute=GetImageAttribute(next,"EXIF:XResolution");
     if ((attribute != (const ImageAttribute *) NULL) &&
         (attribute->value != (char *) NULL))
@@ -530,16 +547,16 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
         (void) DeleteImageAttribute(next,"EXIF:ResolutionUnit");
       }
     (void) SyncImageOptions(image_info,next);
-    option=GetImageOption(read_info,"comment");
-    if (option != (const char *) NULL)
+    value=GetImageOption(read_info,"comment");
+    if (value != (const char *) NULL)
       {
         (void) DeleteImageAttribute(next,"comment");
-        (void) SetImageAttribute(next,"comment",option);
+        (void) SetImageAttribute(next,"comment",value);
       }
-    option=GetImageOption(read_info,"delay");
-    if (option != (const char *) NULL)
+    value=GetImageOption(read_info,"delay");
+    if (value != (const char *) NULL)
       {
-        flags=ParseGeometry(option,&geometry_info);
+        flags=ParseGeometry(value,&geometry_info);
         if ((flags & GreaterValue) != 0)
           {
             if (next->delay > (unsigned long) (geometry_info.rho+0.5))
@@ -557,40 +574,58 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
         if ((flags & SigmaValue) != 0)
           next->ticks_per_second=(unsigned long) (geometry_info.sigma+0.5);
       }
-    option=GetImageOption(read_info,"dispose");
-    if (option != (const char *) NULL)
+    value=GetImageOption(read_info,"dispose");
+    if (value != (const char *) NULL)
       next->dispose=(DisposeType) ParseMagickOption(MagickDisposeOptions,
-        MagickFalse,option);
-    option=GetImageOption(read_info,"label");
-    if (option != (const char *) NULL)
+        MagickFalse,value);
+    value=GetImageOption(read_info,"label");
+    if (value != (const char *) NULL)
       {
         (void) DeleteImageAttribute(next,"label");
-        (void) SetImageAttribute(next,"label",option);
+        (void) SetImageAttribute(next,"label",value);
       }
     if (next->page.width == 0)
       next->page.width=next->columns;
     if (next->page.height == 0)
       next->page.height=next->rows;
-    option=GetImageOption(read_info,"page");
-    if ((option != (const char *) NULL) &&
+    value=GetImageOption(read_info,"page");
+    if ((value != (const char *) NULL) &&
         (LocaleCompare(next->magick,"TEXT") != 0))
       {
         char
           *geometry;
 
-        geometry=GetPageGeometry(option);
+        geometry=GetPageGeometry(value);
         flags=ParseAbsoluteGeometry(geometry,&next->page);
         geometry=(char *) RelinquishMagickMemory(geometry);
       }
+    if (read_info->extract != (char *) NULL)
+      {
+        RectangleInfo
+          geometry;
+
+        flags=ParseSizeGeometry(next,read_info->extract,&geometry);
+        if (((flags & WidthValue) != 0) || ((flags & HeightValue) != 0))
+          {
+            Image
+              *size_image;
+
+            size_image=ResizeImage(next,geometry.width,geometry.height,
+              next->filter,next->blur,exception);
+            if (size_image != (Image *) NULL)
+              ReplaceImageInList(&next,size_image);
+          }
+      }
     if (next->magick_columns == 0)
       next->magick_columns=next->columns;
     if (next->magick_rows == 0)
       next->magick_rows=next->rows;
     if (read_info->verbose != MagickFalse)
       (void) IdentifyImage(next,stdout,MagickFalse);
+    image=next;
   }
   read_info=DestroyImageInfo(read_info);
-  return(image);
+  return(GetFirstImageInList(image));
 }
 
 /*
@@ -855,8 +890,8 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
     }
   (void) CopyMagickString(filename,image->filename,MaxTextExtent);
   SyncImageInfo(write_info,image,&sans_exception);
-  if ((GetNextImageInList(image) == (Image *) NULL) &&
-      (GetPreviousImageInList(image) == (Image *) NULL) &&
+  if ((GetPreviousImageInList(image) == (Image *) NULL) &&
+      (GetNextImageInList(image) == (Image *) NULL) &&
       (write_info->page == (char *) NULL) &&
       (IsTaintImage(image) == MagickFalse))
     {
@@ -961,9 +996,16 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
         }
     }
   if (GetBlobError(image) != MagickFalse)
-    (void) ThrowMagickException(&image->exception,GetMagickModule(),
-      CorruptImageError,"AnErrorHasOccurredWritingToFile","`%s': %s",
-        image->filename,strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      (void) ThrowMagickException(&image->exception,GetMagickModule(),
+        CorruptImageError,"AnErrorHasOccurredWritingToFile","`%s': %s",
+          image->filename,message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
   if (temporary == MagickTrue)
     {
       /*
