@@ -46,6 +46,7 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/client.h"
 #include "magick/configure.h"
@@ -273,7 +274,7 @@ MagickExport char *GetDelegateCommand(const ImageInfo *image_info,Image *image,
         decode ? decode : encode);
       return((char *) NULL);
     }
-  command=TranslateText(image_info,image,commands[0]);
+  command=InterpretImageAttributes(image_info,image,commands[0]);
   if (command == (char *) NULL)
     (void) ThrowMagickException(exception,GetMagickModule(),ResourceLimitError,
       "MemoryAllocationFailed","`%s'",commands[0]);
@@ -698,14 +699,77 @@ static MagickBooleanType InitializeDelegateList(ExceptionInfo *exception)
 %
 %
 */
+
+static void CopyDelegateFile(const char *source,const char *destination)
+{
+  int
+    destination_file,
+    source_file,
+    status;
+
+  size_t
+    length;
+
+  ssize_t
+    count;
+
+  struct stat
+    file_info;
+
+  unsigned char
+    *buffer;
+
+  /*
+    Return if destination file already exists and is not empty.
+  */
+  assert(source != (const char *) NULL);
+  assert(destination != (char *) NULL);
+  status=stat(destination,&file_info);
+  if (file_info.st_size != 0)
+    return;
+  /*
+    Copy source file to destination.
+  */
+  destination_file=open(destination,O_WRONLY | O_BINARY);
+  if (destination_file == -1)
+    return;
+  source_file=open(source,O_RDONLY | O_BINARY);
+  if (source_file == -1)
+    {
+      (void) close(destination_file);
+      return;
+    }
+  buffer=(unsigned char *) AcquireMagickMemory(MagickMaxBufferSize*
+    sizeof(*buffer));
+  if (buffer == (unsigned char *) NULL)
+    {
+      (void) close(source_file);
+      (void) close(destination_file);
+      return;
+    }
+  for (length=0; ; )
+  {
+    count=read(source_file,buffer,MagickMaxBufferSize);
+    if (count <= 0)
+      break;
+    length=(size_t) count;
+    count=write(destination_file,buffer,length);
+    if ((size_t) count != length)
+      break;
+  }
+  (void) close(destination_file);
+  (void) close(source_file);
+  buffer=(unsigned char *) RelinquishMagickMemory(buffer);
+}
+
 MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
   Image *image,const char *decode,const char *encode,ExceptionInfo *exception)
 {
-#define ProhibitedAlphabet  "*?\"'<>|`"
-
   char
     *command,
-    **commands;
+    **commands,
+    input_filename[MaxTextExtent],
+    output_filename[MaxTextExtent];
 
   const DelegateInfo
     *delegate_info;
@@ -773,13 +837,6 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
       /*
         Delegate requires a particular image format.
       */
-      if ((strpbrk(image_info->filename,ProhibitedAlphabet) != (char *) NULL) ||
-          (strpbrk(image->filename,ProhibitedAlphabet) != (char *) NULL))
-        {
-          ThrowFileException(exception,FileOpenError,
-            "FilenameContainsProhibitedCharacters",image->filename);
-          return(MagickFalse);
-        }
       if (AcquireUniqueFilename(image_info->unique) == MagickFalse)
         {
           ThrowFileException(exception,FileOpenError,
@@ -793,7 +850,7 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
             "UnableToCreateTemporaryFile",image_info->zero);
           return(MagickFalse);
         }
-      magick=TranslateText(image_info,image,decode != (char *) NULL ?
+      magick=InterpretImageAttributes(image_info,image,decode != (char *) NULL ?
         delegate_info->encode : delegate_info->decode);
       if (magick == (char *) NULL)
         {
@@ -856,14 +913,15 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
     }
   command=(char *) NULL;
   status=MagickFalse;
+  (void) CopyMagickString(output_filename,image_info->filename,MaxTextExtent);
+  (void) CopyMagickString(input_filename,image->filename,MaxTextExtent);
   for (i=0; commands[i] != (char *) NULL; i++)
   {
-    status=MagickFalse;
-    if ((strpbrk(image_info->filename,ProhibitedAlphabet) != (char *) NULL) ||
-        (strpbrk(image->filename,ProhibitedAlphabet) != (char *) NULL))
+    status=AcquireUniqueSymbolicLink(output_filename,image_info->filename);
+    if (status == MagickFalse)
       {
         ThrowFileException(exception,FileOpenError,
-          "FilenameContainsProhibitedCharacters",image->filename);
+          "UnableToCreateTemporaryFile",output_filename);
         break;
       }
     if (AcquireUniqueFilename(image_info->unique) == MagickFalse)
@@ -879,7 +937,14 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
           "UnableToCreateTemporaryFile",image_info->zero);
         break;
       }
-    command=TranslateText(image_info,image,commands[i]);
+    status=AcquireUniqueSymbolicLink(input_filename,image->filename);
+    if (status == MagickFalse)
+      {
+        ThrowFileException(exception,FileOpenError,
+          "UnableToCreateTemporaryFile",input_filename);
+        break;
+      }
+    command=InterpretImageAttributes(image_info,image,commands[i]);
     if (command == (char *) NULL)
       break;
     /*
@@ -887,11 +952,14 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
     */
     if (delegate_info->spawn != MagickFalse)
       (void) ConcatenateString(&command," &");
-    status=SystemCommand(image_info->verbose,command) != 0 ?
-      MagickTrue : MagickFalse;
+    status=SystemCommand(image_info->verbose,command) != 0 ?  MagickTrue :
+      MagickFalse;
     command=(char *) RelinquishMagickMemory(command);
+    (void) RelinquishUniqueFileResource(image->filename);
     (void) RelinquishUniqueFileResource(image_info->unique);
     (void) RelinquishUniqueFileResource(image_info->zero);
+    CopyDelegateFile(image_info->filename,output_filename);
+    (void) RelinquishUniqueFileResource(image_info->filename);
     if (status != MagickFalse)
       {
         (void) ThrowMagickException(exception,GetMagickModule(),DelegateError,
@@ -900,6 +968,8 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
       }
     commands[i]=(char *) RelinquishMagickMemory(commands[i]);
   }
+  (void) CopyMagickString(image_info->filename,output_filename,MaxTextExtent);
+  (void) CopyMagickString(image->filename,input_filename,MaxTextExtent);
   /*
     Free resources.
   */
@@ -908,105 +978,7 @@ MagickExport MagickBooleanType InvokeDelegate(ImageInfo *image_info,
   commands=(char **) RelinquishMagickMemory(commands);
   if (temporary != MagickFalse)
     (void) RelinquishUniqueFileResource(image->filename);
-  return((MagickBooleanType) (status == MagickFalse));
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%   I n v o k e P o s t s r i p t D e l e g a t e                             %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  InvokePostscriptDelegate() executes the postscript interpreter with the
-%  specified command.
-%
-%  The format of the InvokePostscriptDelegate method is:
-%
-%      MagickBooleanType InvokePostscriptDelegate(
-%        const MagickBooleanType verbose,const char *command)
-%
-%  A description of each parameter follows:
-%
-%    o verbose: A value other than zero displays the command prior to
-%      executing it.
-%
-%    o command: The address of a character string containing the command to
-%      execute.
-%
-%
-*/
-MagickExport MagickBooleanType InvokePostscriptDelegate(
-  const MagickBooleanType verbose,const char *command)
-{
-#if defined(HasGS) || defined(__WINDOWS__)
-  char
-    **argv;
-
-  const GhostscriptVectors
-    *gs_func;
-
-  gs_main_instance
-    *interpreter;
-
-  int
-    argc,
-    code,
-    status;
-
-  register long
-    i;
-
-#if defined(__WINDOWS__)
-  gs_func=NTGhostscriptDLLVectors();
-#else
-  GhostscriptVectors
-    gs_func_struct;
-
-  gs_func=(&gs_func_struct);
-  gs_func_struct.exit=gsapi_exit;
-  gs_func_struct.init_with_args=gsapi_init_with_args;
-  gs_func_struct.new_instance=gsapi_new_instance;
-  gs_func_struct.run_string=gsapi_run_string;
-  gs_func_struct.delete_instance=gsapi_delete_instance;
-#endif
-  if (gs_func == (GhostscriptVectors *) NULL)
-    return(SystemCommand(verbose,command));
-  if (verbose != MagickFalse)
-    {
-      (void) fputs("[ghostscript library]",stdout);
-      (void) fputs(strchr(command,' '),stdout);
-    }
-  status=(gs_func->new_instance)(&interpreter,(void *) NULL);
-  if (status < 0)
-    return(MagickFalse);
-  argv=StringToArgv(command,&argc);
-  status=(gs_func->init_with_args)(interpreter,argc-1,argv+1);
-  if (status == 0)
-    status=(gs_func->run_string)
-      (interpreter,"systemdict /start get exec\n",0,&code);
-  (gs_func->exit)(interpreter);
-  (gs_func->delete_instance)(interpreter);
-#if defined(__WINDOWS__)
-  NTGhostscriptUnLoadDLL();
-#endif
-  for (i=0; i < (long) argc; i++)
-    argv[i]=(char *) RelinquishMagickMemory(argv[i]);
-  argv=(char **) RelinquishMagickMemory(argv);
-  if ((status == 0) || (status <= -100))
-    {
-      (void) LogMagickEvent(CoderEvent,GetMagickModule(),
-        "Ghostscript returns status %d, exit code %d",status,code);
-      return(MagickFalse);
-    }
-  return(MagickTrue);
-#else
-  return(SystemCommand(verbose,command) != 0 ? MagickTrue : MagickFalse);
-#endif
+  return(status == MagickFalse ? MagickTrue : MagickFalse);
 }
 
 /*
