@@ -402,7 +402,7 @@ MagickExport MagickBooleanType IsGeometry(const char *geometry)
   if (geometry == (const char *) NULL)
     return(MagickFalse);
   flags=ParseGeometry(geometry,&geometry_info);
-  return((MagickBooleanType) (flags != NoValue));
+  return(flags != NoValue ? MagickTrue : MagickFalse);
 }
 
 /*
@@ -628,7 +628,8 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
   (void) strtod(p,&q);
   if (LocaleNCompare(p,"0x",2) == 0)
     (void) strtol(p,&q,10);
-  if ((*q == 'x') || (*q == 'X') || (*q == '/') || (*q == ',') || (*q =='\0'))
+  if ((*q == 'x') || (*q == 'X') || (*q == ',') || (*q == '/') ||
+      (*q == ':') || (*q =='\0'))
     {
       /*
         Parse rho.
@@ -645,7 +646,7 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
         }
     }
   q=p;
-  if ((*p == 'x') || (*p == 'X') || (*p == ',') || (*p == '/'))
+  if ((*p == 'x') || (*p == 'X') || (*p == ',') || (*p == '/') || (*p == ':'))
     {
       /*
         Parse sigma.
@@ -666,7 +667,7 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
     }
   while (isspace((int) ((unsigned char) *p)) != 0)
     p++;
-  if ((*p == '+') || (*p == '-') || (*p == ',') || (*p == '/'))
+  if ((*p == '+') || (*p == '-') || (*p == ',') || (*p == '/') || (*p == ':'))
     {
       /*
         Parse xi value.
@@ -684,7 +685,8 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
         }
       while (isspace((int) ((unsigned char) *p)) != 0)
         p++;
-      if ((*p == '+') || (*p == '-') || (*p == ',') || (*p == '/'))
+      if ((*p == '+') || (*p == '-') || (*p == ',') || (*p == '/') ||
+          (*p == ':'))
         {
           /*
             Parse psi value.
@@ -703,7 +705,8 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
         }
       while (isspace((int) ((unsigned char) *p)) != 0)
         p++;
-      if ((*p == '+') || (*p == '-') || (*p == ',') || (*p == '/'))
+      if ((*p == '+') || (*p == '-') || (*p == ',') || (*p == '/') ||
+          (*p == ':'))
         {
           /*
             Parse chi value.
@@ -721,6 +724,16 @@ MagickExport MagickStatusType ParseGeometry(const char *geometry,
             }
         }
     }
+  if (strchr(pedantic_geometry,':') != (char *) NULL)
+    {
+      /*
+        Normalize sampling factor (e.g. 4:2:2 => 2x1).
+      */
+      geometry_info->rho*=geometry_info->sigma;
+      geometry_info->sigma=1.0;
+      if (geometry_info->xi == 0.0)
+        geometry_info->sigma=2.0;
+    }
   return(flags);
 }
 
@@ -1141,11 +1154,7 @@ MagickExport void SetGeometry(const Image *image,RectangleInfo *geometry)
   assert(geometry != (RectangleInfo *) NULL);
   (void) ResetMagickMemory(geometry,0,sizeof(*geometry));
   geometry->width=image->columns;
-  if (image->page.width != 0)
-    geometry->width=image->page.width;
   geometry->height=image->rows;
-  if (image->page.height != 0)
-    geometry->height=image->page.height;
 }
 
 /*
