@@ -358,7 +358,7 @@ MagickExport HashmapInfo *DestroyHashmap(HashmapInfo *hashmap_info)
     list_info=hashmap_info->map[i];
     if (list_info != (LinkedListInfo *) NULL)
       {
-        ResetLinkedListIterator(list_info);
+        list_info->next=list_info->head;
         entry=(EntryInfo *) GetNextValueInLinkedList(list_info);
         while (entry != (EntryInfo *) NULL)
         {
@@ -452,22 +452,28 @@ MagickExport LinkedListInfo *DestroyLinkedList(LinkedListInfo *list_info,
 %
 %  The format of the GetLastValueInLinkedList method is:
 %
-%      void *GetLastValueInLinkedList(const LinkedListInfo *list_info)
+%      void *GetLastValueInLinkedList(LinkedListInfo *list_info)
 %
 %  A description of each parameter follows:
 %
 %    o list_info: The linked_list info.
 %
 */
-MagickExport void *GetLastValueInLinkedList(const LinkedListInfo *list_info)
+MagickExport void *GetLastValueInLinkedList(LinkedListInfo *list_info)
 {
+  void
+    *value;
+
   assert(list_info != (LinkedListInfo *) NULL);
   assert(list_info->signature == MagickSignature);
   if (list_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if (list_info->elements == 0)
     return((void *) NULL);
-  return(list_info->tail->value);
+  AcquireSemaphoreInfo(&list_info->semaphore);
+  value=list_info->tail->value;
+  RelinquishSemaphoreInfo(list_info->semaphore);
+  return(value);
 }
 
 /*
@@ -500,10 +506,14 @@ MagickExport void *GetNextKeyInHashmap(HashmapInfo *hashmap_info)
   register EntryInfo
     *entry;
 
+  void
+    *key;
+
   assert(hashmap_info != (HashmapInfo *) NULL);
   assert(hashmap_info->signature == MagickSignature);
   if (hashmap_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  AcquireSemaphoreInfo(&hashmap_info->semaphore);
   while (hashmap_info->next < hashmap_info->capacity)
   {
     list_info=hashmap_info->map[hashmap_info->next];
@@ -511,16 +521,21 @@ MagickExport void *GetNextKeyInHashmap(HashmapInfo *hashmap_info)
       {
         if (hashmap_info->head_of_list == MagickFalse)
           {
-            ResetLinkedListIterator(list_info);
+            list_info->next=list_info->head;
             hashmap_info->head_of_list=MagickTrue;
           }
         entry=(EntryInfo *) GetNextValueInLinkedList(list_info);
         if (entry != (EntryInfo *) NULL)
-          return(entry->key);
+          {
+            key=entry->key;
+            RelinquishSemaphoreInfo(hashmap_info->semaphore);
+            return(key);
+          }
         hashmap_info->head_of_list=MagickFalse;
       }
     hashmap_info->next++;
   }
+  RelinquishSemaphoreInfo(hashmap_info->semaphore);
   return((void *) NULL);
 }
 
@@ -554,10 +569,14 @@ MagickExport void *GetNextValueInHashmap(HashmapInfo *hashmap_info)
   register EntryInfo
     *entry;
 
+  void
+    *value;
+
   assert(hashmap_info != (HashmapInfo *) NULL);
   assert(hashmap_info->signature == MagickSignature);
   if (hashmap_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  AcquireSemaphoreInfo(&hashmap_info->semaphore);
   while (hashmap_info->next < hashmap_info->capacity)
   {
     list_info=hashmap_info->map[hashmap_info->next];
@@ -565,16 +584,21 @@ MagickExport void *GetNextValueInHashmap(HashmapInfo *hashmap_info)
       {
         if (hashmap_info->head_of_list == MagickFalse)
           {
-            ResetLinkedListIterator(list_info);
+            list_info->next=list_info->head;
             hashmap_info->head_of_list=MagickTrue;
           }
         entry=(EntryInfo *) GetNextValueInLinkedList(list_info);
         if (entry != (EntryInfo *) NULL)
-          return(entry->value);
+          {
+            value=entry->value;
+            RelinquishSemaphoreInfo(hashmap_info->semaphore);
+            return(value);
+          }
         hashmap_info->head_of_list=MagickFalse;
       }
     hashmap_info->next++;
   }
+  RelinquishSemaphoreInfo(hashmap_info->semaphore);
   return((void *) NULL);
 }
 
@@ -611,8 +635,10 @@ MagickExport void *GetNextValueInLinkedList(LinkedListInfo *list_info)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if (list_info->next == (ElementInfo *) NULL)
     return((void *) NULL);
+  AcquireSemaphoreInfo(&list_info->semaphore);
   value=list_info->next->value;
   list_info->next=list_info->next->next;
+  RelinquishSemaphoreInfo(list_info->semaphore);
   return(value);
 }
 
@@ -697,7 +723,7 @@ MagickExport unsigned long GetNumberOfElementsInLinkedList(
 %
 %  The format of the GetValueFromHashmap method is:
 %
-%      void *GetValueFromHashmap(const HashmapInfo *hashmap_info,void *key)
+%      void *GetValueFromHashmap(HashmapInfo *hashmap_info,const void *key)
 %
 %  A description of each parameter follows:
 %
@@ -706,7 +732,7 @@ MagickExport unsigned long GetNumberOfElementsInLinkedList(
 %    o key: The key.
 %
 */
-MagickExport void *GetValueFromHashmap(const HashmapInfo *hashmap_info,
+MagickExport void *GetValueFromHashmap(HashmapInfo *hashmap_info,
   const void *key)
 {
   LinkedListInfo
@@ -718,17 +744,21 @@ MagickExport void *GetValueFromHashmap(const HashmapInfo *hashmap_info,
   size_t
     hash;
 
+  void
+    *value;
+
   assert(hashmap_info != (HashmapInfo *) NULL);
   assert(hashmap_info->signature == MagickSignature);
   if (hashmap_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if (key == (const void *) NULL)
     return((void *) NULL);
+  AcquireSemaphoreInfo(&hashmap_info->semaphore);
   hash=hashmap_info->hash(key);
   list_info=hashmap_info->map[hash % hashmap_info->capacity];
   if (list_info != (LinkedListInfo *) NULL)
     {
-      ResetLinkedListIterator(list_info);
+      list_info->next=list_info->head;
       entry=(EntryInfo *) GetNextValueInLinkedList(list_info);
       while (entry != (EntryInfo *) NULL)
       {
@@ -742,11 +772,16 @@ MagickExport void *GetValueFromHashmap(const HashmapInfo *hashmap_info,
                 (MagickBooleanType (*)(const void *,const void *)) NULL)
               compare=hashmap_info->compare(key,entry->key);
             if (compare == MagickTrue)
-              return(entry->value);
+              {
+                value=entry->value;
+                RelinquishSemaphoreInfo(hashmap_info->semaphore);
+                return(value);
+              }
           }
         entry=(EntryInfo *) GetNextValueInLinkedList(list_info);
       }
     }
+  RelinquishSemaphoreInfo(hashmap_info->semaphore);
   return((void *) NULL);
 }
 
@@ -766,7 +801,7 @@ MagickExport void *GetValueFromHashmap(const HashmapInfo *hashmap_info,
 %
 %  The format of the GetValueFromLinkedList method is:
 %
-%      void *GetValueFromLinkedList(const LinkedListInfo *list_info,
+%      void *GetValueFromLinkedList(LinkedListInfo *list_info,
 %        const unsigned long index)
 %
 %  A description of each parameter follows:
@@ -776,7 +811,7 @@ MagickExport void *GetValueFromHashmap(const HashmapInfo *hashmap_info,
 %    o index: The list index.
 %
 */
-MagickExport void *GetValueFromLinkedList(const LinkedListInfo *list_info,
+MagickExport void *GetValueFromLinkedList(LinkedListInfo *list_info,
   const unsigned long index)
 {
   register ElementInfo
@@ -785,20 +820,34 @@ MagickExport void *GetValueFromLinkedList(const LinkedListInfo *list_info,
   register long
     i;
 
+  void
+    *value;
+
   assert(list_info != (LinkedListInfo *) NULL);
   assert(list_info->signature == MagickSignature);
   if (list_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if (index >= list_info->elements)
     return((void *) NULL);
+  AcquireSemaphoreInfo(&list_info->semaphore);
   if (index == 0)
-    return(list_info->head->value);
+    {
+      value=list_info->head->value;
+      RelinquishSemaphoreInfo(list_info->semaphore);
+      return(value);
+    }
   if (index == (list_info->elements-1))
-    return(list_info->tail->value);
+    {
+      value=list_info->tail->value;
+      RelinquishSemaphoreInfo(list_info->semaphore);
+      return(value);
+    }
   next=list_info->head;
   for (i=0; i < (long) index; i++)
     next=next->next;
-  return(next->value);
+  value=next->value;
+  RelinquishSemaphoreInfo(list_info->semaphore);
+  return(value);
 }
 
 /*
@@ -1095,8 +1144,8 @@ MagickExport MagickBooleanType InsertValueInSortedLinkedList(
             *replace=next->next->value;
             next->next=next->next->next;
             if (element != (ElementInfo *) NULL)
-              element->next=(ElementInfo *)
-                RelinquishMagickMemory(element->next);
+              element->next=(ElementInfo *) RelinquishMagickMemory(
+                element->next);
             list_info->elements--;
           }
         if (element != (ElementInfo *) NULL)
@@ -1149,7 +1198,7 @@ MagickExport MagickBooleanType IsHashmapEmpty(const HashmapInfo *hashmap_info)
   assert(hashmap_info->signature == MagickSignature);
   if (hashmap_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
-  return((MagickBooleanType) (hashmap_info->entries == 0));
+  return(hashmap_info->entries == 0 ? MagickTrue : MagickFalse);
 }
 
 /*
@@ -1181,7 +1230,7 @@ MagickExport MagickBooleanType IsLinkedListEmpty(
   assert(list_info->signature == MagickSignature);
   if (list_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
-  return((MagickBooleanType) (list_info->elements == 0));
+  return(list_info->elements == 0 ? MagickTrue : MagickFalse);
 }
 
 /*
@@ -1224,12 +1273,14 @@ MagickExport MagickBooleanType LinkedListToArray(LinkedListInfo *list_info,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
   if (array == (void **) NULL)
     return(MagickFalse);
+  AcquireSemaphoreInfo(&list_info->semaphore);
   next=list_info->head;
   for (i=0; next != (ElementInfo *) NULL; i++)
   {
     array[i]=next->value;
     next=next->next;
   }
+  RelinquishSemaphoreInfo(list_info->semaphore);
   return(MagickTrue);
 }
 
@@ -1286,8 +1337,15 @@ MagickExport HashmapInfo *NewHashmap(const unsigned long capacity,
 
   hashmap_info=(HashmapInfo *) AcquireMagickMemory(sizeof(*hashmap_info));
   if (hashmap_info == (HashmapInfo *) NULL)
-    ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowMagickFatalException(ResourceLimitFatalError,
+        "MemoryAllocationFailed",message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
   (void) ResetMagickMemory(hashmap_info,0,sizeof(*hashmap_info));
   hashmap_info->hash=HashPointerType;
   if (hash != (size_t (*)(const void *)) NULL)
@@ -1299,11 +1357,18 @@ MagickExport HashmapInfo *NewHashmap(const unsigned long capacity,
   hashmap_info->relinquish_value=relinquish_value;
   hashmap_info->entries=0;
   hashmap_info->capacity=capacity;
-  hashmap_info->map=(LinkedListInfo **)
-    AcquireMagickMemory((size_t) (capacity+1)*sizeof(*hashmap_info->map));
+  hashmap_info->map=(LinkedListInfo **) AcquireMagickMemory((size_t)
+    (capacity+1)*sizeof(*hashmap_info->map));
   if (hashmap_info->map == (LinkedListInfo **) NULL)
-    ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowMagickFatalException(ResourceLimitFatalError,
+        "MemoryAllocationFailed",message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
   (void) ResetMagickMemory(hashmap_info->map,0,(size_t) capacity*
     sizeof(*hashmap_info->map));
   hashmap_info->debug=IsEventLogging();
@@ -1342,8 +1407,15 @@ MagickExport LinkedListInfo *NewLinkedList(const unsigned long capacity)
 
   list_info=(LinkedListInfo *) AcquireMagickMemory(sizeof(*list_info));
   if (list_info == (LinkedListInfo *) NULL)
-    ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowMagickFatalException(ResourceLimitFatalError,
+        "MemoryAllocationFailed",message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
   (void) ResetMagickMemory(list_info,0,sizeof(*list_info));
   list_info->capacity=capacity == 0 ? (unsigned long) (~0) : capacity;
   list_info->elements=0;
@@ -1460,8 +1532,8 @@ static MagickBooleanType IncreaseHashmapCapacity(HashmapInfo *hashmap_info)
     list_info->semaphore=DestroySemaphoreInfo(list_info->semaphore);
     list_info=(LinkedListInfo *) RelinquishMagickMemory(list_info);
   }
-  hashmap_info->map=(LinkedListInfo **)
-    RelinquishMagickMemory(hashmap_info->map);
+  hashmap_info->map=(LinkedListInfo **) RelinquishMagickMemory(
+    hashmap_info->map);
   hashmap_info->map=map;
   hashmap_info->capacity=capacity;
   return(MagickTrue);
@@ -1501,7 +1573,7 @@ MagickExport MagickBooleanType PutEntryInHashmap(HashmapInfo *hashmap_info,
     }
   else
     {
-      ResetLinkedListIterator(list_info);
+      list_info->next=list_info->head;
       entry=(EntryInfo *) GetNextValueInLinkedList(list_info);
       for (i=0; entry != (EntryInfo *) NULL; i++)
       {
@@ -1748,7 +1820,7 @@ MagickExport void *RemoveEntryFromHashmap(HashmapInfo *hashmap_info,
   list_info=hashmap_info->map[hash % hashmap_info->capacity];
   if (list_info != (LinkedListInfo *) NULL)
     {
-      ResetLinkedListIterator(list_info);
+      list_info->next=list_info->head;
       entry=(EntryInfo *) GetNextValueInLinkedList(list_info);
       for (i=0; entry != (EntryInfo *) NULL; i++)
       {
@@ -1875,8 +1947,10 @@ MagickExport void ResetHashmapIterator(HashmapInfo *hashmap_info)
   assert(hashmap_info->signature == MagickSignature);
   if (hashmap_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  AcquireSemaphoreInfo(&hashmap_info->semaphore);
   hashmap_info->next=0;
   hashmap_info->head_of_list=MagickFalse;
+  RelinquishSemaphoreInfo(hashmap_info->semaphore);
 }
 
 /*
@@ -1909,5 +1983,7 @@ MagickExport void ResetLinkedListIterator(LinkedListInfo *list_info)
   assert(list_info->signature == MagickSignature);
   if (list_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  AcquireSemaphoreInfo(&list_info->semaphore);
   list_info->next=list_info->head;
+  RelinquishSemaphoreInfo(list_info->semaphore);
 }
