@@ -190,6 +190,9 @@ MagickExport MagickBooleanType ExecuteModuleProcess(const char *tag,
   ModuleHandle
     handle;
 
+  size_t
+    length;
+
   /*
     Find the module.
   */
@@ -199,14 +202,15 @@ MagickExport MagickBooleanType ExecuteModuleProcess(const char *tag,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",(*image)->filename);
 #if !defined(BuildMagickModules)
   status=ExecuteStaticModuleProcess(tag,image,argc,argv);
+printf("%d\n",status);
   if (status != MagickFalse)
     return(status);
 #endif
   status=MagickFalse;
   TagToFilterModuleName(tag,name);
-  status=GetMagickModulePath(name,MagickFilterModule,path,&(*image)->exception);
-  if (status == MagickFalse)
-    return(status);
+  length=GetMagickModulePath(name,MagickFilterModule,path,&(*image)->exception);
+  if (length == 0)
+    return(MagickFalse);
   /*
     Open the module.
   */
@@ -231,7 +235,10 @@ MagickExport MagickBooleanType ExecuteModuleProcess(const char *tag,
   */
   module=(MagickBooleanType (*)(Image **,const int,char **))
     lt_dlsym(handle,name);
-  if (module != (MagickBooleanType (*)(Image **,const int,char **)) NULL)
+  if (module == (MagickBooleanType (*)(Image **,const int,char **)) NULL)
+    (void) ThrowMagickException(&(*image)->exception,GetMagickModule(),
+      ModuleError,"UnableToLoadModule","`%s': %s",name,lt_dlerror());
+  else
     {
       if ((*image)->debug != MagickFalse)
         (void) LogMagickEvent(ModuleEvent,GetMagickModule(),
@@ -280,9 +287,6 @@ MagickExport MagickBooleanType ExecuteModuleProcess(const char *tag,
 MagickExport const ModuleInfo *GetModuleInfo(const char *tag,
   ExceptionInfo *exception)
 {
-  register const ModuleInfo
-    *p;
-
   if ((module_list == (SplayTreeInfo *) NULL) ||
       (instantiate_module == MagickFalse))
     if (InitializeModuleList(exception) == MagickFalse)
@@ -299,17 +303,7 @@ MagickExport const ModuleInfo *GetModuleInfo(const char *tag,
       ResetSplayTreeIterator(module_list);
       return((const ModuleInfo *) GetNextValueInSplayTree(module_list));
     }
-  AcquireSemaphoreInfo(&module_semaphore);
-  ResetSplayTreeIterator(module_list);
-  p=(const ModuleInfo *) GetNextValueInSplayTree(module_list);
-  while (p != (const ModuleInfo *) NULL)
-  {
-    if (LocaleCompare(p->tag,tag) == 0)
-      break;
-    p=(const ModuleInfo *) GetNextValueInSplayTree(module_list);
-  }
-  RelinquishSemaphoreInfo(module_semaphore);
-  return((const ModuleInfo *) p);
+  return((const ModuleInfo *) GetValueFromSplayTree(module_list,tag));
 }
 
 /*
@@ -468,12 +462,12 @@ MagickExport char **GetModuleList(const char *pattern,
   DIR
     *directory;
 
-  MagickBooleanType
-    status;
-
   register long
     i;
 
+  size_t
+    length;
+
   struct dirent
     *entry;
 
@@ -484,9 +478,9 @@ MagickExport char **GetModuleList(const char *pattern,
     Locate all modules in the coder path.
   */
   TagToCoderModuleName("magick",filename);
-  status=GetMagickModulePath(filename,MagickCoderModule,module_path,
+  length=GetMagickModulePath(filename,MagickCoderModule,module_path,
     exception);
-  if (status == MagickFalse)
+  if (length == 0)
     return((char **) NULL);
   GetPathComponent(module_path,HeadPath,path);
   max_entries=255;
@@ -572,53 +566,54 @@ MagickExport char **GetModuleList(const char *pattern,
 static MagickBooleanType GetMagickModulePath(const char *filename,
   MagickModuleType module_type,char *path,ExceptionInfo *exception)
 {
-  const char
+  char
     *module_path;
 
-  register char
-    *q;
-
-  register const char
-    *p;
-
   assert(filename != (const char *) NULL);
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",filename);
   assert(path != (char *) NULL);
   assert(exception != (ExceptionInfo *) NULL);
   (void) CopyMagickString(path,filename,MaxTextExtent);
-  module_path=(const char *) NULL;
+  module_path=(char *) NULL;
   switch (module_type)
   {
     case MagickCoderModule:
     default:
     {
-      (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+      (void) LogMagickEvent(ModuleEvent,GetMagickModule(),
         "Searching for coder module file \"%s\" ...",filename);
-      module_path=getenv("MAGICK_CODER_MODULE_PATH");
+      module_path=GetEnvironmentValue("MAGICK_CODER_MODULE_PATH");
       break;
     }
     case MagickFilterModule:
     {
-      (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+      (void) LogMagickEvent(ModuleEvent,GetMagickModule(),
         "Searching for filter module file \"%s\" ...",filename);
-      module_path=getenv("MAGICK_FILTER_MODULE_PATH");
+      module_path=GetEnvironmentValue("MAGICK_CODER_FILTER_PATH");
       break;
     }
   }
-  if (module_path != (const char *) NULL)
-    for (p=module_path-1; p != (char *) NULL; )
+  if (module_path != (char *) NULL)
     {
-      (void) CopyMagickString(path,p+1,MaxTextExtent);
-      q=strchr(path,DirectoryListSeparator);
-      if (q != (char *) NULL)
-        *q='\0';
-      q=path+strlen(path)-1;
-      if ((q >= path) && (*q != *DirectorySeparator))
-        (void) ConcatenateMagickString(path,DirectorySeparator,MaxTextExtent);
-      (void) ConcatenateMagickString(path,filename,MaxTextExtent);
-      if (IsAccessible(path) != MagickFalse)
-        return(MagickTrue);
-      p=strchr(p+1,DirectoryListSeparator);
+      register char
+        *p,
+        *q;
+
+      for (p=module_path-1; p != (char *) NULL; )
+      {
+        (void) CopyMagickString(path,p+1,MaxTextExtent);
+        q=strchr(path,DirectoryListSeparator);
+        if (q != (char *) NULL)
+          *q='\0';
+        q=path+strlen(path)-1;
+        if ((q >= path) && (*q != *DirectorySeparator))
+          (void) ConcatenateMagickString(path,DirectorySeparator,MaxTextExtent);
+        (void) ConcatenateMagickString(path,filename,MaxTextExtent);
+        if (IsAccessible(path) != MagickFalse)
+          return(MagickTrue);
+        p=strchr(p+1,DirectoryListSeparator);
+      }
+      module_path=(char *) RelinquishMagickMemory(module_path);
     }
 #if defined(UseInstalledMagick)
 #if defined(MagickCoderModulesPath)
@@ -700,11 +695,11 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
 #endif
 #else
   {
-    const char
+    char
       *home;
 
-    home=getenv("MAGICK_HOME");
-    if (home != (char *) NULL)
+    home=GetEnvironmentValue("MAGICK_HOME");
+		if (home != (char *) NULL)
       {
         /*
           Search MAGICK_HOME.
@@ -733,6 +728,7 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
         (void) FormatMagickString(path,MaxTextExtent,"%s/lib/%s/%s",home,
           directory,filename);
 #endif
+        home=(char *) RelinquishMagickMemory(home);
         if (IsAccessible(path) != MagickFalse)
           return(MagickTrue);
       }
@@ -789,12 +785,12 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
   }
 #endif
   {
-    const char
+    char
       *home;
 
-    home=getenv("HOME");
+    home=GetEnvironmentValue("HOME");
     if (home == (char *) NULL)
-      home=getenv("USERPROFILE");
+      home=GetEnvironmentValue("USERPROFILE");
     if (home != (char *) NULL)
       {
         /*
@@ -804,6 +800,7 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
           *home == '/' ? "/.magick" : "",DirectorySeparator,filename);
         if (IsAccessible(path) != MagickFalse)
           return(MagickTrue);
+        home=(char *) RelinquishMagickMemory(home);
       }
   }
   /*
@@ -881,12 +878,26 @@ static MagickBooleanType InitializeModuleList(
           module_list=NewSplayTree(CompareSplayTreeString,
             RelinquishMagickMemory,DestroyModuleNode);
           if (module_list == (SplayTreeInfo *) NULL)
-            ThrowMagickFatalException(ResourceLimitFatalError,
-              "MemoryAllocationFailed",strerror(errno));
+            {
+              char
+                *message;
+
+              message=GetExceptionMessage(errno);
+              ThrowMagickFatalException(ResourceLimitFatalError,
+                "MemoryAllocationFailed",message);
+              message=(char *) RelinquishMagickMemory(message);
+            }
           module_info=(ModuleInfo *) AcquireMagickMemory(sizeof(*module_info));
           if (module_info == (ModuleInfo *) NULL)
-            ThrowMagickFatalException(ResourceLimitFatalError,
-              "MemoryAllocationFailed",strerror(errno));
+            {
+              char
+                *message;
+
+              message=GetExceptionMessage(errno);
+              ThrowMagickFatalException(ResourceLimitFatalError,
+                "MemoryAllocationFailed",message);
+              message=(char *) RelinquishMagickMemory(message);
+            }
           (void) ResetMagickMemory(module_info,0,sizeof(*module_info));
           module_info->tag=ConstantString("[boot-strap]");
           module_info->stealth=MagickTrue;
@@ -895,8 +906,15 @@ static MagickBooleanType InitializeModuleList(
           status=AddValueToSplayTree(module_list,
             ConstantString(module_info->tag),module_info);
           if (status == MagickFalse)
-            ThrowMagickFatalException(ResourceLimitFatalError,
-              "MemoryAllocationFailed",strerror(errno));
+            {
+              char
+                *message;
+
+              message=GetExceptionMessage(errno);
+              ThrowMagickFatalException(ResourceLimitFatalError,
+                "MemoryAllocationFailed",message);
+              message=(char *) RelinquishMagickMemory(message);
+            }
           if (lt_dlinit() != 0)
             ThrowMagickFatalException(ModuleFatalError,
               "UnableToInitializeModuleLoader",lt_dlerror());
@@ -1008,9 +1026,6 @@ MagickExport MagickBooleanType OpenModule(const char *module,
     name[MaxTextExtent],
     path[MaxTextExtent];
 
-  MagickBooleanType
-    status;
-
   ModuleHandle
     handle;
 
@@ -1020,6 +1035,9 @@ MagickExport MagickBooleanType OpenModule(const char *module,
   register const CoderInfo
     *p;
 
+  size_t
+    length;
+
   /*
     Assign module name from alias.
   */
@@ -1027,21 +1045,30 @@ MagickExport MagickBooleanType OpenModule(const char *module,
   module_info=(ModuleInfo *) GetModuleInfo(module,exception);
   if (module_info != (ModuleInfo *) NULL)
     return(MagickTrue);
+  AcquireSemaphoreInfo(&module_semaphore);
   (void) CopyMagickString(module_name,module,MaxTextExtent);
   p=GetCoderInfo(module,exception);
   if (p != (CoderInfo *) NULL)
     (void) CopyMagickString(module_name,p->name,MaxTextExtent);
+  if (GetValueFromSplayTree(module_list,module_name) != (void *) NULL)
+    {
+      RelinquishSemaphoreInfo(module_semaphore);
+      return(MagickTrue);  /* module alread opened, return */
+    }
   /*
-    Find module file.
+    Locate module.
   */
   handle=(ModuleHandle) NULL;
   TagToCoderModuleName(module_name,filename);
-  (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+  (void) LogMagickEvent(ModuleEvent,GetMagickModule(),
     "Searching for module \"%s\" using filename \"%s\"",module_name,filename);
   *path='\0';
-  status=GetMagickModulePath(filename,MagickCoderModule,path,exception);
-  if (status == MagickFalse)
-    return(status);
+  length=GetMagickModulePath(filename,MagickCoderModule,path,exception);
+  if (length == 0)
+    {
+      RelinquishSemaphoreInfo(module_semaphore);
+      return(MagickFalse);
+    }
   /*
     Load module
   */
@@ -1050,27 +1077,34 @@ MagickExport MagickBooleanType OpenModule(const char *module,
     {
       (void) ThrowMagickException(exception,GetMagickModule(),ModuleError,
         "UnableToLoadModule","`%s': %s",path,lt_dlerror());
+      RelinquishSemaphoreInfo(module_semaphore);
       return(MagickFalse);
     }
-  (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+  (void) LogMagickEvent(ModuleEvent,GetMagickModule(),
     "Opening module at path \"%s\"",path);
   /*
-    Add module to module module list.
+    Register module.
   */
   module_info=(ModuleInfo *) AcquireMagickMemory(sizeof(*module_info));
   if (module_info == (ModuleInfo *) NULL)
-    ThrowMagickFatalException(ResourceLimitFatalError,
-      "UnableToAllocateModuleInfo",module_name);
+    {
+      RelinquishSemaphoreInfo(module_semaphore);
+      ThrowMagickFatalException(ResourceLimitFatalError,
+        "UnableToAllocateModuleInfo",module_name);
+    }
   (void) ResetMagickMemory(module_info,0,sizeof(*module_info));
   module_info->path=ConstantString(path);
   module_info->tag=ConstantString(module_name);
-  module_info->signature=MagickSignature;
   module_info->handle=handle;
   (void) time(&module_info->load_time);
+  module_info->signature=MagickSignature;
   if (RegisterModule(module_info,exception) == (ModuleInfo *) NULL)
-    return(MagickFalse);
+    {
+      RelinquishSemaphoreInfo(module_semaphore);
+      return(MagickFalse);
+    }
   /*
-    Locate and record RegisterFORMATImage function
+    Define RegisterFORMATImage method.
   */
   TagToModuleName(module_name,"Register%sImage",name);
   module_info->register_module=(void (*)(void)) lt_dlsym(handle,name);
@@ -1078,13 +1112,14 @@ MagickExport MagickBooleanType OpenModule(const char *module,
     {
       (void) ThrowMagickException(exception,GetMagickModule(),ModuleError,
         "UnableToRegisterImageFormat","`%s': %s",module_name,lt_dlerror());
+      RelinquishSemaphoreInfo(module_semaphore);
       return(MagickFalse);
     }
-  (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+  (void) LogMagickEvent(ModuleEvent,GetMagickModule(),
     "Method \"%s\" in module \"%s\" at address %p",name,module_name,
     (void *) module_info->register_module);
   /*
-    Locate and record UnregisterFORMATImage function
+    Define UnregisterFORMATImage method.
   */
   TagToModuleName(module_name,"Unregister%sImage",name);
   module_info->unregister_module=(void (*)(void)) lt_dlsym(handle,name);
@@ -1092,15 +1127,14 @@ MagickExport MagickBooleanType OpenModule(const char *module,
     {
       (void) ThrowMagickException(exception,GetMagickModule(),ModuleError,
         "UnableToRegisterImageFormat","`%s': %s",module_name,lt_dlerror());
+      RelinquishSemaphoreInfo(module_semaphore);
       return(MagickFalse);
     }
-  (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+  (void) LogMagickEvent(ModuleEvent,GetMagickModule(),
     "Method \"%s\" in module \"%s\" at address %p",name,module_name,
     (void *) module_info->unregister_module);
-  /*
-    Execute RegisterFORMATImage module.
-  */
   module_info->register_module();
+  RelinquishSemaphoreInfo(module_semaphore);
   return(MagickTrue);
 }
 
@@ -1374,9 +1408,6 @@ static void TagToModuleName(const char *tag,const char *format,char *module)
 static MagickBooleanType UnregisterModule(const ModuleInfo *module_info,
   ExceptionInfo *exception)
 {
-  char
-    name[MaxTextExtent];
-
   /*
     Locate and execute UnregisterFORMATImage module.
   */
@@ -1388,13 +1419,19 @@ static MagickBooleanType UnregisterModule(const ModuleInfo *module_info,
   module_info->unregister_module();
   if (lt_dlclose((ModuleHandle) module_info->handle) != 0)
     {
-      (void) ThrowMagickException(exception,GetMagickModule(),ModuleError,
-        "UnableToCloseModule","`%s': %s",name,lt_dlerror());
+      (void) ThrowMagickException(exception,GetMagickModule(),ModuleWarning,
+        "UnableToCloseModule","`%s': %s",module_info->tag,lt_dlerror());
       return(MagickFalse);
     }
   return(MagickTrue);
 }
 #else
+MagickExport MagickBooleanType ListModuleInfo(FILE *file,
+  ExceptionInfo *exception)
+{
+  return(MagickTrue);
+}
+
 MagickExport MagickBooleanType ExecuteModuleProcess(const char *tag,
   Image **image,const int argc,char **argv)
 {
