@@ -141,11 +141,12 @@ BOOL WINAPI DllMain(HINSTANCE handle,DWORD reason,LPVOID lpvReserved)
       ssize_t
         count;
 
-      module_path=(char *) AcquireMagickMemory(MaxTextExtent);
-      new_path=(char *) AcquireMagickMemory(16*MaxTextExtent);
-      path=(char *) AcquireMagickMemory(16*MaxTextExtent);
+      module_path=(char *) AcquireMagickMemory(MaxTextExtent*
+        sizeof(*module_path));
+      new_path=(char *) AcquireMagickMemory(16*MaxTextExtent*sizeof(*new_path));
+      path=(char *) AcquireMagickMemory(16*MaxTextExtent*sizeof(*path));
       if ((module_path == (char *) NULL) || (new_path == (char *) NULL) ||
-          (path == (char *) NULL))
+         (path == (char *) NULL))
         return(FALSE);
       count=(ssize_t) GetModuleFileName(handle,module_path,MaxTextExtent);
       if (count != 0)
@@ -166,6 +167,7 @@ BOOL WINAPI DllMain(HINSTANCE handle,DWORD reason,LPVOID lpvReserved)
                     "%s;%s",module_path,path);
                   SetEnvironmentVariable("PATH",new_path);
                 }
+              path=(char *) RelinquishMagickMemory(path);
             }
         }
       path=(char *) RelinquishMagickMemory(path);
@@ -411,27 +413,30 @@ MagickExport void NTErrorHandler(const ExceptionType error,const char *reason,
   const char *description)
 {
   char
-    buffer[3*MaxTextExtent];
+    buffer[3*MaxTextExtent],
+    *message;
 
   if (reason == (char *) NULL)
     {
       DestroyMagick();
       exit(0);
     }
+  message=GetExceptionMessage(errno);
   if ((description != (char *) NULL) && errno)
     (void) FormatMagickString(buffer,MaxTextExtent,"%s: %s (%s) [%s].\n",
-      GetClientName(),reason,description,strerror(errno));
+      GetClientName(),reason,description,message);
   else
     if (description != (char *) NULL)
       (void) FormatMagickString(buffer,MaxTextExtent,"%s: %s (%s).\n",
         GetClientName(),reason,description);
     else
-      if (errno)
+      if (errno != 0)
         (void) FormatMagickString(buffer,MaxTextExtent,"%s: %s [%s].\n",
-          GetClientName(),reason,strerror(errno));
+          GetClientName(),reason,message);
       else
         (void) FormatMagickString(buffer,MaxTextExtent,"%s: %s.\n",
           GetClientName(),reason);
+  message=(char *) RelinquishMagickMemory(message);
   (void) MessageBox(NULL,buffer,"ImageMagick Exception",MB_OK | MB_TASKMODAL |
     MB_SETFOREGROUND | MB_ICONEXCLAMATION);
   DestroyMagick();
@@ -476,7 +481,7 @@ MagickExport int NTExitLibrary(void)
 %
 %  The format of the NTFileTruncate method is:
 %
-%      int NTFileTruncate(int file,off_t length)
+%      int NTFileTruncate(int file,const MagickSizeType length)
 %
 %  A description of each parameter follows:
 %
@@ -485,38 +490,26 @@ MagickExport int NTExitLibrary(void)
 %    o length: The file length.
 %
 */
-MagickExport int NTFileTruncate(int file,off_t length)
+MagickExport int NTFileTruncate(int file,const MagickSizeType length)
 {
-  HANDLE
-    handle;
+  DWORD
+    file_pointer;
 
-  unsigned int
-    offset;
+  long
+    file_handle,
+    high,
+    low;
 
-  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
-  if (file < 0)
+  file_handle=_get_osfhandle(file);
+  if (file_handle == -1L)
+    return(-1);
+  low=(long) (length & 0xffffffffUL);
+  high=(long) ((((MagickOffsetType) length) >> 32) & 0xffffffffUL);
+  file_pointer=SetFilePointer((HANDLE) file_handle,low,&high,FILE_BEGIN);
+  if ((file_pointer == 0xFFFFFFFF) && (GetLastError() != NO_ERROR))
+    return(-1);
+  if (SetEndOfFile((HANDLE) file_handle) == 0)
     return(-1);
-  handle=(HANDLE) _get_osfhandle(file);
-  offset=SetFilePointer(handle,0,NULL,FILE_CURRENT);
-  if ((offset == 0xFFFFFFFF) ||
-      (SetFilePointer(handle,length,NULL,FILE_BEGIN) == 0xFFFFFFFF) ||
-      (SetEndOfFile(handle) == 0))
-    {
-      switch (GetLastError())
-      {
-        case ERROR_INVALID_HANDLE:
-        {
-          errno=EBADF;
-          break;
-        }
-        default:
-        {
-          errno=EIO;
-          break;
-        }
-      }
-      return(-1);
-    }
   return(0);
 }
 
@@ -752,14 +745,14 @@ static int NTGhostscriptProductVersions(int *pver,int offset,
     cbData;
 
   char
-    key[256],
+    key[MaxTextExtent],
     *p;
 
   int
     n = 0,
     ver;
 
-  (void) sprintf(key, "Software\\%s",gs_productfamily);
+  (void) FormatMagickString(key,MaxTextExtent,"Software\\%s",gs_productfamily);
   hkeyroot = HKEY_LOCAL_MACHINE;
   if (RegOpenKeyExA(hkeyroot,key,0,KEY_READ,&hkey) == ERROR_SUCCESS)
     {
@@ -890,14 +883,14 @@ static int NTGhostscriptGetProductString(int gs_revision,const char *name,
    * and N.NN is obtained from gs_revision.
    */
 
+  char
+    dotversion[MaxTextExtent],
+    key[MaxTextExtent];
+
   int
     code,
     length;
 
-  char
-    dotversion[16],
-    key[256];
-
   DWORD version = GetVersion();
 
   if (((HIWORD(version) & 0x8000) != 0) && ((HIWORD(version) & 0x4000) == 0))
@@ -905,9 +898,10 @@ static int NTGhostscriptGetProductString(int gs_revision,const char *name,
       /* Win32s */
       return FALSE;
     }
-  (void) sprintf(dotversion,"%d.%02d",(int)(gs_revision/100),(int)
-    (gs_revision % 100));
-  (void) sprintf(key, "Software\\%s\\%s",gs_productfamily, dotversion);
+  (void) FormatMagickString(dotversion,MaxTextExtent,"%d.%02d",(int)
+    (gs_revision/100),(int) (gs_revision % 100));
+  (void) FormatMagickString(key,MaxTextExtent,"Software\\%s\\%s",
+    gs_productfamily,dotversion);
   length = len;
   code = NTGetRegistryValue(HKEY_CURRENT_USER, key, name, ptr, &length);
   if ( code == 0 )
@@ -1280,8 +1274,8 @@ MagickExport void *NTMapMemory(char *address,size_t length,int protection,
   int flags,int file,MagickOffsetType offset)
 {
   DWORD
-    low,
     high,
+    low,
     mode;
 
   HANDLE
@@ -1424,7 +1418,7 @@ MagickExport DIR *NTOpenDirectory(const char *path)
 static const char *GetSearchPath( void )
 {
 #ifdef HasLTDL
-  return(lt_dlGetSearchPath());
+  return(lt_dlgetsearchpath());
 #else
   return(lt_slsearchpath);
 #endif
@@ -1939,8 +1933,8 @@ MagickExport int NTSystemCommand(const char *command)
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  Sysconf() provides  a  way for the application to determine values for
-%  system limits or options at runtime.
+%  NTSystemConfiguration() provides a way for the application to determine
+%  values for system limits or options at runtime.
 %
 %  The format of the exit method is:
 %
@@ -1962,7 +1956,6 @@ MagickExport long NTSystemConfiguration(int name)
 
       GetSystemInfo(&system_info);
       return(system_info.dwPageSize);
-      break;
     }
     case _SC_PHYS_PAGES:
     {
@@ -1972,6 +1965,9 @@ MagickExport long NTSystemConfiguration(int name)
       LPFNDLLFUNC2
         module;
 
+      NTMEMORYSTATUSEX
+        status;
+
       SYSTEM_INFO
         system_info;
 
@@ -1986,24 +1982,19 @@ MagickExport long NTSystemConfiguration(int name)
             status;
 
           GlobalMemoryStatus(&status);
-          return((long) status.dwAvailPhys/system_info.dwPageSize);
+          return((long) status.dwTotalPhys/system_info.dwPageSize);
         }
-      else
-        {
-          NTMEMORYSTATUSEX
-            status;
-
-          status.dwLength=sizeof(status);
-          if (module(&status) == 0)
-            return(0L);
-          return((long) status.ullAvailPhys/system_info.dwPageSize);
-        }
-      break;
+      status.dwLength=sizeof(status);
+      if (module(&status) == 0)
+        return(0L);
+      return((long) status.ullTotalPhys/system_info.dwPageSize);
     }
+    case _SC_OPEN_MAX:
+      return(2048);
     default:
       break;
   }
-  return(0);
+  return(-1);
 }
 
 
@@ -2062,7 +2053,7 @@ MagickExport long NTTellDirectory(DIR *entry)
 */
 MagickExport int NTUnmapMemory(void *map,size_t length)
 {
-  if (!UnmapViewOfFile(map))
+  if (UnmapViewOfFile(map) == 0)
     return(-1);
   return(0);
 }
