@@ -160,8 +160,15 @@ MagickExport void DistillRandomEvent(const unsigned char *event,
       RelinquishSemaphoreInfo(random_semaphore);
       if ((reservoir == (SignatureInfo *) NULL) ||
           (roulette == (unsigned long *) NULL))
-        ThrowMagickFatalException(ResourceLimitFatalError,
-          "UnableToDistillRandomEvent",strerror(errno));
+        {
+          char
+            *message;
+
+          message=GetExceptionMessage(errno);
+          ThrowMagickFatalException(ResourceLimitFatalError,
+            "MemoryAllocationFailed",message);
+          message=(char *) RelinquishMagickMemory(message);
+        }
       (void) ResetMagickMemory(reservoir,0,sizeof(*reservoir));
       (void) ResetMagickMemory(roulette,0,sizeof(*roulette));
     }
@@ -336,7 +343,7 @@ static void InitializeRandomReservoir(void)
     if (gettimeofday(&timer,0) == 0)
       {
         seconds=timer.tv_sec;
-        nanoseconds=1000*timer.tv_usec;
+        nanoseconds=(time_t) (1000*timer.tv_usec);
       }
   }
 #endif
