@@ -60,7 +60,7 @@
 /*
   Define declarations.
 */
-#define MaxSplayTreeDepth  2048
+#define MaxSplayTreeDepth  1024
 
 /*
   Typedef declarations.
@@ -286,8 +286,15 @@ static void BalanceSplayTree(SplayTreeInfo *splay_info)
     }
   nodes=(NodeInfo **) AcquireMagickMemory(splay_info->nodes*sizeof(*nodes));
   if (nodes == (NodeInfo **) NULL)
-    ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowMagickFatalException(ResourceLimitFatalError,
+        "MemoryAllocationFailed",message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
   node=nodes;
   (void) IterateOverSplayTree(splay_info,SplayTreeToNodeArray,
     (const void *) &node);
@@ -483,6 +490,9 @@ MagickExport void *GetNextKeyInSplayTree(SplayTreeInfo *splay_info)
   register NodeInfo
     *node;
 
+  void
+    *key;
+
   assert(splay_info != (SplayTreeInfo *) NULL);
   assert(splay_info->signature == MagickSignature);
   if (splay_info->debug != MagickFalse)
@@ -500,8 +510,9 @@ MagickExport void *GetNextKeyInSplayTree(SplayTreeInfo *splay_info)
         node=node->left;
       splay_info->next=node->key;
     }
+  key=splay_info->root->key;
   RelinquishSemaphoreInfo(splay_info->semaphore);
-  return(splay_info->root->key);
+  return(key);
 }
 
 /*
@@ -533,6 +544,9 @@ MagickExport void *GetNextValueInSplayTree(SplayTreeInfo *splay_info)
   register NodeInfo
     *node;
 
+  void
+    *value;
+
   assert(splay_info != (SplayTreeInfo *) NULL);
   assert(splay_info->signature == MagickSignature);
   if (splay_info->debug != MagickFalse)
@@ -550,8 +564,9 @@ MagickExport void *GetNextValueInSplayTree(SplayTreeInfo *splay_info)
         node=node->left;
       splay_info->next=node->key;
     }
+  value=splay_info->root->value;
   RelinquishSemaphoreInfo(splay_info->semaphore);
-  return(splay_info->root->value);
+  return(value);
 }
 
 /*
@@ -584,6 +599,9 @@ MagickExport void *GetValueFromSplayTree(SplayTreeInfo *splay_info,
   int
     compare;
 
+  void
+    *value;
+
   assert(splay_info != (SplayTreeInfo *) NULL);
   assert(splay_info->signature == MagickSignature);
   if (splay_info->debug != MagickFalse)
@@ -602,8 +620,9 @@ MagickExport void *GetValueFromSplayTree(SplayTreeInfo *splay_info,
       RelinquishSemaphoreInfo(splay_info->semaphore);
       return((void *) NULL);
     }
+  value=splay_info->root->value;
   RelinquishSemaphoreInfo(splay_info->semaphore);
-  return(splay_info->root->value);
+  return(value);
 }
 
 /*
@@ -704,8 +723,15 @@ static int IterateOverSplayTree(SplayTreeInfo *splay_info,
   transitions=(unsigned char *)
     AcquireMagickMemory(splay_info->nodes*sizeof(*transitions));
   if ((nodes == (NodeInfo **) NULL) || (transitions == (unsigned char *) NULL))
-    ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowMagickFatalException(ResourceLimitFatalError,
+        "MemoryAllocationFailed",message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
   status=0;
   final_transition=MagickFalse;
   nodes[0]=splay_info->root;
@@ -803,8 +829,15 @@ MagickExport SplayTreeInfo *NewSplayTree(
 
   splay_info=(SplayTreeInfo *) AcquireMagickMemory(sizeof(*splay_info));
   if (splay_info == (SplayTreeInfo *) NULL)
-    ThrowMagickFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowMagickFatalException(ResourceLimitFatalError,
+        "MemoryAllocationFailed",message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
   (void) ResetMagickMemory(splay_info,0,sizeof(*splay_info));
   splay_info->root=(NodeInfo *) NULL;
   splay_info->compare=compare;
@@ -888,8 +921,12 @@ MagickExport MagickBooleanType RemoveNodeByValueFromSplayTree(
       }
     if (splay_info->root->value == value)
       {
+        void
+          *key;
+
+        key=splay_info->root->key;
         RelinquishSemaphoreInfo(splay_info->semaphore);
-        return(RemoveNodeFromSplayTree(splay_info,splay_info->root->key));
+        return(RemoveNodeFromSplayTree(splay_info,key));
       }
   }
   RelinquishSemaphoreInfo(splay_info->semaphore);
@@ -1008,7 +1045,9 @@ MagickExport void ResetSplayTreeIterator(SplayTreeInfo *splay_info)
   assert(splay_info->signature == MagickSignature);
   if (splay_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  AcquireSemaphoreInfo(&splay_info->semaphore);
   splay_info->next=GetFirstSplayTreeNode(splay_info);
+  RelinquishSemaphoreInfo(splay_info->semaphore);
 }
 
 /*
@@ -1163,8 +1202,15 @@ static void SplaySplayTree(SplayTreeInfo *splay_info,const void *key)
       (void) Splay(splay_info,key,&splay_info->root,(NodeInfo **) NULL,
         (NodeInfo **) NULL);
       if (splay_info->balance != MagickFalse)
-        ThrowMagickFatalException(ResourceLimitFatalError,
-          "MemoryAllocationFailed",strerror(errno));
+        {
+          char
+            *message;
+
+          message=GetExceptionMessage(errno);
+          ThrowMagickFatalException(ResourceLimitFatalError,
+            "MemoryAllocationFailed",message);
+          message=(char *) RelinquishMagickMemory(message);
+        }
     }
   splay_info->key=(void *) key;
 }
