@@ -17,7 +17,7 @@
 %                              January 1993                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -156,11 +156,13 @@ MagickExport void GetMagickToken(const char *start,char **end,char *token)
         for ( ; *p != '\0'; p++)
         {
           if (((isspace((int) ((unsigned char) *p)) != 0) || (*p == '=') ||
-               (*p == ',') || (*p == ':')) && (*(p-1) != '\\'))
+              (*p == ',') || (*p == ':')) && (*(p-1) != '\\'))
             break;
           if ((i > 0) && (*p == '<'))
             break;
           token[i++]=(*p);
+          if (*p == '>')
+            break;
           if (*p == '(')
             for (p++; *p != '\0'; p++)
             {
@@ -253,7 +255,7 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
       (void) CopyMagickString(image_info->filename,pattern,MaxTextExtent);
       GetExceptionInfo(&exception);
       (void) SetImageInfo(image_info,MagickTrue,&exception);
-      DestroyExceptionInfo(&exception);
+      (void) DestroyExceptionInfo(&exception);
       if (LocaleCompare(image_info->filename,pattern) != 0)
         {
           image_info=DestroyImageInfo(image_info);
@@ -448,13 +450,14 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
   }
   while (*pattern == '*')
     pattern++;
-  return((MagickBooleanType) ((*expression == '\0') && (*pattern == '\0')));
+  return((*expression == '\0') && (*pattern == '\0') ? MagickTrue : MagickFalse);
 }
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
 %                                                                             %
+%                                                                             %
 +     I s G l o b                                                             %
 %                                                                             %
 %                                                                             %
@@ -462,7 +465,7 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  IsGlob() returns MagickTrue if the path specification contains a globbing
-%  patten.
+%  pattern.
 %
 %  The format of the IsGlob method is:
 %
@@ -472,7 +475,6 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
 %
 %    o path: The path.
 %
-%
 */
 MagickExport MagickBooleanType IsGlob(const char *path)
 {
