@@ -40,6 +40,7 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/cache-view.h"
 #include "magick/color.h"
 #include "magick/color-private.h"
 #include "magick/composite.h"
@@ -85,7 +86,6 @@
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
   ExceptionInfo *exception)
@@ -235,128 +235,6 @@ MagickExport Image *ChopImage(const Image *image,const RectangleInfo *chop_info,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%     C o a l e s c e I m a g e s                                             %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  CoalesceImages() composites a set of images while respecting any page
-%  offsets and disposal methods.  GIF, MIFF, and MNG animation sequences
-%  typically start with an image background and each subsequent image
-%  varies in size and offset.  CoalesceImages() returns a new sequence
-%  where each image in the sequence is the same size as the first and
-%  composited with the next image in the sequence.
-%
-%  The format of the CoalesceImages method is:
-%
-%      Image *CoalesceImages(Image *image,ExceptionInfo *exception)
-%
-%  A description of each parameter follows:
-%
-%    o image: The image sequence.
-%
-%    o exception: Return any errors or warnings in this structure.
-%
-*/
-MagickExport Image *CoalesceImages(Image *image,ExceptionInfo *exception)
-{
-  Image
-    *coalesce_image;
-
-  Image
-    *dispose_image;
-
-  register Image
-    *next;
-
-  /*
-    Coalesce the image sequence.
-  */
-  assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
-  coalesce_image=CloneImage(image,image->page.width,image->page.height,
-    MagickTrue,exception);
-  if (coalesce_image == (Image *) NULL)
-    return((Image *) NULL);
-  coalesce_image->page=image->page;
-  coalesce_image->background_color.opacity=TransparentOpacity;
-  coalesce_image->page.x=0;
-  coalesce_image->page.y=0;
-  SetImageBackgroundColor(coalesce_image);
-  dispose_image=CloneImage(coalesce_image,0,0,MagickTrue,exception);
-  (void) CompositeImage(coalesce_image,CopyCompositeOp,image,image->page.x,
-    image->page.y);
-  next=GetNextImageInList(image);
-  for ( ; next != (Image *) NULL; next=GetNextImageInList(next))
-  {
-    switch (next->previous->dispose)
-    {
-      case UndefinedDispose:
-      case NoneDispose:
-      {
-        coalesce_image->next=CloneImage(coalesce_image,0,0,MagickTrue,
-          exception);
-        if (dispose_image != (Image *) NULL)
-          dispose_image=DestroyImage(dispose_image);
-        dispose_image=CloneImage(coalesce_image,0,0,MagickTrue,exception);
-        break;
-      }
-      case BackgroundDispose:
-      {
-        coalesce_image->next=CloneImage(coalesce_image,0,0,MagickTrue,
-          exception);
-        coalesce_image->next->background_color.opacity=TransparentOpacity;
-        SetImageBackgroundColor(coalesce_image->next);
-        break;
-      }
-      case PreviousDispose:
-      default:
-      {
-        coalesce_image->next=CloneImage(dispose_image,0,0,MagickTrue,
-          exception);
-        break;
-      }
-    }
-    if (GetNextImageInList(coalesce_image) == (Image *) NULL)
-      {
-        coalesce_image=DestroyImageList(coalesce_image);
-        return((Image *) NULL);
-      }
-    coalesce_image->next->previous=coalesce_image;
-    coalesce_image=GetNextImageInList(coalesce_image);
-    coalesce_image->matte=MagickTrue;
-    coalesce_image->delay=next->delay;
-    coalesce_image->ticks_per_second=next->ticks_per_second;
-    coalesce_image->start_loop=next->start_loop;
-    (void) CompositeImage(coalesce_image,next->matte != MagickFalse ?
-      OverCompositeOp : CopyCompositeOp,next,next->page.x,next->page.y);
-    coalesce_image->page.x=0;
-    coalesce_image->page.y=0;
-  }
-  if (dispose_image != (Image *) NULL)
-    dispose_image=DestroyImage(dispose_image);
-  next=GetFirstImageInList(coalesce_image);
-  next->dispose=BackgroundDispose;
-  for ( ; next->next != (Image *) NULL; next=GetNextImageInList(next))
-  {
-    next->dispose=BackgroundDispose;
-    if (IsOpaqueImage(next->next,exception) == MagickFalse)
-      next->dispose=NoneDispose;
-    next->next->dispose=next->dispose;
-  }
-  return(GetFirstImageInList(coalesce_image));
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
 +     C o n s o l i d a t e C M Y K I m a g e                                 %
 %                                                                             %
 %                                                                             %
@@ -524,7 +402,6 @@ MagickExport Image *ConsolidateCMYKImages(const Image *images,
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
   ExceptionInfo *exception)
@@ -554,6 +431,10 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
   register PixelPacket
     *q;
 
+  ViewInfo
+    *crop_view,
+    *image_view;
+
   /*
     Check crop geometry.
   */
@@ -575,10 +456,10 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
     page.width=bounding_box.width;
   if (page.height == 0)
     page.height=bounding_box.height;
-  if (((bounding_box.x-page.x) > (long) page.width) ||
-      ((bounding_box.y-page.y) > (long) page.height) ||
-      ((page.x-bounding_box.x) >= (long) image->columns) ||
-      ((page.y-bounding_box.y) >= (long) image->rows))
+  if (((bounding_box.x-page.x) >= (long) page.width) ||
+      ((bounding_box.y-page.y) >= (long) page.height) ||
+      ((page.x-bounding_box.x) > (long) image->columns) ||
+      ((page.y-bounding_box.y) > (long) image->rows))
     {
       (void) ThrowMagickException(exception,GetMagickModule(),OptionWarning,
         "GeometryDoesNotContainImage","`%s'",image->filename);
@@ -637,21 +518,31 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
   /*
     Extract crop image.
   */
-  crop_image->page=bounding_box;
+  if (((bounding_box.x+bounding_box.width) > image->page.width) ||
+      ((bounding_box.y+bounding_box.height) > image->page.height))
+    {
+      crop_image->page.width=bounding_box.width;
+      crop_image->page.height=bounding_box.height;
+    }
+  crop_image->page.x=bounding_box.x;
+  crop_image->page.y=bounding_box.y;
+  image_view=OpenCacheView(image);
+  crop_view=OpenCacheView(crop_image);
   for (y=0; y < (long) crop_image->rows; y++)
   {
-    p=AcquireImagePixels(image,page.x,page.y+y,crop_image->columns,1,exception);
-    q=SetImagePixels(crop_image,0,y,crop_image->columns,1);
+    p=AcquireCacheView(image_view,page.x,page.y+y,crop_image->columns,1,
+      exception);
+    q=SetCacheView(crop_view,0,y,crop_image->columns,1);
     if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
       break;
     (void) CopyMagickMemory(q,p,(size_t) crop_image->columns*sizeof(*q));
-    indexes=GetIndexes(image);
-    crop_indexes=GetIndexes(crop_image);
+    indexes=GetCacheViewIndexes(image_view);
+    crop_indexes=GetCacheViewIndexes(crop_view);
     if ((indexes != (IndexPacket *) NULL) &&
         (crop_indexes != (IndexPacket *) NULL))
       (void) CopyMagickMemory(crop_indexes,indexes,(size_t) crop_image->columns*
         sizeof(*crop_indexes));
-    if (SyncImagePixels(crop_image) == MagickFalse)
+    if (SyncCacheView(crop_view) == MagickFalse)
       break;
     if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
         (QuantumTick(y,image->rows) != MagickFalse))
@@ -662,6 +553,8 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
           break;
       }
   }
+  crop_view=CloseCacheView(crop_view);
+  image_view=CloseCacheView(image_view);
   return(crop_image);
 }
 
@@ -670,225 +563,6 @@ MagickExport Image *CropImage(const Image *image,const RectangleInfo *geometry,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%     D e c o n s t r u c t I m a g e s                                       %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  DeconstructImages() compares each image with the next in a sequence and
-%  returns the maximum bounding region of any pixel differences it discovers.
-%
-%  The format of the DeconstructImages method is:
-%
-%      Image *DeconstructImages(const Image *image,ExceptionInfo *exception)
-%
-%  A description of each parameter follows:
-%
-%    o image: The image.
-%
-%    o exception: Return any errors or warnings in this structure.
-%
-%
-*/
-MagickExport Image *DeconstructImages(const Image *image,
-  ExceptionInfo *exception)
-{
-  Image
-    *crop_image,
-    *crop_next,
-    *deconstruct_image;
-
-  long
-    y;
-
-  MagickPixelPacket
-    pixel,
-    target;
-
-  RectangleInfo
-    *bounds;
-
-  register const Image
-    *next;
-
-  register const PixelPacket
-    *p,
-    *q;
-
-  register IndexPacket
-    *indexes,
-    *next_indexes;
-
-  register long
-    i,
-    x;
-
-  assert(image != (const Image *) NULL);
-  assert(image->signature == MagickSignature);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
-  /*
-    Ensure the image are the same size.
-  */
-  for (next=image; next != (Image *) NULL; next=GetNextImageInList(next))
-  {
-    if ((next->columns != image->columns) || (next->rows != image->rows))
-      ThrowImageException(OptionError,"ImagesAreNotTheSameSize");
-  }
-  /*
-    Allocate memory.
-  */
-  bounds=(RectangleInfo *) AcquireMagickMemory((size_t)
-    GetImageListLength(image)*sizeof(*bounds));
-  if (bounds == (RectangleInfo *) NULL)
-    ThrowImageException(ResourceLimitError,"MemoryAllocationFailed");
-  /*
-    Compute the bounding box for each next in the sequence.
-  */
-  i=0;
-  GetMagickPixelPacket(image,&pixel);
-  GetMagickPixelPacket(image,&target);
-  next=GetNextImageInList(image);
-  for ( ; next != (const Image *) NULL; next=GetNextImageInList(next))
-  {
-    /*
-      Set bounding box to the next dimensions.
-    */
-    for (x=0; x < (long) next->columns; x++)
-    {
-      p=AcquireImagePixels(next,x,0,1,next->rows,exception);
-      q=AcquireImagePixels(next->previous,x,0,1,next->previous->rows,exception);
-      if ((p == (const PixelPacket *) NULL) ||
-          (q == (const PixelPacket *) NULL))
-        break;
-      indexes=GetIndexes(next);
-      next_indexes=GetIndexes(next->previous);
-      for (y=0; y < (long) next->rows; y++)
-      {
-        SetMagickPixelPacket(p,indexes+x,&pixel);
-        SetMagickPixelPacket(q,next_indexes+x,&target);
-        if (IsMagickColorSimilar(&pixel,&target) == MagickFalse)
-          break;
-        p++;
-        q++;
-      }
-      if (y < (long) next->rows)
-        break;
-    }
-    bounds[i].x=x;
-    for (y=0; y < (long) next->rows; y++)
-    {
-      p=AcquireImagePixels(next,0,y,next->columns,1,exception);
-      q=AcquireImagePixels(next->previous,0,y,next->previous->columns,1,
-        exception);
-      if ((p == (const PixelPacket *) NULL) ||
-          (q == (const PixelPacket *) NULL))
-        break;
-      indexes=GetIndexes(next);
-      next_indexes=GetIndexes(next->previous);
-      for (x=0; x < (long) next->columns; x++)
-      {
-        SetMagickPixelPacket(p,indexes+x,&pixel);
-        SetMagickPixelPacket(q,next_indexes+x,&target);
-        if (IsMagickColorSimilar(&pixel,&target) == MagickFalse)
-          break;
-        p++;
-        q++;
-      }
-      if (x < (long) next->columns)
-        break;
-    }
-    bounds[i].y=y;
-    for (x=(long) next->columns-1; x >= 0; x--)
-    {
-      p=AcquireImagePixels(next,x,0,1,next->rows,exception);
-      q=AcquireImagePixels(next->previous,x,0,1,next->previous->rows,exception);
-      if ((p == (const PixelPacket *) NULL) ||
-          (q == (const PixelPacket *) NULL))
-        break;
-      indexes=GetIndexes(next);
-      next_indexes=GetIndexes(next->previous);
-      for (y=0; y < (long) next->rows; y++)
-      {
-        SetMagickPixelPacket(p,indexes+x,&pixel);
-        SetMagickPixelPacket(q,next_indexes+x,&target);
-        if (IsMagickColorSimilar(&pixel,&target) == MagickFalse)
-          break;
-        p++;
-        q++;
-      }
-      if (y < (long) next->rows)
-        break;
-    }
-    bounds[i].width=(unsigned long) (x-bounds[i].x+1);
-    for (y=(long) next->rows-1; y >= 0; y--)
-    {
-      p=AcquireImagePixels(next,0,y,next->columns,1,exception);
-      q=AcquireImagePixels(next->previous,0,y,next->previous->columns,1,
-        exception);
-      if ((p == (const PixelPacket *) NULL) ||
-          (q == (const PixelPacket *) NULL))
-        break;
-      indexes=GetIndexes(next);
-      next_indexes=GetIndexes(next->previous);
-      for (x=0; x < (long) next->columns; x++)
-      {
-        SetMagickPixelPacket(p,indexes+x,&pixel);
-        SetMagickPixelPacket(q,next_indexes+x,&target);
-        if (IsMagickColorSimilar(&pixel,&target) == MagickFalse)
-          break;
-        p++;
-        q++;
-      }
-      if (x < (long) next->columns)
-        break;
-    }
-    bounds[i].height=(unsigned long) (y-bounds[i].y+1);
-    i++;
-  }
-  /*
-    Clone first image in sequence.
-  */
-  deconstruct_image=CloneImage(image,0,0,MagickTrue,exception);
-  if (deconstruct_image == (Image *) NULL)
-    {
-      bounds=(RectangleInfo *) RelinquishMagickMemory(bounds);
-      return((Image *) NULL);
-    }
-  /*
-    Deconstruct the image sequence.
-  */
-  i=0;
-  next=GetNextImageInList(image);
-  for ( ; next != (const Image *) NULL; next=GetNextImageInList(next))
-  {
-    crop_image=CloneImage(next,0,0,MagickTrue,exception);
-    if (crop_image == (Image *) NULL)
-      break;
-    crop_next=CropImage(crop_image,&bounds[i],exception);
-    crop_image=DestroyImage(crop_image);
-    if (crop_next == (Image *) NULL)
-      break;
-    AppendImageToList(&deconstruct_image,crop_next);
-    i++;
-  }
-  bounds=(RectangleInfo *) RelinquishMagickMemory(bounds);
-  if (next != (Image *) NULL)
-    {
-      deconstruct_image=DestroyImageList(deconstruct_image);
-      return((Image *) NULL);
-    }
-  return(GetFirstImageInList(deconstruct_image));
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
 %     F l a t t e n I m a g e                                                 %
 %                                                                             %
 %                                                                             %
@@ -1521,7 +1195,11 @@ MagickExport Image *SpliceImage(const Image *image,
   if (splice_image == (Image *) NULL)
     return((Image *) NULL);
   if (SetImageStorageClass(splice_image,DirectClass) == MagickFalse)
-    return((Image *) NULL);
+    {
+      InheritException(exception,&splice_image->exception);
+      splice_image=DestroyImage(splice_image);
+      return((Image *) NULL);
+    }
   /*
     Splice image.
   */
@@ -1634,7 +1312,8 @@ MagickExport Image *SpliceImage(const Image *image,
 %   T r a n s f o r m I m a g e                                               %
 %                                                                             %
 %                                                                             %
-%                                                                             % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  TransformImage() is a convenience method that behaves like ResizeImage() or
 %  CropImage() but accepts scaling and/or cropping information as a region
@@ -1690,8 +1369,7 @@ MagickExport MagickBooleanType TransformImage(Image **image,
       crop_image=NewImageList();
       flags=ParseGravityGeometry(transform_image,crop_geometry,&geometry);
       if (((geometry.width == 0) && (geometry.height == 0)) ||
-          ((flags & XValue) != 0) || ((flags & YValue) != 0) ||
-          ((flags & PercentValue) != 0))
+          ((flags & XValue) != 0) || ((flags & YValue) != 0))
         {
           crop_image=CropImage(transform_image,&geometry,&(*image)->exception);
           if ((crop_image != (Image *) NULL) && ((flags & AspectValue) != 0))
@@ -1843,6 +1521,209 @@ MagickExport MagickBooleanType TransformImages(Image **images,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   T r a n s p o s e I m a g e                                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  TransposeImage() creates a horizontal mirror image by reflecting the pixels
+%  around the central y-axis while rotating them by 90 degrees.
+%
+%  The format of the TransposeImage method is:
+%
+%      Image *TransposeImage(const Image *image,ExceptionInfo *exception)
+%
+%  A description of each parameter follows:
+%
+%    o image: The image.
+%
+%    o exception: Return any errors or warnings in this structure.
+%
+%
+*/
+MagickExport Image *TransposeImage(const Image *image,ExceptionInfo *exception)
+{
+#define TransposeImageTag  "Transpose/Image"
+
+  Image
+    *transpose_image;
+
+  long
+    y;
+
+  MagickBooleanType
+    status;
+
+  RectangleInfo
+    page;
+
+  register const PixelPacket
+    *p;
+
+  register IndexPacket
+    *transpose_indexes,
+    *indexes;
+
+  register PixelPacket
+    *q;
+
+  /*
+    Initialize transpose image attributes.
+  */
+  assert(image != (const Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickSignature);
+  transpose_image=CloneImage(image,image->rows,image->columns,MagickTrue,
+    exception);
+  if (transpose_image == (Image *) NULL)
+    return((Image *) NULL);
+  for (y=0; y < (long) image->rows; y++)
+  {
+    p=AcquireImagePixels(image,0,image->rows-y-1,image->columns,1,exception);
+    q=SetImagePixels(transpose_image,(long) (image->rows-y-1),0,1,
+      transpose_image->rows);
+    if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
+      break;
+    (void) CopyMagickMemory(q,p,(size_t) image->columns*sizeof(*q));
+    indexes=GetIndexes(image);
+    transpose_indexes=GetIndexes(transpose_image);
+    if ((indexes != (IndexPacket *) NULL) &&
+        (transpose_indexes != (IndexPacket *) NULL))
+      (void) CopyMagickMemory(transpose_indexes,indexes,(size_t)
+        image->columns*sizeof(*transpose_indexes));
+    if (SyncImagePixels(transpose_image) == MagickFalse)
+      break;
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(TransposeImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+  }
+  page=transpose_image->page;
+  Swap(page.width,page.height);
+  Swap(page.x,page.y);
+  if (page.width != 0)
+    page.x=(long) (page.width-transpose_image->columns-page.x);
+  transpose_image->page=page;
+  return(transpose_image);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   T r a n s v e r s e I m a g e                                             %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  TransverseImage() creates a vertical mirror image by reflecting the pixels
+%  around the central x-axis while rotating them by 270 degrees.
+%
+%  The format of the TransverseImage method is:
+%
+%      Image *TransverseImage(const Image *image,ExceptionInfo *exception)
+%
+%  A description of each parameter follows:
+%
+%    o image: The image.
+%
+%    o exception: Return any errors or warnings in this structure.
+%
+%
+*/
+MagickExport Image *TransverseImage(const Image *image,ExceptionInfo *exception)
+{
+#define TransverseImageTag  "Transverse/Image"
+
+  Image
+    *transverse_image;
+
+  long
+    y;
+
+  MagickBooleanType
+    status;
+
+  RectangleInfo
+    page;
+
+  register const PixelPacket
+    *p;
+
+  register IndexPacket
+    *transverse_indexes,
+    *indexes;
+
+  register long
+    x;
+
+  register PixelPacket
+    *q;
+
+  /*
+    Initialize transverse image attributes.
+  */
+  assert(image != (const Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickSignature);
+  transverse_image=CloneImage(image,image->rows,image->columns,MagickTrue,
+    exception);
+  if (transverse_image == (Image *) NULL)
+    return((Image *) NULL);
+  for (y=0; y < (long) image->rows; y++)
+  {
+    p=AcquireImagePixels(image,0,y,image->columns,1,exception);
+    q=SetImagePixels(transverse_image,(long) (image->rows-y-1),0,1,
+      transverse_image->rows);
+    if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
+      break;
+    q+=image->columns;
+    for (x=0; x < (long) image->columns; x++)
+      *--q=(*p++);
+    indexes=GetIndexes(image);
+    transverse_indexes=GetIndexes(transverse_image);
+    if ((indexes != (IndexPacket *) NULL) &&
+        (transverse_indexes != (IndexPacket *) NULL))
+      for (x=0; x < (long) image->columns; x++)
+        transverse_indexes[image->columns-x-1]=indexes[x];
+    if (SyncImagePixels(transverse_image) == MagickFalse)
+      break;
+    if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+        (QuantumTick(y,image->rows) != MagickFalse))
+      {
+        status=image->progress_monitor(TransposeImageTag,y,image->rows,
+          image->client_data);
+        if (status == MagickFalse)
+          break;
+      }
+  }
+  page=transverse_image->page;
+  Swap(page.width,page.height);
+  Swap(page.x,page.y);
+  if (page.height != 0)
+    page.y=(long) (page.height-transverse_image->rows-page.y);
+  transverse_image->page=page;
+  return(transverse_image);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   T r i m I m a g e                                                         %
 %                                                                             %
 %                                                                             %
@@ -1863,7 +1744,6 @@ MagickExport MagickBooleanType TransformImages(Image **images,
 %
 %    o exception: Return any errors or warnings in this structure.
 %
-%
 */
 MagickExport Image *TrimImage(const Image *image,ExceptionInfo *exception)
 {
