@@ -109,6 +109,116 @@ MagickExport MagickBooleanType AcquireUniqueFilename(char *path)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   A c q u i r e U n i q u e S ym b o l i c L i n k                          %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  AcquireUniqueSymbolicLink() creates a unique symbolic link to the specified
+%  source path and returns MagickTrue on success otherwise MagickFalse.  If the
+%  symlink() method fails or is not available, a unique file name is generated
+%  and the source file copied to it.  When you are finished with the file, use
+%  RelinquishUniqueFilename() to destroy it.
+%
+%  The format of the AcquireUniqueSymbolicLink method is:
+%
+%      MagickBooleanType AcquireUniqueSymbolicLink(const char *source,
+%        char destination)
+%
+%  A description of each parameter follows.
+%
+%   o  source:  the source path.
+%
+%   o  destination:  the destination path.
+%
+*/
+MagickExport MagickBooleanType AcquireUniqueSymbolicLink(const char *source,
+  char *destination)
+{
+  int
+    destination_file,
+    source_file;
+
+  size_t
+    length;
+
+  ssize_t
+    count;
+
+  unsigned char
+    *buffer;
+
+  assert(source != (const char *) NULL);
+  assert(destination != (char *) NULL);
+#if defined(HAVE_SYMLINK)
+  (void) AcquireUniqueFilename(destination);
+  (void) RelinquishUniqueFileResource(destination);
+  if (*source == *DirectorySeparator)
+    {
+      if (symlink(source,destination) == 0)
+        return(MagickTrue);
+    }
+  else
+    {
+      char
+        path[MaxTextExtent];
+
+      *path='\0';
+      if (getcwd(path,MaxTextExtent) == (char *) NULL)
+        return(MagickFalse);
+      (void) ConcatenateMagickString(path,DirectorySeparator,MaxTextExtent);
+      (void) ConcatenateMagickString(path,source,MaxTextExtent);
+      if (symlink(path,destination) == 0)
+        return(MagickTrue);
+    }
+#endif
+  destination_file=AcquireUniqueFileResource(destination);
+  if (destination_file == -1)
+    return(MagickFalse);
+  source_file=open(source,O_RDONLY | O_BINARY);
+  if (source_file == -1)
+    {
+      (void) close(destination_file);
+      (void) RelinquishUniqueFileResource(destination);
+      return(MagickFalse);
+    }
+  buffer=(unsigned char *) AcquireMagickMemory(MagickMaxBufferSize*
+    sizeof(*buffer));
+  if (buffer == (unsigned char *) NULL)
+    {
+      (void) close(source_file);
+      (void) close(destination_file);
+      (void) RelinquishUniqueFileResource(destination);
+      return(MagickFalse);
+    }
+  for (length=0; ; )
+  {
+    count=read(source_file,buffer,MagickMaxBufferSize);
+    if (count <= 0)
+      break;
+    length=(size_t) count;
+    count=write(destination_file,buffer,length);
+    if ((size_t) count != length)
+      {
+        (void) close(destination_file);
+        (void) close(source_file);
+        buffer=(unsigned char *) RelinquishMagickMemory(buffer);
+        (void) RelinquishUniqueFileResource(destination);
+        return(MagickFalse);
+      }
+  }
+  (void) close(destination_file);
+  (void) close(source_file);
+  buffer=(unsigned char *) RelinquishMagickMemory(buffer);
+  return(MagickTrue);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %  A p p e n d I m a g e F o r m a t                                          %
 %                                                                             %
 %                                                                             %
@@ -472,19 +582,23 @@ MagickExport void ExpandFilename(char *path)
   (void) CopyMagickString(expanded_path,path,MaxTextExtent);
   if ((*(path+1) == *DirectorySeparator) || (*(path+1) == '\0'))
     {
-      register const char
-        *p;
+      char
+        *home;
 
       /*
         Substitute ~ with $HOME.
       */
-      p=getenv("HOME");
-      if (p == (const char *) NULL)
-        p=getenv("USERPROFILE");
-      if (p == (const char *) NULL)
-        p=".";
-      (void) CopyMagickString(expanded_path,p,MaxTextExtent);
+      (void) CopyMagickString(expanded_path,".",MaxTextExtent);
       (void) ConcatenateMagickString(expanded_path,path+1,MaxTextExtent);
+      home=GetEnvironmentValue("HOME");
+      if (home == (char *) NULL)
+        home=GetEnvironmentValue("USERPROFILE");
+      if (home != (char *) NULL)
+        {
+          (void) CopyMagickString(expanded_path,home,MaxTextExtent);
+          (void) ConcatenateMagickString(expanded_path,path+1,MaxTextExtent);
+          home=(char *) RelinquishMagickMemory(home);
+        }
     }
   else
     {
@@ -546,7 +660,6 @@ MagickExport void ExpandFilename(char *path)
 %    o argv: Specifies a pointer to a text array containing the command line
 %      arguments.
 %
-%
 */
 MagickExport MagickBooleanType ExpandFilenames(int *argc,char ***argv)
 {
@@ -587,7 +700,8 @@ MagickExport MagickBooleanType ExpandFilenames(int *argc,char ***argv)
   /*
     Expand any wildcard filenames.
   */
-  (void) getcwd(home_directory,MaxTextExtent);
+  if (getcwd(home_directory,MaxTextExtent) == (char *) NULL)
+    return(MagickFalse);
   count=0;
   for (i=0; i < (long) *argc; i++)
   {
@@ -674,6 +788,22 @@ MagickExport MagickBooleanType ExpandFilenames(int *argc,char ***argv)
     filelist=(char **) RelinquishMagickMemory(filelist);
   }
   vector[count]=(char *) NULL;
+  if (IsEventLogging() != MagickFalse)
+    {
+      char
+        *command_line;
+
+      command_line=AcquireString(vector[0]);
+      for (i=1; i < count; i++)
+      {
+        (void) ConcatenateString(&command_line,"{");
+        (void) ConcatenateString(&command_line,vector[i]);
+        (void) ConcatenateString(&command_line,"}");
+      }
+      (void) LogMagickEvent(ConfigureEvent,GetMagickModule(),
+        "Command line: %s",command_line);
+      command_line=(char *) RelinquishMagickMemory(command_line);
+    }
   *argc=(int) count;
   *argv=vector;
   return(MagickTrue);
@@ -684,84 +814,6 @@ MagickExport MagickBooleanType ExpandFilenames(int *argc,char ***argv)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%  F o r m a t S i z e                                                        %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  FormatSize() converts a size to a human readable format, for example,
-%  14kb, 234mb, 2.7gb, or 3.0tb.  Scaling is done by repetitively dividing by
-%  1024.
-%
-%  The format of the FormatSize method is:
-%
-%      void FormatSize(const MagickSizeType size,char *format)
-%
-%  A description of each parameter follows:
-%
-%    o size:  convert this size to a human readable format.
-%
-%    o format:  human readable format.
-%
-%
-*/
-MagickExport void FormatSize(const MagickSizeType size,char *format)
-{
-  double
-    length;
-
-  register long
-    i;
-
-  length=(double) size;
-  for (i=0; length > 1024.0; i++)
-    length/=1024.0;
-  switch (i)
-  {
-    case 0:
-    {
-      (void) FormatMagickString(format,MaxTextExtent,"%g",length);
-      break;
-    }
-    case 1:
-    {
-      (void) FormatMagickString(format,MaxTextExtent,"%.0gkb",length);
-      break;
-    }
-    case 2:
-    {
-      (void) FormatMagickString(format,MaxTextExtent,"%.1gmb",length);
-      break;
-    }
-    case 3:
-    {
-      (void) FormatMagickString(format,MaxTextExtent,"%.2ggb",length);
-      break;
-    }
-    case 4:
-    {
-      (void) FormatMagickString(format,MaxTextExtent,"%.3gtb",length);
-      break;
-    }
-    case 5:
-    {
-      (void) FormatMagickString(format,MaxTextExtent,"%.4gpb",length);
-      break;
-    }
-    default:
-    {
-      (void) FormatMagickString(format,MaxTextExtent,"%.5feb",length);
-      break;
-    }
-  }
-}
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
 %   G e t E x e c u t i o n P a t h                                           %
 %                                                                             %
 %                                                                             %
@@ -783,7 +835,8 @@ MagickExport void FormatSize(const MagickSizeType size,char *format)
 MagickExport MagickBooleanType GetExecutionPath(char *path)
 {
   *path='\0';
-  (void) getcwd(path,MaxTextExtent);
+  if (getcwd(path,MaxTextExtent) == (char *) NULL)
+    return(MagickFalse);
 #if defined(__WINDOWS__)
   return(NTGetExecutionPath(path));
 #endif
@@ -796,7 +849,7 @@ MagickExport MagickBooleanType GetExecutionPath(char *path)
     if (execution_path != (const char *) NULL)
       {
         if (*execution_path != *DirectorySeparator)
-          (void) ConcatenateMagickString(path,"/",MaxTextExtent);
+          (void) ConcatenateMagickString(path,DirectorySeparator,MaxTextExtent);
         (void) ConcatenateMagickString(path,execution_path,MaxTextExtent);
       }
   }
@@ -970,6 +1023,89 @@ MagickExport void GetPathComponent(const char *path,PathType type,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%  G e t P a t h C o m p o n e n t s                                          %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetPathComponents() returns a list of path components.
+%
+%  The format of the GetPathComponents method is:
+%
+%      char **GetPathComponents(const char *path,
+%        unsigned long *number_componenets)
+%
+%  A description of each parameter follows:
+%
+%    o path:  Specifies the string to segment into a list.
+%
+%    o number_components:  return the number of components in the list
+%
+*/
+MagickExport char **GetPathComponents(const char *path,
+  unsigned long *number_components)
+{
+  char
+    **components;
+
+  register char
+    *q;
+
+  register const char
+    *p;
+
+  register long
+    i;
+
+  if (path == (char *) NULL)
+    return((char **) NULL);
+  *number_components=1;
+  for (p=path; *p != '\0'; p++)
+    if (IsBasenameSeparator(*p))
+      (*number_components)++;
+  components=(char **)
+    AcquireMagickMemory((size_t) (*number_components+1)*sizeof(*components));
+  if (components == (char **) NULL)
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowMagickFatalException(ResourceLimitFatalError,
+        "memory allocation failed `%s'",message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
+  p=path;
+  for (i=0; i < (long) *number_components; i++)
+  {
+    for (q=(char *) p; *q != '\0'; q++)
+      if (IsBasenameSeparator(*q))
+        break;
+    components[i]=(char *)
+      AcquireMagickMemory(((q-p)+MaxTextExtent)*sizeof(*components));
+    if (components[i] == (char *) NULL)
+      {
+        char
+          *message;
+
+        message=GetExceptionMessage(errno);
+        ThrowMagickFatalException(ResourceLimitFatalError,
+          "MemoryAllocationFailed",message);
+        message=(char *) RelinquishMagickMemory(message);
+      }
+    (void) CopyMagickString(components[i],p,(size_t) (q-p+1));
+    p=q+1;
+  }
+  components[i]=(char *) NULL;
+  return(components);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %  I s A c c e s s i b l e                                                    %
 %                                                                             %
 %                                                                             %
@@ -1062,6 +1198,47 @@ static int IsDirectory(const char *path)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   I s M a g i c k T r u e                                                   %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  IsMagickTrue() returns MagickTrue if the value is "true", "on", "yes" or
+%  "1".
+%
+%  The format of the IsMagickTrue method is:
+%
+%      MagickBooleanType IsMagickTrue(const char *value)
+%
+%  A description of each parameter follows:
+%
+%    o option: either MagickTrue or MagickFalse depending on the value
+%      parameter.
+%
+%    o value: Specifies a pointer to a character array.
+%
+*/
+MagickExport MagickBooleanType IsMagickTrue(const char *value)
+{
+  if (value == (char *) NULL)
+    return(MagickFalse);
+  if (LocaleCompare(value,"true") == 0)
+    return(MagickTrue);
+  if (LocaleCompare(value,"on") == 0)
+    return(MagickTrue);
+  if (LocaleCompare(value,"yes") == 0)
+    return(MagickTrue);
+  if (LocaleCompare(value,"1") == 0)
+    return(MagickTrue);
+  return(MagickFalse);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   L i s t F i l e s                                                         %
 %                                                                             %
 %                                                                             %
@@ -1284,7 +1461,7 @@ MagickExport int SystemCommand(const MagickBooleanType verbose,
     status;
 
   if (verbose != MagickFalse)
-    (void) fputs(command,stdout);
+    (void) fprintf(stdout,"%s\n",command);
 #if defined(POSIX)
   status=system(command);
 #elif defined(__WINDOWS__)
@@ -1298,563 +1475,19 @@ MagickExport int SystemCommand(const MagickBooleanType verbose,
 #endif
   if (status < 0)
     {
+      char
+        *message;
+
       ExceptionInfo
         exception;
 
       GetExceptionInfo(&exception);
+      message=GetExceptionMessage(errno);
       (void) ThrowMagickException(&exception,GetMagickModule(),DelegateError,
-        "`%s': %s",command,strerror(errno));
+        "`%s': %s",command,message);
+      message=(char *) RelinquishMagickMemory(message);
       CatchException(&exception);
       (void) DestroyExceptionInfo(&exception);
     }
   return(status);
 }
-
-/*
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%   T r a n s l a t e T e x t                                                 %
-%                                                                             %
-%                                                                             %
-%                                                                             %
-%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-%
-%  TranslateText() replaces any embedded formatting characters with the
-%  appropriate image attribute and returns the translated text.
-%
-%  The format of the TranslateText method is:
-%
-%      char *TranslateText(const ImageInfo *image_info,Image *image,
-%        const char *embed_text)
-%
-%  A description of each parameter follows:
-%
-%    o image_info: The image info.
-%
-%    o image: The image.
-%
-%    o embed_text: The address of a character string containing the embedded
-%      formatting characters.
-%
-%
-*/
-MagickExport char *TranslateText(const ImageInfo *image_info,Image *image,
-  const char *embed_text)
-{
-  char
-    filename[MaxTextExtent],
-    *text,
-    *translate_text;
-
-  const ImageAttribute
-    *attribute;
-
-  ImageInfo
-    *text_info;
-
-  register char
-    *p,
-    *q;
-
-  register long
-    i;
-
-  size_t
-    length;
-
-  assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
-  if (image->debug != MagickFalse)
-    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  if ((embed_text == (const char *) NULL) || (*embed_text == '\0'))
-    return((char *) NULL);
-  text=(char *) embed_text;
-  if ((*text == '@') &&
-      ((*(text+1) == '-') || (IsAccessible(text+1) != MagickFalse)))
-    {
-      text=FileToString(text+1,~0,&image->exception);
-      if (text == (char *) NULL)
-        return((char *) NULL);
-    }
-  /*
-    Translate any embedded format characters.
-  */
-  text_info=CloneImageInfo(image_info);
-  translate_text=AcquireString(text);
-  length=strlen(text)+MaxTextExtent;
-  p=text;
-  for (q=translate_text; *p != '\0'; p++)
-  {
-    *q='\0';
-    if ((size_t) (q-translate_text+MaxTextExtent) >= length)
-      {
-        length<<=1;
-        translate_text=(char *) ResizeMagickMemory(translate_text,
-          (length+MaxTextExtent)*sizeof(*translate_text));
-        if (translate_text == (char *) NULL)
-          break;
-        q=translate_text+strlen(translate_text);
-      }
-    /*
-      Process formatting characters in text.
-    */
-    if ((*p == '\\') && (*(p+1) == 'r'))
-      {
-        *q++='\r';
-        p++;
-        continue;
-      }
-    if ((*p == '\\') && (*(p+1) == 'n'))
-      {
-        *q++='\n';
-        p++;
-        continue;
-      }
-    if (*p == '\\')
-      {
-        p++;
-        *q++=(*p++);
-        continue;
-      }
-    if (*p != '%')
-      {
-        *q++=(*p);
-        continue;
-      }
-    p++;
-    switch (*p)
-    {
-      case 'O':
-      {
-        (void) FormatMagickString(q,MaxTextExtent,"%+ld%+ld",image->page.x,
-          image->page.y);
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case 'P':
-      {
-        (void) FormatMagickString(q,MaxTextExtent,"%lux%lu",image->page.width,
-          image->page.height);
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case 'b':
-      {
-        char
-          format[MaxTextExtent];
-
-        MagickSizeType
-          length;
-
-        /*
-          File size.
-        */
-        length=GetBlobSize(image);
-        (void) FormatMagickString(format,MaxTextExtent,"%lu",
-          (unsigned long) length);
-        if (length != (MagickSizeType) ((size_t) length))
-          FormatSize(length,format);
-        (void) ConcatenateMagickString(q,format,MaxTextExtent);
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case 'c':
-      {
-        /*
-          Image comment.
-        */
-        attribute=GetImageAttribute(image,"Comment");
-        if (attribute == (const ImageAttribute *) NULL)
-          break;
-        (void) CopyMagickString(q,attribute->value,MaxTextExtent);
-        q+=strlen(attribute->value);
-        break;
-      }
-      case 'd':
-      case 'e':
-      case 'f':
-      case 't':
-      {
-        /*
-          Label segment is the base of the filename.
-        */
-        if (strlen(image->magick_filename) == 0)
-          break;
-        switch (*p)
-        {
-          case 'd':
-          {
-            /*
-              Directory.
-            */
-            GetPathComponent(image->magick_filename,HeadPath,filename);
-            (void) CopyMagickString(q,filename,MaxTextExtent);
-            q+=strlen(filename);
-            break;
-          }
-          case 'e':
-          {
-            /*
-              Filename extension.
-            */
-            GetPathComponent(image->magick_filename,ExtensionPath,filename);
-            (void) CopyMagickString(q,filename,MaxTextExtent);
-            q+=strlen(filename);
-            break;
-          }
-          case 'f':
-          {
-            /*
-              Filename.
-            */
-            GetPathComponent(image->magick_filename,TailPath,filename);
-            (void) CopyMagickString(q,filename,MaxTextExtent);
-            q+=strlen(filename);
-            break;
-          }
-          case 't':
-          {
-            /*
-              Base filename.
-            */
-            GetPathComponent(image->magick_filename,BasePath,filename);
-            (void) CopyMagickString(q,filename,MaxTextExtent);
-            q+=strlen(filename);
-            break;
-          }
-        }
-        break;
-      }
-      case 'g':
-      {
-        /*
-          Image geometry.
-        */
-        (void) FormatMagickString(q,MaxTextExtent,"%lux%lu%+ld%+ld",
-          image->page.width,image->page.height,image->page.x,image->page.y);
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case 'h':
-      {
-        /*
-          Image height.
-        */
-        (void) FormatMagickString(q,MaxTextExtent,"%lu",
-          image->rows != 0 ? image->rows : image->magick_rows);
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case 'i':
-      {
-        /*
-          Image filename.
-        */
-        (void) CopyMagickString(q,image->filename,MaxTextExtent);
-        q+=strlen(image->filename);
-        break;
-      }
-      case 'k':
-      {
-        /*
-          Number of unique colors.
-        */
-        (void) FormatMagickString(q,MaxTextExtent,"%lu",
-          GetNumberColors(image,(FILE *) NULL,&image->exception));
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case 'l':
-      {
-        /*
-          Image label.
-        */
-        attribute=GetImageAttribute(image,"label");
-        if (attribute == (const ImageAttribute *) NULL)
-          break;
-        (void) CopyMagickString(q,attribute->value,MaxTextExtent);
-        q+=strlen(attribute->value);
-        break;
-      }
-      case 'm':
-      {
-        /*
-          Image format.
-        */
-        (void) CopyMagickString(q,image->magick,MaxTextExtent);
-        q+=strlen(image->magick);
-        break;
-      }
-      case 'n':
-      {
-        /*
-          Number of images in the list.
-        */
-        (void) FormatMagickString(q,MaxTextExtent,"%lu",(unsigned long)
-          GetImageListLength(image));
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case 'o':
-      {
-        /*
-          Image output filename.
-        */
-        (void) CopyMagickString(q,text_info->filename,MaxTextExtent);
-        q+=strlen(text_info->filename);
-        break;
-      }
-      case 'p':
-      {
-        register const Image
-          *p;
-
-        unsigned long
-          page;
-
-        /*
-          Image page number.
-        */
-        p=image;
-        for (page=1; GetPreviousImageInList(p) != (Image *) NULL; page++)
-          p=GetPreviousImageInList(p);
-        (void) FormatMagickString(q,MaxTextExtent,"%lu",page);
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case 'q':
-      {
-        /*
-          Image depth.
-        */
-        (void) FormatMagickString(q,MaxTextExtent,"%lu",image->depth);
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case 'r':
-      {
-        ColorspaceType
-          colorspace;
-
-        /*
-          Image storage class and colorspace.
-        */
-        colorspace=image->colorspace;
-        if (IsGrayImage(image,&image->exception) != MagickFalse)
-          colorspace=GRAYColorspace;
-        (void) FormatMagickString(q,MaxTextExtent,"%s%s%s",
-          MagickOptionToMnemonic(MagickClassOptions,(long) image->storage_class),
-          MagickOptionToMnemonic(MagickColorspaceOptions,(long) colorspace),
-          image->matte != MagickFalse ? "Matte" : "");
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case 's':
-      {
-        /*
-          Image scene number.
-        */
-        (void) FormatMagickString(q,MaxTextExtent,"%lu",image->scene);
-        if (text_info->number_scenes != 0)
-          (void) FormatMagickString(q,MaxTextExtent,"%lu",text_info->scene);
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case 'u':
-      {
-        /*
-          Unique filename.
-        */
-        (void) CopyMagickString(filename,text_info->unique,MaxTextExtent);
-        (void) CopyMagickString(q,filename,MaxTextExtent);
-        q+=strlen(filename);
-        break;
-      }
-      case 'w':
-      {
-        /*
-          Image width.
-        */
-        (void) FormatMagickString(q,MaxTextExtent,"%lu",
-          image->columns != 0 ? image->columns : image->magick_columns);
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case 'x':
-      {
-        /*
-          Image horizontal resolution.
-        */
-        (void) FormatMagickString(q,MaxTextExtent,"%g %s",image->x_resolution,
-          MagickOptionToMnemonic(MagickResolutionOptions,(long) image->units));
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case 'y':
-      {
-        /*
-          Image vertical resolution.
-        */
-        (void) FormatMagickString(q,MaxTextExtent,"%g %s",image->y_resolution,
-          MagickOptionToMnemonic(MagickResolutionOptions,(long) image->units));
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case 'z':
-      {
-        /*
-          Unique filename.
-        */
-        (void) FormatMagickString(q,MaxTextExtent,"%lu",
-          GetImageDepth(image,&image->exception));
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case '[':
-      {
-        char
-          key[MaxTextExtent];
-
-        MagickOffsetType
-          offset;
-
-        /*
-          Image attribute.
-        */
-        if (strchr(p,']') == (char *) NULL)
-          break;
-        p++;
-        for (i=0; (i < (long) MaxTextExtent) && (*p != ']'); i++)
-          key[i]=(*p++);
-        key[i]='\0';
-        attribute=GetImageAttribute(image,key);
-        if (attribute != (const ImageAttribute *) NULL)
-          {
-            offset=(MagickOffsetType) strlen(attribute->value);
-            if ((size_t) (q-translate_text+offset+1) >= length)
-              {
-                length+=offset;
-                translate_text=(char *) ResizeMagickMemory(translate_text,
-                  (length+MaxTextExtent)*sizeof(*translate_text));
-                if (translate_text == (char *) NULL)
-                  break;
-                q=translate_text+strlen(translate_text);
-              }
-            (void) strcpy(q,attribute->value);
-            q+=offset;
-            break;
-          }
-        else
-          if (IsGlob(key) != MagickFalse)
-            {
-              /*
-                Iterate over image attributes.
-              */
-              ResetImageAttributeIterator(image);
-              attribute=GetNextImageAttribute(image);
-              while (attribute != (const ImageAttribute *) NULL)
-              {
-                if (GlobExpression(attribute->key,key) != MagickFalse)
-                  {
-                    offset=(MagickOffsetType) strlen(attribute->key)+
-                      strlen(attribute->value)+2;
-                    if ((size_t) (q-translate_text+offset+1) >= length)
-                      {
-                        length+=offset;
-                        translate_text=(char *) ResizeMagickMemory(
-                          translate_text,(length+MaxTextExtent)*
-                          sizeof(*translate_text));
-                        if (translate_text == (char *) NULL)
-                          break;
-                        q=translate_text+strlen(translate_text);
-                      }
-                    (void) FormatMagickString(q,MaxTextExtent,"%s=%s\n",
-                      attribute->key,attribute->value);
-                    q+=offset;
-                  }
-                attribute=GetNextImageAttribute(image);
-              }
-            }
-        attribute=GetImageInfoAttribute(text_info,image,key);
-        if (attribute == (const ImageAttribute *) NULL)
-          break;
-        offset=(MagickOffsetType) strlen(attribute->value);
-        if ((size_t) (q-translate_text+offset+1) >= length)
-          {
-            length+=offset;
-            translate_text=(char *) ResizeMagickMemory(translate_text,
-              (length+MaxTextExtent)*sizeof(*translate_text));
-            if (translate_text == (char *) NULL)
-              break;
-            q=translate_text+strlen(translate_text);
-          }
-        (void) strcpy(q,attribute->value);
-        q+=offset;
-        break;
-      }
-      case '@':
-      {
-        RectangleInfo
-          page;
-
-        /*
-          Image bounding box.
-        */
-        page=GetImageBoundingBox(image,&image->exception);
-        (void) FormatMagickString(q,MaxTextExtent,"%lux%lu%+ld%+ld",
-          page.width,page.height,page.x,page.y);
-        while (*q != '\0')
-          q++;
-        break;
-      }
-      case '#':
-      {
-        /*
-          Image signature.
-        */
-        (void) SignatureImage(image);
-        attribute=GetImageAttribute(image,"Signature");
-        if (attribute == (const ImageAttribute *) NULL)
-          break;
-        (void) CopyMagickString(q,attribute->value,MaxTextExtent);
-        q+=strlen(attribute->value);
-        break;
-      }
-      case '%':
-      {
-        *q++=(*p);
-        break;
-      }
-      default:
-      {
-        *q++='%';
-        *q++=(*p);
-        break;
-      }
-    }
-  }
-  *q='\0';
-  text_info=DestroyImageInfo(text_info);
-  if (text != (char *) embed_text)
-    text=(char *) RelinquishMagickMemory(text);
-  (void) SubstituteString(&translate_text,"&lt;","<");
-  (void) SubstituteString(&translate_text,"&gt;",">");
-  return(translate_text);
-}
