@@ -56,6 +56,7 @@
 #include "magick/semaphore.h"
 #include "magick/string_.h"
 #include "magick/xml-tree.h"
+#include "magick/utility.h"
 
 /*
   Define declarations.
@@ -228,6 +229,274 @@ MagickExport XMLTreeInfo *AddChildToXMLTree(XMLTreeInfo *xml_info,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   A d d P a t h T o X M L T r e e                                           %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  AddPathToXMLTree() adds a child tag at an offset relative to the start of
+%  the parent tag's character content.  This method returns the child tag.
+%
+%  The format of the AddPathToXMLTree method is:
+%
+%      XMLTreeInfo *AddPathToXMLTree(XMLTreeInfo *xml_info,const char *path,
+%        const size_t offset)
+%
+%  A description of each parameter follows:
+%
+%    o xml_info: The xml info.
+%
+%    o path: The path.
+%
+%    o offset: The tag offset.
+%
+*/
+MagickExport XMLTreeInfo *AddPathToXMLTree(XMLTreeInfo *xml_info,
+  const char *path,const size_t offset)
+{
+  char
+    **components,
+    subnode[MaxTextExtent],
+    tag[MaxTextExtent];
+
+  long
+    j;
+
+  register long
+    i;
+
+  XMLTreeInfo
+    *child,
+    *node;
+
+  unsigned long
+    number_components;
+
+  assert(xml_info != (XMLTreeInfo *) NULL);
+  assert((xml_info->signature == MagickSignature) ||
+         (((XMLTreeRoot *) xml_info)->signature == MagickSignature));
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  node=xml_info;
+  components=GetPathComponents(path,&number_components);
+  if (components == (char **) NULL)
+    return((XMLTreeInfo *) NULL);
+  for (i=0; i < (long) number_components; i++)
+  {
+    GetPathComponent(components[i],SubimagePath,subnode);
+    GetPathComponent(components[i],CanonicalPath,tag);
+    child=GetXMLTreeChild(node,tag);
+    if (child == (XMLTreeInfo *) NULL)
+      child=AddChildToXMLTree(node,tag,offset);
+    node=child;
+    if (node == (XMLTreeInfo *) NULL)
+      break;
+    for (j=atol(subnode)-1; j > 0; j--)
+    {
+      node=GetXMLTreeOrdered(node);
+      if (node == (XMLTreeInfo *) NULL)
+        break;
+    }
+    if (node == (XMLTreeInfo *) NULL)
+      break;
+    components[i]=(char *) RelinquishMagickMemory(components[i]);
+  }
+  for ( ; i < (long) number_components; i++)
+    components[i]=(char *) RelinquishMagickMemory(components[i]);
+  components=(char **) RelinquishMagickMemory(components);
+  return(node);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   C a n o n i c a l X M L C o n t e n t                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  CanonicalXMLContent() converts text to canonical XML content by converting
+%  to UTF-8, substituting predefined entities, wrapping as CDATA, or encoding
+%  as base-64 as required.
+%
+%  The format of the CanonicalXMLContent method is:
+%
+%      char *CanonicalXMLContent(const char *content)
+%
+%  A description of each parameter follows:
+%
+%    o content: The content.
+%
+*/
+
+static unsigned char *ConvertLatin1ToUTF8(const unsigned char *content)
+{
+  register const unsigned char
+    *p;
+
+  register unsigned char
+    *q;
+
+  size_t
+    length;
+
+  unsigned char
+    *utf8;
+
+  unsigned int
+    c;
+
+  length=0;
+  for (p=content; *p != '\0'; p++)
+    length+=(*p & 0x80) != 0 ? 2 : 1;
+  utf8=(unsigned char *) AcquireMagickMemory((length+1)*sizeof(*utf8));
+  if (utf8 == (unsigned char *) NULL)
+    return((unsigned char *) NULL);
+  q=utf8;
+  for (p=content; *p != '\0'; p++)
+  {
+    c=(*p);
+    if ((c & 0x80) == 0)
+      *q++=c;
+    else
+      {
+        *q++=0xc0 | ((c >> 6) & 0x3f);
+        *q++=0x80 | (c & 0x3f);
+      }
+  }
+  *q='\0';
+  return(utf8);
+}
+
+MagickExport char *CanonicalXMLContent(const char *content,
+  const MagickBooleanType pedantic)
+{
+  char
+    *base64,
+    *canonical_content;
+
+  register const unsigned char
+    *p;
+
+  register long
+    i;
+  size_t
+    length;
+
+  unsigned char
+    *utf8;
+
+  unsigned long
+    extent;
+
+  utf8=ConvertLatin1ToUTF8((const unsigned char *) content);
+  if (utf8 == (unsigned char *) NULL)
+    return((char *) NULL);
+  for (p=utf8; *p != '\0'; p++)
+  {
+    if (*p < 0x07)
+      break;
+    if ((*p > 0x0d) && (*p < 0x20))
+      break;
+  }
+  if (*p != '\0')
+    {
+      /*
+        String is binary, base64-encode it.
+      */
+      base64=Base64Encode(utf8,strlen((char *) utf8),&length);
+      utf8=(unsigned char *) RelinquishMagickMemory(utf8);
+      if (base64 == (char *) NULL)
+        return((char *) NULL);
+      canonical_content=AcquireString("<base64>");
+      (void) ConcatenateString(&canonical_content,base64);
+      base64=(char *) RelinquishMagickMemory(base64);
+      (void) ConcatenateString(&canonical_content,"</base64>");
+      return(canonical_content);
+    }
+  /*
+    Substitute predefined entities.
+  */
+  i=0;
+  extent=MaxTextExtent;
+  canonical_content=AcquireString((char *) NULL);
+  for (p=utf8; *p != '\0'; p++)
+  {
+    if ((i+MaxTextExtent) > (long) extent)
+      {
+        extent=i+MaxTextExtent;
+        canonical_content=(char *) ResizeMagickMemory(canonical_content,
+          extent*sizeof(*canonical_content));
+        if (canonical_content == (char *) NULL)
+          return(canonical_content);
+      }
+    switch (*p)
+    {
+      case '&':
+      {
+        i+=FormatMagickString(canonical_content+i,extent,"&amp;");
+        break;
+      }
+      case '<':
+      {
+        i+=FormatMagickString(canonical_content+i,extent,"&lt;");
+        break;
+      }
+      case '>':
+      {
+        i+=FormatMagickString(canonical_content+i,extent,"&gt;");
+        break;
+      }
+      case '"':
+      {
+        i+=FormatMagickString(canonical_content+i,extent,"&quot;");
+        break;
+      }
+      case '\n':
+      {
+        if (pedantic == MagickFalse)
+          {
+            canonical_content[i++]=(*p);
+            break;
+          }
+        i+=FormatMagickString(canonical_content+i,extent,"&#xA;");
+        break;
+      }
+      case '\t':
+      {
+        if (pedantic == MagickFalse)
+          {
+            canonical_content[i++]=(*p);
+            break;
+          }
+        i+=FormatMagickString(canonical_content+i,extent,"&#x9;");
+        break;
+      }
+      case '\r':
+      {
+        i+=FormatMagickString(canonical_content+i,extent,"&#xD;");
+        break;
+      }
+      default:
+      {
+        canonical_content[i++]=(*p);
+        break;
+      }
+    }
+  }
+  canonical_content[i]='\0';
+  utf8=(unsigned char *) RelinquishMagickMemory(utf8);
+  return(canonical_content);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   D e s t r o y X M L T r e e                                               %
 %                                                                             %
 %                                                                             %
@@ -248,9 +517,6 @@ MagickExport XMLTreeInfo *AddChildToXMLTree(XMLTreeInfo *xml_info,
 
 static char **DestroyXMLTreeAttributes(char **attributes)
 {
-  register char
-    *p;
-
   register long
     i;
 
@@ -259,17 +525,16 @@ static char **DestroyXMLTreeAttributes(char **attributes)
   */
   if ((attributes == (char **) NULL) || (attributes == sentinel))
     return((char **) NULL);
-  for (i=0; attributes[i] != (char *) NULL; i+=2);
-  p=attributes[i+1];
-  for (i=0; p[i] != (char) NULL; i++)
+  for (i=0; attributes[i] != (char *) NULL; i+=2)
   {
     /*
       Destroy attribute tag and value.
     */
-    attributes[2*i]=(char *) RelinquishMagickMemory(attributes[2*i]);
-    attributes[2*i+1]=(char *) RelinquishMagickMemory(attributes[2*i+1]);
+    if (attributes[i] != (char *) NULL)
+      attributes[i]=(char *) RelinquishMagickMemory(attributes[i]);
+    if (attributes[i+1] != (char *) NULL)
+      attributes[i+1]=(char *) RelinquishMagickMemory(attributes[i+1]);
   }
-  p=(char *) RelinquishMagickMemory(p);
   attributes=(char **) RelinquishMagickMemory(attributes);
   return((char **) NULL);
 }
@@ -508,6 +773,112 @@ MagickExport const char *GetXMLTreeContent(XMLTreeInfo *xml_info)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   G e t X M L T r e e O r d e r e d                                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetXMLTreeOrdered() returns the next ordered node if found, otherwise NULL.
+%
+%  The format of the GetXMLTreeOrdered method is:
+%
+%      XMLTreeInfo *GetXMLTreeOrdered(XMLTreeInfo *xml_info)
+%
+%  A description of each parameter follows:
+%
+%    o xml_info: The xml info.
+%
+*/
+MagickExport XMLTreeInfo *GetXMLTreeOrdered(XMLTreeInfo *xml_info)
+{
+  assert(xml_info != (XMLTreeInfo *) NULL);
+  assert((xml_info->signature == MagickSignature) ||
+         (((XMLTreeRoot *) xml_info)->signature == MagickSignature));
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  return(xml_info->ordered);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   G e t X M L T r e e P a t h                                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetXMLTreePath() traverses the XML-tree as defined by the specified path
+%  and returns the node if found, otherwise NULL.
+%
+%  The format of the GetXMLTreePath method is:
+%
+%      XMLTreeInfo *GetXMLTreePath(XMLTreeInfo *xml_info,const char *path)
+%
+%  A description of each parameter follows:
+%
+%    o xml_info: The xml info.
+%
+%    o path: The path (e.g. property/elapsed-time).
+%
+*/
+MagickExport XMLTreeInfo *GetXMLTreePath(XMLTreeInfo *xml_info,const char *path)
+{
+  char
+    **components,
+    subnode[MaxTextExtent],
+    tag[MaxTextExtent];
+
+  long
+    j;
+
+  register long
+    i;
+
+  XMLTreeInfo
+    *node;
+
+  unsigned long
+    number_components;
+
+  assert(xml_info != (XMLTreeInfo *) NULL);
+  assert((xml_info->signature == MagickSignature) ||
+         (((XMLTreeRoot *) xml_info)->signature == MagickSignature));
+  (void) LogMagickEvent(TraceEvent,GetMagickModule(),"...");
+  node=xml_info;
+  components=GetPathComponents(path,&number_components);
+  if (components == (char **) NULL)
+    return((XMLTreeInfo *) NULL);
+  for (i=0; i < (long) number_components; i++)
+  {
+    GetPathComponent(components[i],SubimagePath,subnode);
+    GetPathComponent(components[i],CanonicalPath,tag);
+    node=GetXMLTreeChild(node,tag);
+    if (node == (XMLTreeInfo *) NULL)
+      break;
+    for (j=atol(subnode)-1; j > 0; j--)
+    {
+      node=GetXMLTreeOrdered(node);
+      if (node == (XMLTreeInfo *) NULL)
+        break;
+    }
+    if (node == (XMLTreeInfo *) NULL)
+      break;
+    components[i]=(char *) RelinquishMagickMemory(components[i]);
+  }
+  for ( ; i < (long) number_components; i++)
+    components[i]=(char *) RelinquishMagickMemory(components[i]);
+  components=(char **) RelinquishMagickMemory(components);
+  return(node);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   G e t X M L T r e e P r o c e s s i n g I n s t r u c t i o n s           %
 %                                                                             %
 %                                                                             %
@@ -563,11 +934,11 @@ MagickExport const char **GetXMLTreeProcessingInstructions(
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  GetXMLTreeSibling() returns the tag sibling if found, otherwise NULL.
+%  GetXMLTreeSibling() returns the node sibling if found, otherwise NULL.
 %
 %  The format of the GetXMLTreeSibling method is:
 %
-%      XMLTreeInfo *GetXMLTreeSibling(XMLTreeInfo *xml_info,const char *tag)
+%      XMLTreeInfo *GetXMLTreeSibling(XMLTreeInfo *xml_info)
 %
 %  A description of each parameter follows:
 %
@@ -640,7 +1011,7 @@ MagickExport const char *GetXMLTreeTag(XMLTreeInfo *xml_info)
 %
 */
 
-static char *ConvertUTF16ToUTF8(const char *xml,size_t *length)
+static char *ConvertUTF16ToUTF8(const char *content,size_t *length)
 {
   char
     *utf8;
@@ -663,28 +1034,29 @@ static char *ConvertUTF16ToUTF8(const char *xml,size_t *length)
   utf8=(char *) AcquireMagickMemory((*length)*sizeof(*utf8));
   if (utf8 == (char *) NULL)
     return((char *) NULL);
-  encoding=(*xml == '\xFE') ? 1 : (*xml == '\xFF') ? 0 : -1;
+  encoding=(*content == '\xFE') ? 1 : (*content == '\xFF') ? 0 : -1;
   if (encoding == -1)
     {
       /*
         Already UTF-8.
       */
-      (void) CopyMagickMemory(utf8,xml,*length);
+      (void) CopyMagickMemory(utf8,content,*length);
       return(utf8);
     }
   j=0;
   max_length=(*length);
   for (i=2; i < (long) (*length-1); i+=2)
   {
-    c=(encoding != 0) ? ((xml[i] & 0xff) << 8) | (xml[i+1] & 0xff) :
-      ((xml[i+1] & 0xff) << 8) | (xml[i] & 0xff);
+    c=(encoding != 0) ? ((content[i] & 0xff) << 8) | (content[i+1] & 0xff) :
+      ((content[i+1] & 0xff) << 8) | (content[i] & 0xff);
     if ((c >= 0xd800) && (c <= 0xdfff) && ((i+=2) < (long) (*length-1)))
       {
-        byte=(encoding != 0) ? ((xml[i] & 0xff) << 8) | (xml[i+1] & 0xff) :
-          ((xml[i+1] & 0xff) << 8) | (xml[i] & 0xff);
+        byte=(encoding != 0) ? ((content[i] & 0xff) << 8) |
+          (content[i+1] & 0xff) : ((content[i+1] & 0xff) << 8) |
+          (content[i] & 0xff);
         c=(((c & 0x3ff) << 10) | (byte & 0x3ff))+0x10000;
       }
-    if ((j+MaxTextExtent) > (long) max_length)
+    if ((size_t) (j+MaxTextExtent) > max_length)
       {
         max_length=(size_t) j+MaxTextExtent;
         utf8=(char *) ResizeMagickMemory(utf8,max_length*sizeof(*utf8));
@@ -841,7 +1213,8 @@ static char *ParseEntities(char *xml,char **entities,char state)
                       xml=(char *) AcquireMagickMemory((size_t) j*sizeof(*xml));
                       if (xml != (char *) NULL)
                         {
-                          (void) strcpy(xml,p);
+                          (void) CopyMagickString(xml,p,(size_t) j*
+                            sizeof(*xml));
                           p=xml;
                         }
                     }
@@ -905,19 +1278,24 @@ void ParseCharacterContent(XMLTreeRoot *root,char *xml,const size_t length,
     }
   else
     {
+      size_t
+        length;
+
       offset=strlen(xml_info->content);
+      length=strlen(xml);
       xml_info->content=(char *) ResizeMagickMemory(xml_info->content,
-        (offset+strlen(xml)+1)*sizeof(*xml_info->content));
+        (offset+length+1)*sizeof(*xml_info->content));
       if (xml_info->content == (char *) NULL)
         ThrowMagickFatalException(ResourceLimitFatalError,
           "UnableToAcquireString",xml);
-      (void) strcpy(xml_info->content+offset,xml);
+      (void) CopyMagickString(xml_info->content+offset,xml,
+        (length+1)*sizeof(*xml_info->content));
       xml=(char *) RelinquishMagickMemory(xml);
   }
 }
 
-static XMLTreeInfo *ParseCloseTag(XMLTreeRoot *root,char *tag,char *xml,
-  ExceptionInfo *exception)
+static XMLTreeInfo *ParseCloseTag(XMLTreeRoot *root,char *tag,
+  char *magick_unused(xml),ExceptionInfo *exception)
 {
   if ((root->node == (XMLTreeInfo *) NULL) ||
       (root->node->tag == (char *) NULL) || (strcmp(tag,root->node->tag) != 0))
@@ -1030,8 +1408,8 @@ static void ParseProcessingInstructions(XMLTreeRoot *root,char *xml,
   if (root->processing_instructions[i][j+2] == (char *) NULL)
     ThrowMagickFatalException(ResourceLimitFatalError,"UnableToAcquireString",
       xml);
-  (void) strcpy(root->processing_instructions[i][j+2]+j-1,
-    root->root.tag != (char *) NULL ? ">" : "<");
+  (void) CopyMagickString(root->processing_instructions[i][j+2]+j-1,
+    root->root.tag != (char *) NULL ? ">" : "<",2);
   root->processing_instructions[i][j+1]=(char *) NULL;
   root->processing_instructions[i][j]=ConstantString(xml);
 }
@@ -1058,8 +1436,13 @@ static MagickBooleanType ParseInternalDoctype(XMLTreeRoot *root,char *xml,
   predefined_entitites=(char **) AcquireMagickMemory(sizeof(sentinel));
   if (predefined_entitites == (char **) NULL)
     {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
       (void) ThrowMagickException(exception,GetMagickModule(),
-        ResourceLimitError,"MemoryAllocationFailed","`%s'",strerror(errno));
+        ResourceLimitError,"MemoryAllocationFailed","`%s'",message);
+      message=(char *) RelinquishMagickMemory(message);
       return(MagickFalse);
     }
   (void) CopyMagickMemory(predefined_entitites,sentinel,sizeof(sentinel));
@@ -1203,7 +1586,7 @@ static MagickBooleanType ParseInternalDoctype(XMLTreeRoot *root,char *xml,
                       "UnableToAcquireString",xml);
                   root->attributes[i]=(char **) AcquireMagickMemory(2*
                     sizeof(**root->attributes));
-                  if ( root->attributes[i] == (char **) NULL)
+                  if (root->attributes[i] == (char **) NULL)
                     ThrowMagickFatalException(ResourceLimitFatalError,
                       "UnableToAcquireString",xml);
                   root->attributes[i][0]=ConstantString(t);
@@ -1219,7 +1602,6 @@ static MagickBooleanType ParseInternalDoctype(XMLTreeRoot *root,char *xml,
                     ThrowMagickFatalException(ResourceLimitFatalError,
                       "UnableToAcquireString",xml);
               }
-              root->attributes[i][j+3]=(char *)  NULL;
               root->attributes[i][j+2]=ConstantString(c);
               root->attributes[i][j+1]=(char *) NULL;
               if (v != (char *) NULL)
@@ -1311,8 +1693,13 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
   utf8=ConvertUTF16ToUTF8(xml,&length);
   if (utf8 == (char *) NULL)
     {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
       (void) ThrowMagickException(exception,GetMagickModule(),
-        ResourceLimitError,"MemoryAllocationFailed","`%s'",strerror(errno));
+        ResourceLimitError,"MemoryAllocationFailed","`%s'",message);
+      message=(char *) RelinquishMagickMemory(message);
       return((XMLTreeInfo *) NULL);
     }
   terminal=utf8[length-1];
@@ -1372,25 +1759,15 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
               "UnableToAcquireString",xml);
           if (attributes == (char **) NULL)
             {
+              char
+                *message;
+
+              message=GetExceptionMessage(errno);
               (void) ThrowMagickException(exception,GetMagickModule(),
-                ResourceLimitError,"MemoryAllocationFailed","`%s'",
-                strerror(errno));
+                ResourceLimitError,"MemoryAllocationFailed","`%s'",message);
+              message=(char *) RelinquishMagickMemory(message);
               return(&root->root);
             }
-          if (l == 0)
-            attributes[l+3]=(char *) AcquireMagickMemory(2*
-              sizeof(**attributes));
-          else
-            attributes[l+3]=(char *) ResizeMagickMemory(attributes[l+1],
-              (size_t) ((l/2)+2)*sizeof(**attributes));
-          if (attributes[l+3] == (char *) NULL)
-            {
-              (void) ThrowMagickException(exception,GetMagickModule(),
-                ResourceLimitError,"MemoryAllocationFailed","`%s'",
-                strerror(errno));
-              return(&root->root);
-            }
-          (void) strcpy(attributes[l+3]+(l/2)," ");
           attributes[l+2]=(char *) NULL;
           attributes[l+1]=(char *) NULL;
           attributes[l]=p;
@@ -1794,44 +2171,37 @@ MagickExport void SetXMLTreeAttribute(XMLTreeInfo *xml_info,const char *tag,
       if (value == (const char *) NULL)
         return;
       if (xml_info->attributes != sentinel)
-        {
-          xml_info->attributes=(char **) ResizeMagickMemory(
-            xml_info->attributes,(size_t) (i+4)*sizeof(*xml_info->attributes));
-          if (xml_info->attributes == (char **) NULL)
-            ThrowMagickFatalException(ResourceLimitFatalError,
-              "UnableToAcquireString",tag);
-        }
+        xml_info->attributes=(char **) ResizeMagickMemory(
+          xml_info->attributes,(size_t) (i+4)*sizeof(*xml_info->attributes));
       else
         {
           xml_info->attributes=(char **) AcquireMagickMemory(4*sizeof(
             *xml_info->attributes));
-          if (xml_info->attributes == (char **) NULL)
-            ThrowMagickFatalException(ResourceLimitFatalError,
-              "UnableToAcquireString",tag);
-          xml_info->attributes[1]=ConstantString("");
+          if (xml_info->attributes != (char **) NULL)
+            xml_info->attributes[1]=ConstantString("");
         }
+      if (xml_info->attributes == (char **) NULL)
+        ThrowMagickFatalException(ResourceLimitFatalError,
+          "UnableToAcquireString",tag);
       xml_info->attributes[i]=ConstantString(tag);
       xml_info->attributes[i+2]=(char *) NULL;
       length=strlen(xml_info->attributes[i+1]);
-      xml_info->attributes[i+3]=(char *) ResizeMagickMemory(
-        xml_info->attributes[i+1],(length+2)*sizeof(*xml_info->attributes));
-      if (xml_info->attributes[i+3] == (char *) NULL)
-        ThrowMagickFatalException(ResourceLimitFatalError,
-          "UnableToAcquireString",tag);
     }
   /*
     Add new value to an existing attribute.
   */
-  for (j=i; xml_info->attributes[j]; j+=2);
-  xml_info->attributes[i+1]=(char *) RelinquishMagickMemory(
-    xml_info->attributes[i+1]);
+  for (j=i; xml_info->attributes[j] != (char *) NULL; j+=2);
+  if (xml_info->attributes[i+1] != (char *) NULL)
+    xml_info->attributes[i+1]=(char *) RelinquishMagickMemory(
+      xml_info->attributes[i+1]);
   if (value != (const char *) NULL)
     {
       xml_info->attributes[i+1]=ConstantString(value);
       return;
     }
-  xml_info->attributes[i]=(char *) RelinquishMagickMemory(
-    xml_info->attributes[i]);
+  if (xml_info->attributes[i] != (char *) NULL)
+    xml_info->attributes[i]=(char *) RelinquishMagickMemory(
+      xml_info->attributes[i]);
   (void) CopyMagickMemory(xml_info->attributes+i,xml_info->attributes+i+2,
     (size_t) (j-i+2)*sizeof(*xml_info->attributes));
   xml_info->attributes=(char **) ResizeMagickMemory(xml_info->attributes,
@@ -1882,7 +2252,6 @@ MagickExport XMLTreeInfo *SetXMLTreeContent(XMLTreeInfo *xml_info,
   xml_info->content=(char *) ConstantString(content);
   return(xml_info);
 }
-
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -1907,89 +2276,29 @@ MagickExport XMLTreeInfo *SetXMLTreeContent(XMLTreeInfo *xml_info,
 %
 */
 
-static char *EncodePredefinedEntities(const char *source,size_t offset,
+static char *EncodePredefinedEntities(const char *source,ssize_t offset,
   char **destination,size_t *length,size_t *max_length,
   MagickBooleanType pedantic)
 {
-  const char
-    *p;
+  char
+    *content;
 
-  for (p=source+offset; p != source; source++)
-  {
-    if ((*length+MaxTextExtent) > *max_length)
-      {
-        *max_length=(*length)+MaxTextExtent;
-        *destination=(char *) ResizeMagickMemory(*destination,*max_length*
-          sizeof(**destination));
-        if (*destination == (char *) NULL)
-          return(*destination);
-      }
-    switch (*source)
+  if (offset == -1)
+    content=CanonicalXMLContent(source,pedantic);
+  else
+    content=CanonicalXMLContent(source+offset,pedantic);
+  if (content == (char *) NULL)
+    return(*destination);
+  if ((*length+strlen(content)+MaxTextExtent) > *max_length)
     {
-      case '\0':
+      *max_length=(*length)+strlen(content)+MaxTextExtent;
+      *destination=(char *) ResizeMagickMemory(*destination,*max_length*
+        sizeof(**destination));
+      if (*destination == (char *) NULL)
         return(*destination);
-      case '&':
-      {
-        *length+=FormatMagickString(*destination+(*length),*max_length,"&amp;");
-        break;
-      }
-      case '<':
-      {
-        *length+=FormatMagickString(*destination+(*length),*max_length,"&lt;");
-        break;
-      }
-      case '>':
-      {
-        *length+=FormatMagickString(*destination+(*length),*max_length,"&gt;");
-        break;
-      }
-      case '"':
-      {
-        if (pedantic == MagickFalse)
-          {
-            (*destination)[*length]=(*source);
-            (*length)++;
-            break;
-          }
-        *length+=FormatMagickString(*destination+(*length),*max_length,
-          "&quot;");
-        break;
-      }
-      case '\n':
-      {
-        if (pedantic == MagickFalse)
-          {
-            (*destination)[*length]=(*source);
-            (*length)++;
-            break;
-          }
-        *length+=FormatMagickString(*destination+(*length),*max_length,"&#xA;");
-        break;
-      }
-      case '\t':
-      {
-        if (pedantic == MagickFalse)
-          {
-            (*destination)[*length]=(*source);
-            (*length)++;
-            break;
-          }
-        *length+=FormatMagickString(*destination+(*length),*max_length,"&#x9;");
-        break;
-      }
-      case '\r':
-      {
-        *length+=FormatMagickString(*destination+(*length),*max_length,"&#xD;");
-        break;
-      }
-      default:
-      {
-        (*destination)[*length]=(*source);
-        (*length)++;
-        break;
-      }
     }
-  }
+  *length+=FormatMagickString(*destination+(*length),*max_length,"%s",content);
+  content=(char *) RelinquishMagickMemory(content);
   return(*destination);
 }
 
