@@ -196,8 +196,8 @@ int main ( int argc, char **argv )
    * Save image to BLOB
    */
   blob_length = 8192;
-  (void) strncpy( original->magick, format, MaxTextExtent-1 );
-  strcpy( imageInfo->filename, "" );
+  (void) strncpy(original->magick,format,MaxTextExtent-1);
+  (void) CopyMagickString(imageInfo->filename,"",MaxTextExtent);
   original->delay = 10;
   (void) LogMagickEvent(CoderEvent,GetMagickModule(),
     "Writing image to BLOB");
@@ -223,7 +223,7 @@ int main ( int argc, char **argv )
    * Read image back from BLOB
    */
   (void) strncpy( imageInfo->magick, format, MaxTextExtent-1 );
-  strcpy( imageInfo->filename, "" );
+  (void) CopyMagickString(imageInfo->filename,"",MaxTextExtent);
   if ( size[0] != '\0' )
     CloneString( &imageInfo->size, size );
   original = BlobToImage( imageInfo, blob, blob_length, &exception );
@@ -247,7 +247,7 @@ int main ( int argc, char **argv )
    */
   blob_length = 8192;
   (void) strncpy( original->magick, format, MaxTextExtent-1 );
-  strcpy( imageInfo->filename, "" );
+  (void) CopyMagickString(imageInfo->filename,"",MaxTextExtent);
   original->delay = 10;
   blob = (char *) ImageToBlob ( imageInfo, original, &blob_length, &exception );
   if (exception.severity >= ErrorException)
@@ -269,7 +269,7 @@ int main ( int argc, char **argv )
    * Read image back from BLOB
    */
   (void) strncpy( imageInfo->magick, format, MaxTextExtent-1 );
-  strcpy( imageInfo->filename, "" );
+  (void) CopyMagickString(imageInfo->filename,"",MaxTextExtent);
   if ( size[0] != '\0' )
     CloneString( &imageInfo->size, size );
   (void) LogMagickEvent(CoderEvent,GetMagickModule(),
@@ -328,7 +328,8 @@ int main ( int argc, char **argv )
      final =DestroyImage( final );
   final = (Image*)NULL;
 
-   imageInfo =DestroyImageInfo( imageInfo );
+  imageInfo =DestroyImageInfo( imageInfo );
+  DestroyExceptionInfo( &exception );
   DestroyMagick();
 
   if (pause)
