@@ -17,7 +17,7 @@
 %                               January 1993                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2005 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2006 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -45,7 +45,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
-#include "magick/ImageMagick.h"
+#include "wand/MagickWand.h"
 #if defined(__WINDOWS__)
 #include <windows.h>
 #endif
@@ -89,7 +89,8 @@ int main(int argc,char **argv)
       continue;
     if ((LocaleCompare("debug",option+1) == 0) && (i < (long) (argc-1)))
       (void) SetLogEventMask(argv[++i]);
-    if (LocaleCompare("version",option+1) == 0)
+    if ((LocaleCompare("version",option+1) == 0) ||
+        (LocaleCompare("-version",option+1) == 0))
       {
         (void) fprintf(stdout,"Version: %s\n",
           GetMagickVersion((unsigned long *) NULL));
@@ -99,12 +100,14 @@ int main(int argc,char **argv)
   }
   image_info=CloneImageInfo((ImageInfo *) NULL);
   status=CompositeImageCommand(image_info,argc,argv,(char **) NULL,&exception);
-  if ((status == MagickFalse) && (exception.severity != UndefinedException))
-    CatchException(&exception);
+  if ((status == MagickFalse) || (exception.severity != UndefinedException))
+    {
+      if (exception.severity < ErrorException)
+        status=MagickTrue;
+      CatchException(&exception);
+    }
   image_info=DestroyImageInfo(image_info);
-  DestroyExceptionInfo(&exception);
+  (void) DestroyExceptionInfo(&exception);
   DestroyMagick();
-  if ((status == MagickFalse) && (exception.severity > ErrorException))
-    return(1);
-  return(0);
+  return(status == MagickFalse ? 1 : 0);
 }
