@@ -130,12 +130,14 @@ int main(int argc,char **argv)
   }
   image_info=CloneImageInfo((ImageInfo *) NULL);
   status=DisplayImageCommand(image_info,argc,argv,(char **) NULL,&exception);
-  if ((status == MagickFalse) && (exception.severity != UndefinedException))
-    CatchException(&exception);
+  if ((status == MagickFalse) || (exception.severity != UndefinedException))
+    {
+      if (exception.severity < ErrorException)
+        status=MagickTrue;
+      CatchException(&exception);
+    }
   image_info=DestroyImageInfo(image_info);
   (void) DestroyExceptionInfo(&exception);
   DestroyMagick();
-  if ((status == MagickFalse) && (exception.severity > ErrorException))
-    return(1);
-  return(0);
+  return(status == MagickFalse ? 1 : 0);
 }
