@@ -188,7 +188,7 @@ static void AnimateUsage(void)
   exit(0);
 }
 
-MagickExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
+WandExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
   int argc,char **argv,char **wand_unused(metadata),ExceptionInfo *exception)
 {
 #if defined(HasX11)
@@ -334,10 +334,10 @@ MagickExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
     ParseMagickOption(MagickInterlaceOptions,MagickFalse,resource_value);
   resource_value=XGetResourceInstance(resource_database,GetClientName(),
     "verbose","False");
-  image_info->verbose=IsTrue(resource_value);
+  image_info->verbose=IsMagickTrue(resource_value);
   resource_value=XGetResourceInstance(resource_database,GetClientName(),
     "dither","True");
-  quantize_info->dither=IsTrue(resource_value);
+  quantize_info->dither=IsMagickTrue(resource_value);
   /*
     Parse command line.
   */
@@ -353,11 +353,11 @@ MagickExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
           option="logo:";
         else
           {
-            char
+            int
               c;
 
             c=getc(stdin);
-            if ((int) c == EOF)
+            if (c == EOF)
               option="logo:";
             else
               {
@@ -398,10 +398,19 @@ MagickExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
         MogrifyImageStack(image_stack[k],MagickTrue,pend);
         for (scene=first_scene; scene <= last_scene ; scene++)
         {
+          const char
+            *filename;
+
           /*
             Read image.
           */
-          (void) CopyMagickString(image_info->filename,option,MaxTextExtent);
+          filename=option;
+          if ((LocaleCompare(filename,"--") == 0) && (i < (argc-1)))
+            {
+              option=argv[++i];
+              filename=option;
+            }
+          (void) CopyMagickString(image_info->filename,filename,MaxTextExtent);
           if (first_scene != last_scene)
             {
               char
@@ -410,8 +419,8 @@ MagickExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
               /*
                 Form filename for multi-part images.
               */
-              (void) FormatMagickString(filename,MaxTextExtent,
-                image_info->filename,scene);
+              (void) InterpretImageFilename(filename,MaxTextExtent,
+                image_info->filename,(int) scene);
               if (LocaleCompare(filename,image_info->filename) == 0)
                 (void) FormatMagickString(filename,MaxTextExtent,"%s[%lu]",
                   image_info->filename,scene);
@@ -882,7 +891,7 @@ MagickExport MagickBooleanType AnimateImageCommand(ImageInfo *image_info,
             resource_info.map_type=(char *) NULL;
             if (*option == '+')
               break;
-            (void) strcpy(argv[i]+1,"{1}");
+            (void) CopyMagickString(argv[i]+1,"sans",MaxTextExtent);
             i++;
             if (i == (long) argc)
               ThrowAnimateException(OptionError,"MissingArgument",option);
