@@ -130,7 +130,7 @@ static void CompareUsage(void)
   exit(0);
 }
 
-MagickExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
+WandExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
   int argc,char **argv,char **metadata,ExceptionInfo *exception)
 {
 #define DestroyCompare() \
@@ -221,8 +221,15 @@ MagickExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
   ReadCommandlLine(argc,&argv);
   status=ExpandFilenames(&argc,&argv);
   if (status == MagickFalse)
-    ThrowCompareException(ResourceLimitError,"MemoryAllocationFailed",
-      strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowCompareException(ResourceLimitError,"MemoryAllocationFailed",
+        message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
   for (i=1; i < (long) (argc-1); i++)
   {
     option=argv[i];
@@ -255,6 +262,8 @@ MagickExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
         */
         MogrifyImageStack(image_stack[k],MagickTrue,pend);
         filename=argv[i];
+        if ((LocaleCompare(filename,"--") == 0) && (i < (argc-1)))
+          filename=argv[++i];
         (void) CopyMagickString(image_info->filename,filename,MaxTextExtent);
         image=ReadImage(image_info,exception);
         status&=(image != (Image *) NULL) &&
@@ -630,7 +639,7 @@ MagickExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
   }
   if (k != 0)
     ThrowCompareException(OptionError,"UnbalancedParenthesis",argv[i]);
-  if (i != (long) (argc-1))
+  if (i-- != (long) (argc-1))
     ThrowCompareException(OptionError,"MissingAnImageFilename",argv[i]);
   if ((image_stack[k] == (Image *) NULL) ||
       (GetImageListLength(image_stack[k]) < 2))
@@ -652,10 +661,17 @@ MagickExport MagickBooleanType CompareImageCommand(ImageInfo *image_info,
           char
             *text;
 
-          text=TranslateText(image_info,difference_image,format);
+          text=InterpretImageAttributes(image_info,difference_image,format);
           if (text == (char *) NULL)
-            ThrowCompareException(ResourceLimitError,"MemoryAllocationFailed",
-              strerror(errno));
+            {
+              char
+                *message;
+
+              message=GetExceptionMessage(errno);
+              ThrowCompareException(ResourceLimitError,"MemoryAllocationFailed",
+                message);
+              message=(char *) RelinquishMagickMemory(message);
+            }
           (void) ConcatenateString(&(*metadata),text);
           (void) ConcatenateString(&(*metadata),"\n");
           text=(char *) RelinquishMagickMemory(text);
