@@ -137,7 +137,7 @@ static void IdentifyUsage(void)
   exit(0);
 }
 
-MagickExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
+WandExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
   int argc,char **argv,char **metadata,ExceptionInfo *exception)
 {
 #define DestroyIdentify() \
@@ -212,8 +212,15 @@ MagickExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
   ReadCommandlLine(argc,&argv);
   status=ExpandFilenames(&argc,&argv);
   if (status == MagickFalse)
-    ThrowIdentifyException(ResourceLimitError,"MemoryAllocationFailed",
-      strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowIdentifyException(ResourceLimitError,"MemoryAllocationFailed",
+        message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
   for (i=1; i < (long) argc; i++)
   {
     option=argv[i];
@@ -241,6 +248,9 @@ MagickExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
       }
     if (IsMagickOption(option) == MagickFalse)
       {
+        char
+          *filename;
+
         Image
           *image;
 
@@ -253,7 +263,10 @@ MagickExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
         MogrifyImageStack(image_stack[k],MagickFalse,pend);
         identify_info=CloneImageInfo(image_info);
         identify_info->verbose=MagickFalse;
-        (void) CopyMagickString(identify_info->filename,argv[i],MaxTextExtent);
+        filename=argv[i];
+        if ((LocaleCompare(filename,"--") == 0) && (i < (argc-1)))
+          filename=argv[++i];
+        (void) CopyMagickString(identify_info->filename,filename,MaxTextExtent);
         if ((image_info->verbose != MagickFalse) ||
             (identify_info->ping == MagickFalse))
           image=ReadImage(identify_info,exception);
@@ -281,10 +294,17 @@ MagickExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
               char
                 *text;
 
-              text=TranslateText(image_info,image,format);
+              text=InterpretImageAttributes(image_info,image,format);
               if (text == (char *) NULL)
-                ThrowIdentifyException(ResourceLimitError,
-                  "MemoryAllocationFailed",strerror(errno));
+                {
+                  char
+                    *message;
+
+                  message=GetExceptionMessage(errno);
+                  ThrowIdentifyException(ResourceLimitError,
+                    "MemoryAllocationFailed",message);
+                  message=(char *) RelinquishMagickMemory(message);
+                }
               (void) ConcatenateString(&(*metadata),text);
               text=(char *) RelinquishMagickMemory(text);
               if (LocaleCompare(format,"%n") == 0)
