@@ -286,8 +286,15 @@ WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
   ReadCommandlLine(argc,&argv);
   status=ExpandFilenames(&argc,&argv);
   if (status == MagickFalse)
-    ThrowImportException(ResourceLimitError,"MemoryAllocationFailed",
-      strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowImportException(ResourceLimitError,"MemoryAllocationFailed",
+        message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
   for (i=1; i < (long) argc; i++)
   {
     /*
@@ -325,7 +332,7 @@ WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
   quantize_info=resource_info.quantize_info;
   resource_value=XGetResourceInstance(resource_database,GetClientName(),
     "border","False");
-  ximage_info.borders=IsTrue(resource_value);
+  ximage_info.borders=IsMagickTrue(resource_value);
   resource_value=XGetResourceInstance(resource_database,GetClientName(),
     "delay","0");
   resource_info.delay=(unsigned int) atoi(resource_value);
@@ -333,10 +340,10 @@ WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
     "density",(char *) NULL);
   resource_value=XGetResourceInstance(resource_database,GetClientName(),
     "descend","True");
-  ximage_info.descend=IsTrue(resource_value);
+  ximage_info.descend=IsMagickTrue(resource_value);
   resource_value=XGetResourceInstance(resource_database,GetClientName(),
     "frame","False");
-  ximage_info.frame=IsTrue(resource_value);
+  ximage_info.frame=IsMagickTrue(resource_value);
   resource_value=XGetResourceInstance(resource_database,GetClientName(),
     "interlace","none");
   image_info->interlace=UndefinedInterlace;
@@ -361,16 +368,16 @@ WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
   image_info->quality=(unsigned long) atol(resource_value);
   resource_value=XGetResourceInstance(resource_database,GetClientName(),
     "screen","False");
-  ximage_info.screen=IsTrue(resource_value);
+  ximage_info.screen=IsMagickTrue(resource_value);
   resource_value=XGetResourceInstance(resource_database,GetClientName(),
     "silent","False");
-  ximage_info.silent=IsTrue(resource_value);
+  ximage_info.silent=IsMagickTrue(resource_value);
   resource_value=XGetResourceInstance(resource_database,GetClientName(),
     "verbose","False");
-  image_info->verbose=IsTrue(resource_value);
+  image_info->verbose=IsMagickTrue(resource_value);
   resource_value=XGetResourceInstance(resource_database,GetClientName(),
     "dither","True");
-  quantize_info->dither=IsTrue(resource_value);
+  quantize_info->dither=IsMagickTrue(resource_value);
   snapshots=1;
   status=MagickTrue;
   filename=(char *) NULL;
@@ -428,7 +435,7 @@ WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
           if (image == (Image *) NULL)
             continue;
           (void) CopyMagickString(image->filename,filename,MaxTextExtent);
-          (void) strcpy(image->magick,"PS");
+          (void) CopyMagickString(image->magick,"PS",MaxTextExtent);
           image->scene=scene;
           AppendImageToList(&image_stack[k],image);
           MogrifyImageStack(image_stack[k],MagickFalse,MagickTrue);
@@ -463,7 +470,7 @@ WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
         if (LocaleCompare("border",option+1) == 0)
           {
             ximage_info.borders=(*option == '-') ? MagickTrue : MagickFalse;
-            (void) strcpy(argv[i]+1,"{0}");
+            (void) CopyMagickString(argv[i]+1,"sans0",MaxTextExtent);
             break;
           }
         if (LocaleCompare("bordercolor",option+1) == 0)
@@ -728,7 +735,7 @@ WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
         if (LocaleCompare("frame",option+1) == 0)
           {
             ximage_info.frame=(*option == '-') ? MagickTrue : MagickFalse;
-            (void) strcpy(argv[i]+1,"{0}");
+            (void) CopyMagickString(argv[i]+1,"sans0",MaxTextExtent);
             break;
           }
         if (LocaleCompare("format",option+1) == 0)
@@ -1012,7 +1019,7 @@ WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
           }
         if (LocaleCompare("snaps",option+1) == 0)
           {
-            (void) strcpy(argv[i]+1,"{1}");
+            (void) CopyMagickString(argv[i]+1,"sans",MaxTextExtent);
             i++;
             if (i == (long) argc)
               ThrowImportException(OptionError,"MissingArgument",option);
