@@ -690,6 +690,48 @@ WandExport MagickBooleanType MagickGetSize(const MagickWand *wand,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   M a g i c k G e t S i z e O f f s e t                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickGetSizeOffset() returns the size offset associated with the magick
+%  wand.
+%
+%  The format of the MagickGetSizeOffset method is:
+%
+%      MagickBooleanType MagickGetSizeOffset(const MagickWand *wand,
+%        long *offset)
+%
+%  A description of each parameter follows:
+%
+%    o wand: The magick wand.
+%
+%    o offset: The image offset.
+%
+*/
+WandExport MagickBooleanType MagickGetSizeOffset(const MagickWand *wand,
+  long *offset)
+{
+  RectangleInfo
+    geometry;
+
+  assert(wand != (const MagickWand *) NULL);
+  assert(wand->signature == WandSignature);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
+  (void) ResetMagickMemory(&geometry,0,sizeof(geometry));
+  (void) ParseAbsoluteGeometry(wand->image_info->size,&geometry);
+  *offset=geometry.x;
+  return(MagickTrue);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   M a g i c k G e t V e r s i o n                                           %
 %                                                                             %
 %                                                                             %
@@ -1277,7 +1319,7 @@ WandExport MagickBooleanType MagickSetSamplingFactors(MagickWand *wand,
 %
 %    o columns: The width in pixels.
 %
-%    o height: The height in pixels.
+%    o rows: The rows in pixels.
 %
 %
 */
@@ -1301,6 +1343,53 @@ WandExport MagickBooleanType MagickSetSize(MagickWand *wand,
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   M a g i c k S e t S i z e O f f s e t                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  MagickSetSizeOffset() sets the size and offset of the magick wand.  Set it
+%  before you read a raw image format such as RGB, GRAY, or CMYK.
+%
+%  The format of the MagickSetSizeOffset method is:
+%
+%      MagickBooleanType MagickSetSizeOffset(MagickWand *wand,
+%        const unsigned long columns,const unsigned long rows,
+%        const long offset)
+%
+%  A description of each parameter follows:
+%
+%    o wand: The magick wand.
+%
+%    o columns: The width in pixels.
+%
+%    o rows: The rows in pixels.
+%
+%    o height: The height in pixels.
+%
+*/
+WandExport MagickBooleanType MagickSetSizeOffset(MagickWand *wand,
+  const unsigned long columns,const unsigned long rows,const long offset)
+{
+  char
+    geometry[MaxTextExtent];
+
+  assert(wand != (MagickWand *) NULL);
+  assert(wand->signature == WandSignature);
+  if (wand->debug != MagickFalse)
+    (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
+  (void) FormatMagickString(geometry,MaxTextExtent,"%lux%lu%+ld",columns,rows,
+    offset);
+  (void) CloneString(&wand->image_info->size,geometry);
+  return(MagickTrue);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   M a g i c k S e t T y p e                                                 %
 %                                                                             %
 %                                                                             %
