@@ -313,8 +313,15 @@ WandExport PixelWand *NewPixelWand(void)
     ThrowWandFatalException(WandError,"QuantumDepthMismatch",quantum);
   wand=(struct _PixelWand *) AcquireMagickMemory(sizeof(*wand));
   if (wand == (PixelWand *) NULL)
-    ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
+        message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
   (void) ResetMagickMemory(wand,0,sizeof(*wand));
   wand->id=AcquireWandId();
   (void) FormatMagickString(wand->name,MaxTextExtent,"%s-%lu",PixelWandId,
@@ -361,8 +368,15 @@ WandExport PixelWand **NewPixelWands(const unsigned long number_wands)
   wands=(struct _PixelWand **)
     AcquireMagickMemory((size_t) number_wands*sizeof(*wands));
   if (wands == (PixelWand **) NULL)
-    ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
+        message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
   for (i=0; i < (long) number_wands; i++)
     wands[i]=NewPixelWand();
   return(wands);
@@ -439,8 +453,15 @@ WandExport char *PixelGetException(const PixelWand *wand,
   *severity=wand->exception.severity;
   description=(char *) AcquireMagickMemory(2*MaxTextExtent);
   if (description == (char *) NULL)
-    ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
-      strerror(errno));
+    {
+      char
+        *message;
+
+      message=GetExceptionMessage(errno);
+      ThrowWandFatalException(ResourceLimitFatalError,"MemoryAllocationFailed",
+        message);
+      message=(char *) RelinquishMagickMemory(message);
+    }
   *description='\0';
   if (wand->exception.reason != (char *) NULL)
     (void) CopyMagickString(description,GetLocaleExceptionMessage(
@@ -947,8 +968,9 @@ WandExport void PixelGetHSL(const PixelWand *wand,double *hue,
   assert(wand->signature == WandSignature);
   if (wand->debug != MagickFalse)
     (void) LogMagickEvent(WandEvent,GetMagickModule(),"%s",wand->name);
-  TransformHSL(wand->pixel.red,wand->pixel.green,wand->pixel.blue,
-    hue,saturation,luminosity);
+  TransformHSL(RoundToQuantum(wand->pixel.red),RoundToQuantum(
+    wand->pixel.green),RoundToQuantum(wand->pixel.blue),hue,saturation,
+    luminosity);
 }
 
 /*
